@@ -39,6 +39,7 @@
 #endif
 
 #include "checkvm_version.h"
+#include "vm_version.h"
 #include "embed_version.h"
 VM_EMBED_VERSION(CHECKVM_VERSION_STRING);
 
@@ -115,18 +116,10 @@ main(int argc,
          * obtained from getVersion().
          */
          switch (version[1]) {
-         case VMX_TYPE_EXPRESS:
-            printf("Express\n");
-            break;
-
          case VMX_TYPE_SCALABLE_SERVER:
             printf("ESX Server\n");
             break;
 
-         case VMX_TYPE_WGS:
-            printf("VMware Server\n");
-            break;
-
          case VMX_TYPE_WORKSTATION:
             printf("Workstation\n");
             break;
