@@ -24,6 +24,7 @@
  *
  */
 
+#define G_LOG_DOMAIN "hgfsClient"
 #include "vmware.h"
 #include "guestApp.h"
 #include "vmcheck.h"
@@ -34,9 +35,11 @@
 #include "hgfsProto.h"
 #include "conf.h"
 #include "str.h"
+#include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
 
 #include "hgfsclient_version.h"
+#include "vm_version.h"
 #include "embed_version.h"
 VM_EMBED_VERSION(HGFSCLIENT_VERSION_STRING);
 
