@@ -22,18 +22,28 @@
 #include <unistd.h> // for access, crypt, etc.
 
 #include "vmware.h"
-#include "vm_version.h"
+#include "vm_product.h"
 #include "codeset.h"
 #include "posix.h"
 #include "auth.h"
 #include "str.h"
+#include "log.h"
 
 #ifdef USE_PAM
-#  include "file.h"
-#  include "config.h"
-#  include "localconfig.h"
-#  include <security/pam_appl.h>
-#  include <dlfcn.h>
+#   include "file.h"
+#   include "config.h"
+#   include "localconfig.h"
+#   if defined __APPLE__
+#      include <AvailabilityMacros.h>
+#      if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_5
+#         include <pam/pam_appl.h>
+#      else
+#         include <security/pam_appl.h>
+#      endif
+#   else
+#      include <security/pam_appl.h>
+#   endif
+#   include <dlfcn.h>
 #endif
 
 #if defined(HAVE_CONFIG_H) || defined(sun)
@@ -43,9 +53,19 @@
 #define LOGLEVEL_MODULE auth
 #include "loglevel_user.h"
 
+typedef struct {
+   struct passwd  pwd;      /* must be first member */
+   size_t         bufSize;
+   uint8          buf[];
+} AuthTokenInternal;
+
 #ifdef USE_PAM
 #if defined(sun)
 #define CURRENT_PAM_LIBRARY	"libpam.so.1"
+#elif defined(__FreeBSD__)
+#define CURRENT_PAM_LIBRARY	"libpam.so"
+#elif defined(__APPLE__)
+#define CURRENT_PAM_LIBRARY	"libpam.dylib"
 #else
 #define CURRENT_PAM_LIBRARY	"libpam.so.0"
 #endif
@@ -55,6 +75,7 @@ static typeof(&pam_end) dlpam_end;
 static typeof(&pam_authenticate) dlpam_authenticate;
 static typeof(&pam_setcred) dlpam_setcred;
 static typeof(&pam_acct_mgmt) dlpam_acct_mgmt;
+static typeof(&pam_strerror) dlpam_strerror;
 #if 0  /* These three functions are not used yet */
 static typeof(&pam_open_session) dlpam_open_session;
 static typeof(&pam_close_session) dlpam_close_session;
@@ -71,6 +92,7 @@ static struct {
    IMPORT_SYMBOL(pam_authenticate),
    IMPORT_SYMBOL(pam_setcred),
    IMPORT_SYMBOL(pam_acct_mgmt),
+   IMPORT_SYMBOL(pam_strerror),
 #undef IMPORT_SYMBOL
 };
 
@@ -147,7 +169,7 @@ AuthLoadPAM(void)
       if (!symbol) {
          Log("PAM library does not contain required function: %s\n",
              dlerror());
-
+         dlclose(pam_library);
          return FALSE;
       }
 
@@ -226,6 +248,120 @@ static struct pam_conv PAM_conversation = {
 #endif /* USE_PAM */
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * AuthAllocateToken --
+ *
+ *      Allocates an AuthTokenInternal structure, plus helper buffer
+ *      large enough for the Posix_Get*_r calls.
+ *
+ * Side effects:
+ *      None.
+ *
+ * Results:
+ *      An AuthTokenInternal pointer. Free with Auth_CloseToken.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static AuthTokenInternal *
+AuthAllocateToken(void)
+{
+   AuthTokenInternal *ati;
+   size_t bufSize;
+
+   /*
+    * We need to get the maximum size buffer needed by getpwuid_r from
+    * sysconf. Multiply by 4 to compensate for the conversion to UTF-8
+    * by the Posix_Get*_r() wrappers.
+    */
+
+   bufSize = (size_t) sysconf(_SC_GETPW_R_SIZE_MAX) * 4;
+
+   ati = Util_SafeMalloc(sizeof *ati + bufSize);
+   ati->bufSize = bufSize;
+
+   return ati;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Auth_GetPwnam --
+ *
+ *      Wrapper aroung Posix_Getpwnam_r.
+ *
+ * Side effects:
+ *      None.
+ *
+ * Results:
+ *      An AuthToken. Free with Auth_CloseToken.
+ *
+ *----------------------------------------------------------------------
+ */
+
+AuthToken
+Auth_GetPwnam(const char *user)  // IN
+{
+   AuthTokenInternal *ati;
+   int res;
+   struct passwd *ppwd;
+
+   ASSERT(user);
+
+   ati = AuthAllocateToken();
+   res = Posix_Getpwnam_r(user, &ati->pwd, ati->buf, ati->bufSize, &ppwd);
+
+   if ((0 != res) || (ppwd == NULL)) {
+      Auth_CloseToken((AuthToken) ati);
+      return NULL;
+   }
+
+   ASSERT(ppwd == &ati->pwd);
+
+   return (AuthToken) ati;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Auth_AuthenticateSelf --
+ *
+ *      Authenticate as the current user.
+ *
+ * Side effects:
+ *      None.
+ *
+ * Results:
+ *      An AuthToken. Free with Auth_CloseToken.
+ *
+ *----------------------------------------------------------------------
+ */
+
+AuthToken
+Auth_AuthenticateSelf(void)  // IN
+{
+   AuthTokenInternal *ati;
+   int res;
+   struct passwd *ppwd;
+
+   ati = AuthAllocateToken();
+   res = Posix_Getpwuid_r(getuid(), &ati->pwd, ati->buf, ati->bufSize, &ppwd);
+
+   if ((0 != res) || (ppwd == NULL)) {
+      Auth_CloseToken((AuthToken) ati);
+      return NULL;
+   }
+
+   ASSERT(ppwd == &ati->pwd);
+
+   return (AuthToken) ati;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -248,108 +384,108 @@ AuthToken
 Auth_AuthenticateUser(const char *user,  // IN:
                       const char *pass)  // IN:
 {
-   struct passwd *pwd;
-
 #ifdef USE_PAM
    pam_handle_t *pamh;
    int pam_error;
 #endif
 
+   Bool success = FALSE;
+   AuthTokenInternal *ati = NULL;
+
    if (!CodeSet_Validate(user, strlen(user), "UTF-8")) {
       Log("User not in UTF-8\n");
-      return NULL;
+      goto exit;
    }
    if (!CodeSet_Validate(pass, strlen(pass), "UTF-8")) {
-      Log("Password not in UTF-8\n");                                                     
-      return NULL;
+      Log("Password not in UTF-8\n");
+      goto exit;
    }
 
 #ifdef USE_PAM
-#ifdef ACCEPT_XXX_PASS
-   if (strcmp("XXX", pass) != 0) {
-#endif
-      if (!AuthLoadPAM()) {
-         return NULL;
-      }
+   if (!AuthLoadPAM()) {
+      goto exit;
+   }
 
-      /*
-       * XXX PAM can blow away our syslog level settings so we need
-       * to call Log_InitEx() again before doing any more Log()s
-       */
+   /*
+    * XXX PAM can blow away our syslog level settings so we need
+    * to call Log_InitEx() again before doing any more Log()s
+    */
 
 #define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
+                  Log_Error("%s:%d: PAM failure - %s (%d)\n", \
+                            __FUNCTION__, __LINE__, \
+                            dlpam_strerror(pamh, pam_error), pam_error); \
                   dlpam_end(pamh, pam_error); \
-                  return NULL; \
+                  goto exit; \
                  }
-      PAM_username = user;
-      PAM_password = pass;
+   PAM_username = user;
+   PAM_password = pass;
 
 #if defined(VMX86_TOOLS)
-      pam_error = dlpam_start("vmtoolsd", PAM_username, &PAM_conversation,
-                              &pamh);
+   pam_error = dlpam_start("vmtoolsd", PAM_username, &PAM_conversation,
+                           &pamh);
 #else
-      pam_error = dlpam_start("vmware-authd", PAM_username, &PAM_conversation,
-                              &pamh);
+   pam_error = dlpam_start("vmware-authd", PAM_username, &PAM_conversation,
+                           &pamh);
 #endif
-
-      PAM_BAIL;
-      pam_error = dlpam_authenticate(pamh, 0);
-      PAM_BAIL;
-      pam_error = dlpam_acct_mgmt(pamh, 0);
-      PAM_BAIL;
-      pam_error = dlpam_setcred(pamh, PAM_ESTABLISH_CRED);
-      PAM_BAIL;
-      dlpam_end(pamh, PAM_SUCCESS);
-
-#if ACCEPT_XXX_PASS
+   if (pam_error != PAM_SUCCESS) {
+      Log("Failed to start PAM (error = %d).\n", pam_error);
+      goto exit;
    }
-#endif
-   /* If this point is reached, the user has been authenticated. */
-   setpwent();
-   pwd = Posix_Getpwnam(user);
-   endpwent();
 
+   pam_error = dlpam_authenticate(pamh, 0);
+   PAM_BAIL;
+   pam_error = dlpam_acct_mgmt(pamh, 0);
+   PAM_BAIL;
+   pam_error = dlpam_setcred(pamh, PAM_ESTABLISH_CRED);
+   PAM_BAIL;
+   dlpam_end(pamh, PAM_SUCCESS);
+
+   /* If this point is reached, the user has been authenticated. */
+   ati = (AuthTokenInternal *) Auth_GetPwnam(user);
 #else /* !USE_PAM */
 
    /* All of the following issues are dealt with in the PAM configuration
       file, so put all authentication/priviledge checks before the
       corresponding #endif below. */
-   
-   setpwent(); //XXX can kill?
-   pwd = Posix_Getpwnam(user);
-   endpwent(); //XXX can kill?
 
-   if (!pwd) {
-      // No such user
-      return NULL;
+   ati = (AuthTokenInternal *) Auth_GetPwnam(user);
+
+   if (ati == NULL) {
+      goto exit;
    }
 
-   if (*pwd->pw_passwd != '\0') {
-      char *namep = (char *) crypt(pass, pwd->pw_passwd);
+   if (*ati->pwd.pw_passwd != '\0') {
+      char *namep = (char *) crypt(pass, ati->pwd.pw_passwd);
 
-      if (strcmp(namep, pwd->pw_passwd)
-#ifdef ACCEPT_XXX_PASS
-          && strcmp("XXX", pass) != 0
-#endif
-          ) {
+      if (strcmp(namep, ati->pwd.pw_passwd) != 0) {
          // Incorrect password
-         return NULL;
+         goto exit;
       }
 
       // Clear out crypt()'s internal state, too.
-      crypt("glurp", pwd->pw_passwd);
+      crypt("glurp", ati->pwd.pw_passwd);
    }
 #endif /* !USE_PAM */
-   
-   return pwd;
+
+   success = TRUE;
+
+  exit:
+   if (success) {
+      return (AuthToken) ati;
+   } else {
+      Auth_CloseToken((AuthToken) ati);
+      return NULL;
+   }
 }
 
+
 /*
  *----------------------------------------------------------------------
  *
  * Auth_CloseToken --
  *
- *      Do nothing.
+ *      Free the token allocated in Auth_AuthenticateUser.
  *
  * Side effects:
  *      None
@@ -361,7 +497,7 @@ Auth_AuthenticateUser(const char *user,  // IN:
  */
 
 void
-Auth_CloseToken(AuthToken token)  // IN:
+Auth_CloseToken(AuthToken token)  // IN (OPT):
 {
+   free((void *) token);
 }
-
