@@ -42,7 +42,7 @@
  * think once more.
  */
 
-#define ADFS_SUPER_MAGIC      0xADF5
+#define ADFS_SUPER_MAGIC      0xadf5
 #define AFFS_SUPER_MAGIC      0xADFF
 #define EXT_SUPER_MAGIC       0x137D
 #define EXT2_OLD_SUPER_MAGIC  0xEF51
@@ -53,10 +53,10 @@
 #define NFS_SUPER_MAGIC       0x6969
 #define SMB_SUPER_MAGIC       0x517B
 #define ISOFS_SUPER_MAGIC     0x9660
-#define JFFS2_SUPER_MAGIC     0x72B6
-#define PROC_SUPER_MAGIC      0x9FA0
-#define OPENPROM_SUPER_MAGIC  0x9FA1
-#define USBDEVICE_SUPER_MAGIC 0x9FA2
+#define JFFS2_SUPER_MAGIC     0x72b6
+#define PROC_SUPER_MAGIC      0x9fa0
+#define OPENPROM_SUPER_MAGIC  0x9fa1
+#define USBDEVICE_SUPER_MAGIC 0x9fa2
 #define AUTOFS_SUPER_MAGIC    0x0187
 
 #if !defined(MSDOS_SUPER_MAGIC)
@@ -71,7 +71,7 @@
 #define XFS_SUPER_MAGIC       0x58465342
 #define VMFS_SUPER_MAGIC      0x2fABF15E
 #define TMPFS_SUPER_MAGIC     0x01021994
-#define JFS_SUPER_MAGIC       0x3153464A
+#define JFS_SUPER_MAGIC       0x3153464a
 #define AFS_SUPER_MAGIC       0x5346414F
 #define CIFS_SUPER_MAGIC      0xFF534D42
 
@@ -104,6 +104,8 @@ typedef struct FileData {
 
 #define FILE_MAX_WAIT_TIME_MS 2000  // maximum wait time in milliseconds
 
+void FileIOResolveLockBits(int *access);
+
 #if defined(_WIN32)
 int FileMapErrorToErrno(const char *functionName,
                         Err_Number status);
@@ -116,10 +118,6 @@ int FileAttributesRetry(ConstUnicode pathName,
                         uint32 msecMaxWaitTime,
                         FileData *fileData);
 
-int FileRenameRetry(ConstUnicode fromPath,
-                    ConstUnicode toPath,
-                    uint32 msecMaxWaitTime);
-
 int FileDeletionRetry(ConstUnicode pathName,
                       Bool handleLink,
                       uint32 msecMaxWaitTime);
@@ -136,7 +134,6 @@ int FileListDirectoryRetry(ConstUnicode pathName,
                            Unicode **ids);
 
 #define FileAttributes(a, b)       FileAttributesRetry((a), 0, (b))
-#define FileRename(a, b)           FileRenameRetry((a), (b), 0)
 #define FileDeletion(a, b)         FileDeletionRetry((a), (b), 0)
 #define FileCreateDirectory(a, b)  FileCreateDirectoryRetry((a), (b), 0)
 #define FileRemoveDirectory(a)     FileRemoveDirectoryRetry((a), 0)
@@ -146,7 +143,7 @@ int FileListDirectoryRetry(ConstUnicode pathName,
 #define FileAttributesRobust(a, b) \
                     FileAttributesRetry((a), FILE_MAX_WAIT_TIME_MS, (b))
 #define FileRenameRobust(a, b) \
-                    FileRenameRetry((a), (b), FILE_MAX_WAIT_TIME_MS)
+                    File_RenameRetry((a), (b), FILE_MAX_WAIT_TIME_MS)
 #define FileDeletionRobust(a, b) \
                     FileDeletionRetry((a), (b), FILE_MAX_WAIT_TIME_MS)
 #define FileCreateDirectoryRobust(a, b) \
@@ -166,9 +163,6 @@ char *FilePosixGetBlockDevice(char const *path);
 int FileAttributes(ConstUnicode pathName,
                    FileData *fileData);
 
-int FileRename(ConstUnicode fromPath,
-               ConstUnicode toPath);
-
 int FileDeletion(ConstUnicode pathName,
                  Bool handleLink);
 
@@ -179,7 +173,7 @@ int FileRemoveDirectory(ConstUnicode pathName);
 
 #define FileListDirectoryRobust(a, b)    File_ListDirectory((a), (b))
 #define FileAttributesRobust(a, b)       FileAttributes((a), (b))
-#define FileRenameRobust(a, b)           FileRename((a), (b))
+#define FileRenameRobust(a, b)           File_Rename((a), (b))
 #define FileDeletionRobust(a, b)         FileDeletion((a), (b))
 #define FileCreateDirectoryRobust(a, b)  FileCreateDirectory((a), (b))
 #define FileRemoveDirectoryRobust(a)     FileRemoveDirectory((a))
@@ -201,6 +195,7 @@ typedef struct lock_values
    char         *locationChecksum;
    Unicode       memberName;
    unsigned int  lamportNumber;
+   Bool          exclusivity;
    uint32        waitTime;
    uint32        msecMaxWaitTime;
    ActiveLock   *lockList;
@@ -244,6 +239,9 @@ Bool FileLockValidExecutionID(const char *executionID);
 
 Bool FileLockValidName(ConstUnicode fileName);
 
+void FileLockAppendMessage(MsgList **msgs,
+                           int err);
+
 Bool FileIsWritableDir(ConstUnicode dirName);
 
 UnicodeIndex FileFirstSlashIndex(ConstUnicode pathName,
