@@ -52,7 +52,6 @@
 #include "util.h"
 #include "str.h"
 #include "err.h"
-#include "vm_version.h"
 #include "localconfig.h"
 #include "hostinfo.h"
 #include "su.h"
@@ -595,7 +594,7 @@ FileLockAppendMessage(MsgList **msgs,  // IN/OPT:
                       int err)         // IN: errno
 {
    MsgList_Append(msgs, MSGID(fileLock.posix)
-                  "A file locking error (%d) has occurred: %s",
+                  "A file locking error (%d) has occurred: %s.",
                   err, Err_Errno2String(err));
 }
 
