@@ -26,7 +26,6 @@
  */
 
 #include "vmware.h"
-#include "vm_version.h"
 #include "util.h"
 #include "str.h"
 #include "base64.h"
@@ -452,13 +451,13 @@ static const VixCommandInfo vixCommandInfoTable[] = {
    VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_HOT_CHANGE_MONITOR_TYPE,
                            VIX_COMMAND_CATEGORY_PRIVILEGED),
 
-   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_ADD_AUTH_PRINCIPAL,
+   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_ADD_AUTH_ALIAS,
                            VIX_COMMAND_CATEGORY_ALWAYS_ALLOWED),
-   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_REMOVE_AUTH_PRINCIPAL,
+   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_REMOVE_AUTH_ALIAS,
                            VIX_COMMAND_CATEGORY_ALWAYS_ALLOWED),
-   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_LIST_AUTH_PROVIDER_PRINCIPALS,
+   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_LIST_AUTH_PROVIDER_ALIASES,
                            VIX_COMMAND_CATEGORY_ALWAYS_ALLOWED),
-   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_LIST_AUTH_MAPPED_PRINCIPALS,
+   VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_LIST_AUTH_MAPPED_ALIASES,
                            VIX_COMMAND_CATEGORY_ALWAYS_ALLOWED),
 
    VIX_DEFINE_COMMAND_INFO(VIX_COMMAND_CREATE_REGISTRY_KEY,
