@@ -22,50 +22,13 @@
 
 #include "vm_basic_types.h"
 
-/*
- * Definitions of wrapped internal primitives.
- *
- * We wrap open file handles and directory entries so that cross-platform HGFS
- * server code can use them without platform-specific pre-processing.
- *
- * On Linux, we use dirent64 from the kernel headers so as to alleviate any
- * confusion between what the kernel will give us from the getdents64()
- * syscall and what userspace will expect. Note that to avoid depending on
- * kernel headers directly, I've copied the dirent struct, replacing certain
- * kernel basic types with our own.
- *
- * On Windows, we define our own DirectoryEntry struct with d_reclen and
- * d_name, as those are the only two fields we're interested in. It's not
- * essential that it match the dirents from another platform, because we
- * control how they get created and populated, and they never pass down a wire.
- *
- * Otherwise, we use the native dirent struct provided by the platform's libc,
- * as nobody else seems to suffer from the 32-bit vs. 64-bit ino_t and off_t
- * insanity that plagues Linux.
- */
+struct DirectoryEntry;
+
 #ifndef _WIN32
-#  ifdef linux
-   typedef struct DirectoryEntry {
-      uint64 d_ino;
-      uint64 d_off;
-      uint16 d_reclen;
-      uint8  d_type;
-      char   d_name[256];
-   } DirectoryEntry;
-#  else
-#    include <dirent.h>
-     typedef struct dirent DirectoryEntry;
-#  endif
    typedef int fileDesc;
 #else
 #  include <windows.h>
    typedef HANDLE fileDesc;
-   typedef struct DirectoryEntry {
-      unsigned short d_reclen;      /* The total length of this record. */
-      char d_name[PATH_MAX * 4];    /* 4 bytes is the maximum size of a utf8 */
-                                    /* representation of utf-16 encoded */
-                                    /* unicode character in the BMP */
-   } DirectoryEntry;
 #endif
 
 #include "dbllnklst.h"
@@ -88,9 +51,6 @@
 #define HGFS_OPLOCKS
 #endif
 
-/* Value of config option to require using host timestamps */
-extern Bool alwaysUseHostTime;
-
 /* Identifier for a local file */
 typedef struct HgfsLocalId {
    uint64 volumeId;
@@ -118,6 +78,9 @@ typedef enum {
                                        "root/drive" or contents of "root" */
 } DirectorySearchType;
 
+#define HGFS_SEARCH_LAST_ENTRY_INDEX         ((uint32)~((uint32)0))
+
+
 /* Two possible volume info type */
 typedef enum {
    VOLUME_INFO_TYPE_MIN,
@@ -199,7 +162,7 @@ typedef struct HgfsFileNode {
    uint32 shareAccess;
 
    /* The server lock that the node currently has. */
-   HgfsServerLock serverLock;
+   HgfsLockType serverLock;
 
    /* File node state on lists */
    FileNodeState state;
@@ -240,6 +203,9 @@ typedef struct HgfsSearch {
    /* Links to place the object on various lists */
    DblLnkLst_Links links;
 
+   /* Flags to track state and information: see below. */
+   uint32 flags;
+
    /* HGFS handle uniquely identifying this search. */
    HgfsHandle handle;
 
@@ -256,7 +222,7 @@ typedef struct HgfsSearch {
    size_t utf8ShareNameLen;
 
    /* Directory entries for this search */
-   DirectoryEntry **dents;
+   struct DirectoryEntry **dents;
 
    /* Number of dents */
    uint32 numDents;
@@ -272,6 +238,11 @@ typedef struct HgfsSearch {
    HgfsShareInfo shareInfo;
 } HgfsSearch;
 
+/* HgfsSearch flags. */
+
+/* TRUE if opened in append mode */
+#define HGFS_SEARCH_FLAG_READ_ALL_ENTRIES      (1 << 0)
+
 /* HgfsSessionInfo flags. */
 typedef enum {
    HGFS_SESSION_TYPE_REGULAR,      /* Dynamic session, created by the HgfsTransport. */
@@ -435,8 +406,8 @@ typedef struct HgfsFileOpenInfo {
    uint64 allocationSize;            /* How much space to pre-allocate during creation */
    uint32 desiredAccess;             /* Extended support for windows access modes */
    uint32 shareAccess;               /* Windows only, share access modes */
-   HgfsServerLock desiredLock;       /* The type of lock desired by the client */
-   HgfsServerLock acquiredLock;      /* The type of lock acquired by the server */
+   HgfsLockType desiredLock;         /* The type of lock desired by the client */
+   HgfsLockType acquiredLock;        /* The type of lock acquired by the server */
    uint32 cpNameSize;
    char *cpName;
    char *utf8Name;
@@ -515,7 +486,7 @@ typedef struct HgfsCreateSessionInfo {
 typedef struct {
    fileDesc fileDesc;
    int32 event;
-   HgfsServerLock serverLock;
+   HgfsLockType serverLock;
 } ServerLockData;
 
 typedef struct HgfsInputParam {
@@ -553,11 +524,22 @@ Bool
 HgfsRemoveSearch(HgfsHandle searchHandle,
                  HgfsSessionInfo *session);
 
-void
-HgfsServerDumpDents(HgfsHandle searchHandle,   // IN: Handle to dump dents from
-                    HgfsSessionInfo *session); // IN: Session info
+#ifdef VMX86_LOG
+#define HGFS_SERVER_DIR_DUMP_DENTS(_searchHandle, _session) do {    \
+      if (DOLOG(4)) {                                               \
+         HgfsServerDirDumpDents(_searchHandle, _session);           \
+      }                                                             \
+   } while (0)
 
-DirectoryEntry *
+void
+HgfsServerDirDumpDents(HgfsHandle searchHandle,   // IN: Handle to dump dents from
+                       HgfsSessionInfo *session); // IN: Session info
+#else
+#define HGFS_SERVER_DIR_DUMP_DENTS(_searchHandle, _session) do {} while (0)
+#endif
+
+
+struct DirectoryEntry *
 HgfsGetSearchResult(HgfsHandle handle,        // IN: Handle to search
                     HgfsSessionInfo *session, // IN: Session info
                     uint32 offset,            // IN: Offset to retrieve at
@@ -590,11 +572,11 @@ HgfsServerIsSharedFolderOnly(char const *in,  // IN:  CP filename to check
                              size_t inSize);  // IN:  Size of name in
 
 HgfsInternalStatus
-HgfsServerScandir(char const *baseDir,      // IN: Directory to search in
-                  size_t baseDirLen,        // IN: Length of directory
-                  Bool followSymlinks,      // IN: followSymlinks config option
-                  DirectoryEntry ***dents,  // OUT: Array of DirectoryEntrys
-                  int *numDents);           // OUT: Number of DirectoryEntrys
+HgfsServerGetDirEntry(HgfsHandle handle,                // IN: Handle to search
+                      HgfsSessionInfo *session,         // IN: Session info
+                      uint32 index,                     // IN: index to retrieve at
+                      Bool remove,                      // IN: If true, removes the result
+                      struct DirectoryEntry **dirEntry);// OUT: directory entry
 
 HgfsInternalStatus
 HgfsServerSearchRealDir(char const *baseDir,      // IN: Directory to search
@@ -612,6 +594,13 @@ HgfsServerSearchVirtualDir(HgfsGetNameFunc *getName,     // IN: Name enumerator
                            HgfsSessionInfo *session,     // IN: Session info
                            HgfsHandle *handle);          // OUT: Search handle
 
+HgfsInternalStatus
+HgfsServerRestartSearchVirtualDir(HgfsGetNameFunc *getName,     // IN: Name enumerator
+                                  HgfsInitFunc *initName,       // IN: Init function
+                                  HgfsCleanupFunc *cleanupName, // IN: Cleanup function
+                                  HgfsSessionInfo *session,     // IN: Session info
+                                  HgfsHandle searchHandle);     // IN: search to restart
+
 /* Allocate/Add sessions helper functions. */
 
 Bool HgfsServerAllocateSession(HgfsTransportSessionInfo *transportSession,
@@ -883,7 +872,7 @@ HgfsPackCreateSessionReply(HgfsPacket *packet,        // IN/OUT: Hgfs Packet
                            size_t *payloadSize,       // OUT: size of packet
                            HgfsSessionInfo *session); // IN: Session Info
 Bool
-HgfsPackDestorySessionReply(HgfsPacket *packet,        // IN/OUT: Hgfs Packet
+HgfsPackDestroySessionReply(HgfsPacket *packet,        // IN/OUT: Hgfs Packet
                             char const *packetHeader,  // IN: packet header
                             size_t *payloadSize,       // OUT: size of packet
                             HgfsSessionInfo *session); // IN: Session Info
@@ -989,7 +978,7 @@ HgfsHandle2LocalId(HgfsHandle handle,        // IN: Hgfs file handle
 Bool
 HgfsHandle2ServerLock(HgfsHandle handle,        // IN: Hgfs file handle
                       HgfsSessionInfo *session, // IN: session info
-                      HgfsServerLock *lock);    // OUT: Server lock
+                      HgfsLockType *lock);      // OUT: Server lock
 
 Bool
 HgfsUpdateNodeFileDesc(HgfsHandle handle,        // IN: Hgfs file handle
@@ -1000,7 +989,7 @@ HgfsUpdateNodeFileDesc(HgfsHandle handle,        // IN: Hgfs file handle
 Bool
 HgfsUpdateNodeServerLock(fileDesc fd,                // IN: OS handle
                          HgfsSessionInfo *session,   // IN: session info
-                         HgfsServerLock serverLock); // IN: new oplock
+                         HgfsLockType serverLock);   // IN: new oplock
 
 Bool
 HgfsUpdateNodeAppendFlag(HgfsHandle handle,        // IN: Hgfs file handle
@@ -1010,7 +999,7 @@ HgfsUpdateNodeAppendFlag(HgfsHandle handle,        // IN: Hgfs file handle
 Bool
 HgfsFileHasServerLock(const char *utf8Name,             // IN: Name in UTF8
                       HgfsSessionInfo *session,         // IN: Session info
-                      HgfsServerLock *serverLock,       // OUT: Existing oplock
+                      HgfsLockType *serverLock,         // OUT: Existing oplock
                       fileDesc   *fileDesc);            // OUT: Existing fd
 Bool
 HgfsGetNodeCopy(HgfsHandle handle,        // IN: Hgfs file handle
@@ -1033,10 +1022,6 @@ HgfsGetSearchCopy(HgfsHandle handle,        // IN: Hgfs search handle
                   HgfsSessionInfo *session, // IN: Session info
                   HgfsSearch *copy);        // IN/OUT: Copy of the search
 
-HgfsInternalStatus
-HgfsCloseFile(fileDesc fileDesc,            // IN: OS handle of the file
-              void *fileCtx);               // IN: file context
-
 Bool
 HgfsServerGetOpenMode(HgfsFileOpenInfo *openInfo, // IN:  Open info to examine
                       uint32 *modeOut);           // OUT: Local mode
@@ -1044,34 +1029,11 @@ HgfsServerGetOpenMode(HgfsFileOpenInfo *openInfo, // IN:  Open info to examine
 Bool
 HgfsAcquireServerLock(fileDesc fileDesc,            // IN: OS handle
                       HgfsSessionInfo *session,     // IN: Session info
-                      HgfsServerLock *serverLock);  // IN/OUT: Oplock asked for/granted
-
-Bool
-HgfsServerPlatformInit(void);
-
-void
-HgfsServerPlatformDestroy(void);
-
-HgfsNameStatus
-HgfsServerHasSymlink(const char *fileName,      // IN: fileName to be checked
-                     size_t fileNameLength,     // IN
-                     const char *sharePath,     // IN: share path in question
-                     size_t sharePathLen);      // IN
-HgfsNameStatus
-HgfsServerConvertCase(const char *sharePath,             // IN: share path in question
-                      size_t sharePathLength,            // IN
-                      char *fileName,                    // IN: filename to be looked up
-                      size_t fileNameLength,             // IN
-                      uint32 caseFlags,                  // IN: case-sensitivity flags
-                      char **convertedFileName,          // OUT: case-converted filename
-                      size_t *convertedFileNameLength);  // OUT
-
-Bool
-HgfsServerCaseConversionRequired(void);
+                      HgfsLockType *serverLock);    // IN/OUT: Oplock asked for/granted
 
 char*
-HgfsBuildRelativePath(const char* source,    // IN: source file name
-                      const char* target);   // IN: target file name
+HgfsServerGetTargetRelativePath(const char* source,    // IN: source file name
+                                const char* target);   // IN: target file name
 
 /* All oplock-specific functionality is defined here. */
 #ifdef HGFS_OPLOCKS
@@ -1080,7 +1042,7 @@ HgfsServerOplockBreak(ServerLockData *data); // IN: server lock info
 
 void
 HgfsAckOplockBreak(ServerLockData *lockData,  // IN: server lock info
-                   HgfsServerLock replyLock); // IN: client has this lock
+                   HgfsLockType replyLock);   // IN: client has this lock
 
 #endif
 
@@ -1109,8 +1071,32 @@ HgfsPacketSend(HgfsPacket *packet,            // IN/OUT: Hgfs Packet
 Bool
 HgfsServerCheckOpenFlagsForShare(HgfsFileOpenInfo *openInfo, // IN: Hgfs file handle
                                  HgfsOpenFlags *flags);      // IN/OUT: open mode
+
+/* Platform specific exports. */
+Bool
+HgfsPlatformInit(void);
+void
+HgfsPlatformDestroy(void);
+HgfsInternalStatus
+HgfsPlatformCloseFile(fileDesc fileDesc,            // IN: OS handle of the file
+                      void *fileCtx);               // IN: file context
+Bool
+HgfsPlatformDoFilenameLookup(void);
+HgfsNameStatus
+HgfsPlatformFilenameLookup(const char *sharePath,             // IN: share path in question
+                           size_t sharePathLength,            // IN
+                           char *fileName,                    // IN: filename to be looked up
+                           size_t fileNameLength,             // IN
+                           uint32 caseFlags,                  // IN: case-sensitivity flags
+                           char **convertedFileName,          // OUT: case-converted filename
+                           size_t *convertedFileNameLength);  // OUT
 HgfsInternalStatus
 HgfsPlatformConvertFromNameStatus(HgfsNameStatus status);  // IN: name status
+HgfsNameStatus
+HgfsPlatformPathHasSymlink(const char *fileName,      // IN: fileName to be checked
+                           size_t fileNameLength,     // IN
+                           const char *sharePath,     // IN: share path in question
+                           size_t sharePathLen);      // IN
 HgfsInternalStatus
 HgfsPlatformSymlinkCreate(char *localSymlinkName,   // IN: symbolic link file name
                           char *localTargetName);   // IN: symlink target name
@@ -1121,6 +1107,33 @@ HgfsPlatformGetattrFromName(char *fileName,                 // IN: file name
                             HgfsFileAttrInfo *attr,         // OUT: file attributes
                             char **targetName);             // OUT: Symlink target
 HgfsInternalStatus
+HgfsPlatformGetDirEntry(HgfsSearch *search,                // IN: search
+                        HgfsSessionInfo *session,          // IN: Session info
+                        uint32 offset,                     // IN: Offset to retrieve at
+                        Bool remove,                       // IN: If true, removes the result
+                        struct DirectoryEntry **dirEntry); // OUT: dirent
+HgfsInternalStatus
+HgfsPlatformSetDirEntry(HgfsSearch *search,              // IN: search
+                        HgfsShareOptions configOptions,  // IN: share configuration settings
+                        HgfsSessionInfo *session,        // IN: session info
+                        struct DirectoryEntry *dirEntry, // IN: the indexed dirent
+                        Bool getAttr,                    // IN: get the entry attributes
+                        HgfsFileAttrInfo *entryAttr,     // OUT: entry attributes, optional
+                        char **entryName,                // OUT: entry name
+                        uint32 *entryNameLength);        // OUT: entry name length
+HgfsInternalStatus
+HgfsPlatformScandir(char const *baseDir,             // IN: Directory to search in
+                    size_t baseDirLen,               // IN: Length of directory
+                    Bool followSymlinks,             // IN: followSymlinks config option
+                    struct DirectoryEntry ***dents,  // OUT: Array of DirectoryEntrys
+                    int *numDents);                  // OUT: Number of DirectoryEntrys
+HgfsInternalStatus
+HgfsPlatformScanvdir(HgfsGetNameFunc enumNamesGet,     // IN: Function to get name
+                     HgfsInitFunc enumNamesInit,       // IN: Setup function
+                     HgfsCleanupFunc enumNamesExit,    // IN: Cleanup function
+                     struct DirectoryEntry ***dents,   // OUT: Array of DirectoryEntrys
+                     uint32 *numDents);                // OUT: total number of directory entrys
+HgfsInternalStatus
 HgfsPlatformSearchDir(HgfsNameStatus nameStatus,       // IN: name status
                       char *dirName,                   // IN: relative directory name
                       uint32 dirNameLength,            // IN: length of dirName
@@ -1131,9 +1144,14 @@ HgfsPlatformSearchDir(HgfsNameStatus nameStatus,       // IN: name status
                       HgfsSessionInfo *session,        // IN: session info
                       HgfsHandle *handle);             // OUT: search handle
 HgfsInternalStatus
-HgfsAccess(char *fileName,         // IN: local file path
-           char *shareName,        // IN: Name of the share
-           size_t shareNameLen);   // IN: Length of the share name
+HgfsPlatformRestartSearchDir(HgfsHandle handle,               // IN: search handle
+                             HgfsSessionInfo *session,        // IN: session info
+                             DirectorySearchType searchType); // IN: Kind of search
+#ifdef VMX86_LOG
+void
+HgfsPlatformDirDumpDents(HgfsSearch *search);         // IN: search
+#endif
+
 HgfsInternalStatus
 HgfsPlatformReadFile(HgfsHandle file,             // IN: Hgfs file handle
                      HgfsSessionInfo *session,    // IN: session info
@@ -1156,6 +1174,12 @@ HgfsPlatformWriteWin32Stream(HgfsHandle file,           // IN: packet header
                              Bool doSecurity,           // IN: write ACL
                              uint32  *actualSize,       // OUT: written data size
                              HgfsSessionInfo *session); // IN: session info
+HgfsInternalStatus
+HgfsPlatformVDirStatsFs(HgfsSessionInfo *session,  // IN: session info
+                        HgfsNameStatus nameStatus, // IN:
+                        VolumeInfoType infoType,   // IN:
+                        uint64 *outFreeBytes,      // OUT:
+                        uint64 *outTotalBytes);    // OUT:
 Bool
 HgfsValidatePacket(char const *packetIn, // IN: HGFS packet
                    size_t packetSize,    // IN: HGFS packet size
@@ -1215,12 +1239,14 @@ HgfsInternalStatus
 HgfsPlatformSetattrFromFd(HgfsHandle file,          // IN: file descriptor
                           HgfsSessionInfo *session, // IN: session info
                           HgfsFileAttrInfo *attr,   // IN: attrs to set
-                          HgfsAttrHint hints);      // IN: attr hints
+                          HgfsAttrHint hints,       // IN: attr hints
+                          Bool useHostTime);        // IN: use current host time
 HgfsInternalStatus
 HgfsPlatformSetattrFromName(char *utf8Name,                 // IN: local file path
                             HgfsFileAttrInfo *attr,         // IN: attrs to set
                             HgfsShareOptions configOptions, // IN: share options
-                            HgfsAttrHint hints);            // IN: attr hints
+                            HgfsAttrHint hints,             // IN: attr hints
+                            Bool useHostTime);              // IN: use current host time
 HgfsInternalStatus
 HgfsPlatformValidateOpen(HgfsFileOpenInfo *openInfo, // IN: Open info struct
                          Bool followLinks,           // IN: follow symlinks on the host
