@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -24,7 +24,6 @@
  */
 
 #include <stdlib.h>
-#include "vm_basic_defs.h"
 #include "vm_assert.h"
 #include "vm_atomic.h"
 #include "util.h"
@@ -78,7 +77,7 @@
  * also removed and the HGFS server exit is called and this object is torn down.
  */
 typedef struct HgfsChannelServerData {
-   HgfsServerCallbacks        *serverCBTable; /* HGFS server entry points. */
+   HgfsServerSessionCallbacks *serverCBTable; /* HGFS server entry points. */
    Atomic_uint32              refCount;       /* Server data reference count. */
 } HgfsChannelServerData;
 
@@ -106,11 +105,6 @@ static HgfsChannelData gHgfsChannels[] = {
    { "guest", &gGuestBackdoorOps, 0, NULL, NULL, {0} },
 };
 
-static HgfsServerConfig gHgfsGuestCfgSettings = {
-   (HGFS_CONFIG_SHARE_ALL_HOST_DRIVES_ENABLED | HGFS_CONFIG_VOL_INFO_MIN),
-   HGFS_MAX_CACHED_FILENODES
-};
-
 /* HGFS server info state. Referenced by each separate channel that uses it. */
 static HgfsChannelServerData gHgfsChannelServerInfo = { NULL, {0} };
 
@@ -148,7 +142,7 @@ static uint32
 HgfsChannelGetServer(HgfsChannelServerData *serverInfo)   // IN/OUT: ref count
 {
    ASSERT(NULL != serverInfo);
-   return Atomic_ReadInc32(&serverInfo->refCount);
+   return Atomic_FetchAndInc(&serverInfo->refCount);
 }
 
 
@@ -174,7 +168,7 @@ static void
 HgfsChannelPutServer(HgfsChannelServerData *serverInfo)   // IN/OUT: ref count
 {
    ASSERT(NULL != serverInfo);
-   if (Atomic_ReadDec32(&serverInfo->refCount) == 1) {
+   if (Atomic_FetchAndDec(&serverInfo->refCount) == 1) {
       HgfsChannelTeardownServer(serverInfo);
    }
 }
@@ -197,8 +191,7 @@ HgfsChannelPutServer(HgfsChannelServerData *serverInfo)   // IN/OUT: ref count
  */
 
 static Bool
-HgfsChannelInitServer(HgfsServerMgrCallbacks *mgrCb,       // IN: server manager callbacks
-                      HgfsChannelServerData *serverInfo)   // IN/OUT: ref count
+HgfsChannelInitServer(HgfsChannelServerData *serverInfo)   // IN/OUT: ref count
 {
    Bool result;
 
@@ -207,7 +200,7 @@ HgfsChannelInitServer(HgfsServerMgrCallbacks *mgrCb,       // IN: server manager
    Debug("%s: Initialize Hgfs server.\n", __FUNCTION__);
 
    /* If we have a new connection initialize the server session with default settings. */
-   result = HgfsServer_InitState(&serverInfo->serverCBTable, &gHgfsGuestCfgSettings, mgrCb);
+   result = HgfsServer_InitState(&serverInfo->serverCBTable, NULL, NULL);
    if (!result) {
       Debug("%s: Could not init Hgfs server.\n", __FUNCTION__);
    }
@@ -294,7 +287,7 @@ uint32
 HgfsChannelGetChannel(HgfsChannelData *channel)   // IN/OUT: ref count
 {
    ASSERT(NULL != channel);
-   return Atomic_ReadInc32(&channel->refCount);
+   return Atomic_FetchAndInc(&channel->refCount);
 }
 
 
@@ -320,7 +313,7 @@ static void
 HgfsChannelPutChannel(HgfsChannelData *channel)   // IN/OUT: ref count
 {
    ASSERT(NULL != channel);
-   if (Atomic_ReadDec32(&channel->refCount) == 1) {
+   if (Atomic_FetchAndDec(&channel->refCount) == 1) {
       HgfsChannelTeardownChannel(channel);
    }
 }
@@ -344,7 +337,6 @@ HgfsChannelPutChannel(HgfsChannelData *channel)   // IN/OUT: ref count
 
 static Bool
 HgfsChannelInitChannel(HgfsChannelData *channel,          // IN/OUT: channel object
-                       HgfsServerMgrCallbacks *mgrCb,     // IN: server manager callbacks
                        HgfsChannelServerData *serverInfo) // IN/OUT: server info
 {
    Bool result = TRUE;
@@ -362,7 +354,7 @@ HgfsChannelInitChannel(HgfsChannelData *channel,          // IN/OUT: channel obj
    channel->serverInfo = serverInfo;
    if (0 == serverInfoCount) {
       /* The HGFS server has not been initialized, do it now. */
-      result = HgfsChannelInitServer(mgrCb, channel->serverInfo);
+      result = HgfsChannelInitServer(channel->serverInfo);
       if (!result) {
          Debug("%s: Could not init Hgfs server.\n", __FUNCTION__);
          goto exit;
@@ -433,7 +425,7 @@ HgfsChannelActivateChannel(HgfsChannelData *channel,   // IN/OUT: channel object
    Bool success = FALSE;
    struct HgfsGuestConn *connData = NULL;
 
-   if (channel->ops->init(&channel->serverInfo->serverCBTable->session,
+   if (channel->ops->init(channel->serverInfo->serverCBTable,
                           rpc,
                           rpcCallback,
                           &connData)) {
@@ -581,8 +573,7 @@ HgfsChannelTeardownChannel(HgfsChannelData *channel) // IN/OUT: connection manag
  */
 
 Bool
-HgfsChannelGuest_Init(HgfsServerMgrData *mgrData,    // IN/OUT: server manager data
-                      HgfsServerMgrCallbacks *mgrCb) // IN: server manager callbacks
+HgfsChannelGuest_Init(HgfsServerMgrData *mgrData) // IN/OUT: connection manager object
 {
    Bool success = FALSE;
    HgfsChannelData *channel = &gHgfsChannels[0]; // Shared channel (internal RPC)
@@ -619,7 +610,7 @@ HgfsChannelGuest_Init(HgfsServerMgrData *mgrData,    // IN/OUT: server manager d
    if (0 == channelRefCount) {
 
       /* Initialize channels objects. */
-      if (!HgfsChannelInitChannel(channel, mgrCb, &gHgfsChannelServerInfo)) {
+      if (!HgfsChannelInitChannel(channel, &gHgfsChannelServerInfo)) {
          Debug("%s: Could not init channel.\n", __FUNCTION__);
          goto exit;
       }
