@@ -108,7 +108,10 @@ char *FileMacos_DiskDevToVolumeName(char const *bsdDiskDev);
 
 char *FileMacos_DiskDeviceToUniqueID(char const *bsdPath);
 char *FileMacos_UniqueIDToDiskDevice(char const *identifier);
-
+Bool FileMacOS_MakeSecureLibraryCopies(const char   *inDir,
+                                       const char  **dylibName,
+                                       unsigned      numDylibs,
+                                       char        **outDir);
 #elif defined VMX86_SERVER
 struct FS_PartitionListResult;
 
@@ -158,6 +161,9 @@ Unicode File_PathJoin(ConstUnicode dirName,
                       ConstUnicode baseName);
 
 Bool File_CreateDirectory(ConstUnicode pathName);
+
+Bool File_CreateDirectoryEx(ConstUnicode pathName, int mask);
+
 Bool File_EnsureDirectory(ConstUnicode pathName);
 
 Bool File_DeleteEmptyDirectory(ConstUnicode pathName);
@@ -165,6 +171,10 @@ Bool File_DeleteEmptyDirectory(ConstUnicode pathName);
 Bool File_CreateDirectoryHierarchy(ConstUnicode pathName,
                                    Unicode *topmostCreated);
 
+Bool File_CreateDirectoryHierarchyEx(ConstUnicode pathName,
+                                     int mask,
+                                     Unicode *topmostCreated);
+
 Bool File_DeleteDirectoryTree(ConstUnicode pathName);
 
 int File_ListDirectory(ConstUnicode pathName,
@@ -241,6 +251,9 @@ Bool File_SetFilePermissions(ConstUnicode pathName,
 Bool File_SupportsFileSize(ConstUnicode pathName,
                            uint64 fileSize);
 
+Bool File_GetMaxFileSize(ConstUnicode pathName,
+                         uint64 *maxFileSize);
+
 Bool File_SupportsLargeFiles(ConstUnicode pathName);
 
 char *File_MapPathPrefix(const char *oldPath,
