@@ -147,6 +147,10 @@ typedef enum {
  * (supported on ESX file systems)
  */
 #define FILEIO_OPEN_MULTIWRITER_LOCK     (1 << 14)
+/*
+ * Flag the file to be cached by the vBlob caching layer
+ */
+#define FILEIO_OPEN_USE_AIO_CACHE        (1 << 15)
 /*
  * Valid only for MacOS. It eventually results into O_EXLOCK flag passed to open
  * system call.
@@ -174,6 +178,21 @@ typedef enum {
  * Valid only on Windows. Set FILE_SHARE_DELETE.
  */
 #define FILEIO_OPEN_SHARE_DELETE (1 << 19)
+/*
+ * Strengths of file lock.
+ * Advisory:
+ *   Must use FileIO plus lock flags to get locking.
+ *   Never uses kernel/fs-level lock, so naked open() bypasses locking.
+ * Mandatory:
+ *   Requires kernel/fs-level, so naked open() respects lock.
+ *   Kernel/fs-level locks are available on ESX but not hosted.
+ * Best:
+ *   Adaptively picks between mandatory and advisory.
+ * Almost all cases should use the "best" lock.
+ */
+#define FILEIO_OPEN_LOCK_BEST      FILEIO_OPEN_LOCKED /* historical */
+#define FILEIO_OPEN_LOCK_ADVISORY  (1 << 20)
+#define FILEIO_OPEN_LOCK_MANDATORY (1 << 21)
 
 /*
  * Flag passed to open() to not attempt to get the lun attributes as part of
@@ -290,6 +309,14 @@ FileIOResult FileIO_Write(FileIODescriptor *file,
                           size_t requested,
                           size_t *actual);
 
+Unicode FileIO_AtomicTempPath(ConstUnicode path);
+
+FileIOResult FileIO_AtomicTempFile(FileIODescriptor *fileFD,
+                                   FileIODescriptor *tempFD);
+
+Bool FileIO_AtomicUpdate(FileIODescriptor *newFD,
+                         FileIODescriptor *currFD);
+
 #if !defined(VMX86_TOOLS) || !defined(__FreeBSD__)
 
 FileIOResult FileIO_Readv(FileIODescriptor *fd,
@@ -309,13 +336,15 @@ FileIOResult FileIO_Preadv(FileIODescriptor *fd,   // IN: File descriptor
                            struct iovec *entries,  // IN: Vector to read into
                            int numEntries,         // IN: Number of vector entries
                            uint64 offset,          // IN: Offset to start reading
-                           size_t totalSize);      // IN: totalSize (bytes) in entries
+                           size_t totalSize,       // IN: totalSize (bytes) in entries
+                           size_t *actual);        // OUT: number of bytes read
 
 FileIOResult FileIO_Pwritev(FileIODescriptor *fd,  // IN: File descriptor
                             struct iovec *entries, // IN: Vector to write from
                             int numEntries,        // IN: Number of vector entries
                             uint64 offset,         // IN: Offset to start writing
-                            size_t totalSize);     // IN: Total size (bytes) in entries
+                            size_t totalSize,      // IN: Total size (bytes) in entries
+                            size_t *actual);       // OUT: number of bytes written
 
 FileIOResult FileIO_Pread(FileIODescriptor *fd,    // IN: File descriptor
                           void *buf,               // IN: Buffer to read into
@@ -335,17 +364,23 @@ Bool    FileIO_Truncate(FileIODescriptor *file,
 
 int     FileIO_Sync(const FileIODescriptor *file);
 
+FileIOResult FileIO_GetAllocSize(const FileIODescriptor *fd,
+                                 uint64 *logicalBytes,
+                                 uint64 *allocedBytes);
 int64   FileIO_GetSize(const FileIODescriptor *fd);
 
-int64   FileIO_GetAllocSize(const FileIODescriptor *fd);
+Bool    FileIO_SetAllocSize(const FileIODescriptor *fd, uint64 size);
 
-Bool    FileIO_SetAllocSize(const FileIODescriptor *fd,
-                            uint64 size);
+FileIOResult FileIO_GetAllocSizeByPath(ConstUnicode pathName,
+                                       uint64 *logicalBytes,
+                                       uint64 *allocedBytes);
 
 int64   FileIO_GetSizeByPath(ConstUnicode pathName);
 
 Bool    FileIO_Close(FileIODescriptor *file);
 
+Bool    FileIO_CloseAndUnlink(FileIODescriptor *file);
+
 uint32  FileIO_GetFlags(FileIODescriptor *file);
 
 Bool    FileIO_GetVolumeSectorSize(const char *name,
