@@ -296,7 +296,9 @@ typedef enum {
    HGFS_LOCK_OPPORTUNISTIC,
    HGFS_LOCK_EXCLUSIVE,
    HGFS_LOCK_SHARED,
-} HgfsServerLock;
+   HGFS_LOCK_BATCH,
+   HGFS_LOCK_LEASE,
+} HgfsLockType;
 
 
 /*
@@ -396,6 +398,8 @@ typedef uint64 HgfsAttrFlags;
 #define HGFS_ATTR_SPARSE                (1 << 9)
 #define HGFS_ATTR_TEMPORARY             (1 << 10)
 
+#define HGFS_ATTR_SEQUENTIAL_ONLY       (1 << 11)
+
 /*
  * Specifies which open request fields contain
  * valid values.
@@ -713,7 +717,7 @@ struct HgfsRequestOpenV2 {
    uint64 allocationSize;        /* How much space to pre-allocate during creation */
    uint32 desiredAccess;         /* Extended support for windows access modes */
    uint32 shareAccess;           /* Windows only, share access modes */
-   HgfsServerLock desiredLock;   /* The type of lock desired by the client */
+   HgfsLockType desiredLock;     /* The type of lock desired by the client */
    uint64 reserved1;             /* Reserved for future use */
    uint64 reserved2;             /* Reserved for future use */
    HgfsFileName fileName;
@@ -738,7 +742,7 @@ struct HgfsRequestOpenV3 {
    uint64 allocationSize;        /* How much space to pre-allocate during creation */
    uint32 desiredAccess;         /* Extended support for windows access modes */
    uint32 shareAccess;           /* Windows only, share access modes */
-   HgfsServerLock desiredLock;   /* The type of lock desired by the client */
+   HgfsLockType desiredLock;     /* The type of lock desired by the client */
    uint64 reserved1;             /* Reserved for future use */
    uint64 reserved2;             /* Reserved for future use */
    HgfsFileNameV3 fileName;
@@ -766,7 +770,7 @@ typedef
 struct HgfsReplyOpenV2 {
    HgfsReply header;
    HgfsHandle file;                  /* Opaque file ID used by the server */
-   HgfsServerLock acquiredLock;      /* The type of lock acquired by the server */
+   HgfsLockType acquiredLock;        /* The type of lock acquired by the server */
 }
 #include "vmware_pack_end.h"
 HgfsReplyOpenV2;
@@ -778,7 +782,7 @@ typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyOpenV3 {
    HgfsHandle file;                  /* Opaque file ID used by the server */
-   HgfsServerLock acquiredLock;      /* The type of lock acquired by the server */
+   HgfsLockType acquiredLock;        /* The type of lock acquired by the server */
    uint64 reserved;                  /* Reserved for future use */
 }
 #include "vmware_pack_end.h"
@@ -1588,7 +1592,7 @@ typedef
 struct HgfsRequestServerLockChange {
    HgfsRequest header;
    HgfsHandle file;
-   HgfsServerLock newServerLock;
+   HgfsLockType newServerLock;
 }
 #include "vmware_pack_end.h"
 HgfsRequestServerLockChange;
@@ -1598,7 +1602,7 @@ typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyServerLockChange {
    HgfsReply header;
-   HgfsServerLock serverLock;
+   HgfsLockType serverLock;
 }
 #include "vmware_pack_end.h"
 HgfsReplyServerLockChange;
@@ -2109,18 +2113,11 @@ HgfsReplyUnlockRangeV4;
  * HGFS_OPLOCK_BATCH: batch oplock. Read/Write and Open caching is allowed.
  */
 
-typedef enum {
-   HGFS_OPLOCK_NONE,
-   HGFS_OPLOCK_SHARED,
-   HGFS_OPLOCK_EXCLUSIVE,
-   HGFS_OPLOCK_BATCH,
-} HgfsOpportunisticLock;
-
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsRequestServerLockChangeV2 {
    HgfsHandle fid;                    /* File to take lock on. */
-   HgfsOpportunisticLock serverLock;  /* Lock type. */
+   HgfsLockType serverLock;           /* Lock type. */
    uint64 reserved;
 }
 #include "vmware_pack_end.h"
@@ -2129,7 +2126,7 @@ HgfsRequestServerLockChangeV2;
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyServerLockChangeV2 {
-   HgfsOpportunisticLock serverLock;  /* Lock granted. */
+   HgfsLockType serverLock;            /* Lock granted. */
    uint64 reserved;
 }
 #include "vmware_pack_end.h"
@@ -2144,7 +2141,7 @@ typedef
 #include "vmware_pack_begin.h"
 struct HgfsRequestOplockBreakV4 {
    HgfsHandle fid;                    /* File handle. */
-   HgfsOpportunisticLock serverLock;  /* Lock downgraded to this type. */
+   HgfsLockType serverLock;           /* Lock downgraded to this type. */
    uint64 reserved;                   /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
@@ -2303,7 +2300,7 @@ struct HgfsRequestOpenV4 {
    uint32 desiredAccess;         /* Extended support for windows access modes */
    uint32 shareAccess;           /* Windows only, share access modes */
    HgfsOpenCreateOptions createOptions; /* Various options. */
-   HgfsOpportunisticLock requestedLock;   /* The type of lock desired by the client */
+   HgfsLockType requestedLock;   /* The type of lock desired by the client */
    HgfsFileNameV3 fileName;      /* fid can be used only for relative open,
                                   * i.e. to open named stream.
                                   */
@@ -2348,7 +2345,7 @@ typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyOpenV4 {
    HgfsHandle file;                   /* Opaque file ID used by the server */
-   HgfsOpportunisticLock grantedLock; /* The type of lock acquired by the server */
+   HgfsLockType grantedLock;          /* The type of lock acquired by the server */
    HgfsOpenResult openResult;         /* Opened/overwritten or a new file created? */
    uint32 grantedAccess;              /* Granted access rights. */
    uint64 fileId;                     /* Persistent volume-wide unique file id. */
