@@ -50,8 +50,9 @@ struct HgfsVaIov {
 }HgfsVaIov;
 
 typedef enum {
-   BUF_READABLE,  /* Establish readable mappings */
-   BUF_WRITEABLE, /* Establish writeable mappings */
+   BUF_READABLE,      /* Establish readable mappings */
+   BUF_WRITEABLE,     /* Establish writeable mappings */
+   BUF_READWRITEABLE, /* Establish read-writeable mappings */
 } MappingType;
 
 typedef
@@ -121,6 +122,19 @@ HgfsSessionSendFunc(void *opaqueSession,  // IN
                     size_t bufferLen,     // IN
                     HgfsSendFlags flags); // IN
 
+/* Default maximum number of open nodes. */
+#define HGFS_MAX_CACHED_FILENODES   30
+
+typedef uint32 HgfsConfigFlags;
+#define HGFS_CONFIG_USE_HOST_TIME   (1 << 0)
+#define HGFS_CONFIG_NOTIFY_ENABLED  (1 << 1)
+#define HGFS_CONFIG_VOL_INFO_MIN    (1 << 2)
+
+typedef struct HgfsServerConfig {
+   HgfsConfigFlags flags;
+   uint32 maxCachedOpenNodes;
+}HgfsServerConfig;
+
 typedef struct HgfsServerChannelCallbacks {
     void* (*getReadVa)(uint64 pa, uint32 size, char **token);
     void* (*getWriteVa)(uint64 pa, uint32 size, char **token);
@@ -135,10 +149,13 @@ typedef struct HgfsServerSessionCallbacks {
    void (*close)(void *);
    void (*receive)(HgfsPacket *packet, void *);
    void (*invalidateObjects)(void *, DblLnkLst_Links *);
+   uint32 (*invalidateInactiveSessions)(void *);
    void (*sendComplete)(HgfsPacket *, void *);
 } HgfsServerSessionCallbacks;
 
-Bool HgfsServer_InitState(HgfsServerSessionCallbacks **, HgfsServerStateLogger *);
+Bool HgfsServer_InitState(HgfsServerSessionCallbacks **,
+                          HgfsServerConfig *,
+                          HgfsServerStateLogger *);
 void HgfsServer_ExitState(void);
 
 uint32 HgfsServer_GetHandleCounter(void);
