@@ -103,6 +103,7 @@
 #define RANK_scsiStateLock           (RANK_libLockBase + 0x5060)
 #define RANK_parInitLock             (RANK_libLockBase + 0x5070)
 #define RANK_namespaceLock           (RANK_libLockBase + 0x5080)
+#define RANK_vvolLibLock             (RANK_libLockBase + 0x5090)
 
 /*
  * VMDB range:
@@ -123,12 +124,10 @@
  * (RANK_libLockBase + 0x6500, RANK_libLockBase + 0x6600)
  */
 
-#define RANK_usbArbCliClientLock     (RANK_libLockBase + 0x6505)
-#define RANK_usbEnumClientsLock      (RANK_libLockBase + 0x6506)
-#define RANK_usbArbCliGlobalLock     (RANK_libLockBase + 0x6507)
-#define RANK_usbEnumBackendsLock     (RANK_libLockBase + 0x6508)
-#define RANK_usbEnumBackendLock      (RANK_libLockBase + 0x6509)
-
+#define RANK_usbArbLibGlobalLock     (RANK_libLockBase + 0x6505)
+#define RANK_usbEnumGlobalLock       (RANK_libLockBase + 0x6506)
+#define RANK_usbArbLibAsockLock      (RANK_libLockBase + 0x6507)
+#define RANK_usbEnumBackendLock      (RANK_libLockBase + 0x6508)
 
 /*
  * misc locks
@@ -146,7 +145,7 @@
  *    licenseCheck < preference
  */
 
-#define RANK_vigorConnLock           (RANK_libLockBase + 0x7010)
+#define RANK_vigorTransportListLock  (RANK_libLockBase + 0x7010)
 #define RANK_getSafeTmpDirLock       (RANK_libLockBase + 0x7020)
 #define RANK_batteryLock             (RANK_libLockBase + 0x7030)
 #define RANK_buttonLock              (RANK_libLockBase + 0x7040)
