@@ -22,6 +22,9 @@
 #define INCLUDE_ALLOW_VMKERNEL
 #include "includeCheck.h"
 
+#include "vm_assert.h"
+#include "vm_basic_types.h"
+
 /*
  * Defines util functions shared between the userlevel code and the monitor.
  */
@@ -77,8 +80,6 @@ Util_Throttle(uint32 count)  // IN:
  *----------------------------------------------------------------------
  */
 
-#define UTIL_FASTRAND_SEED_MAX (0x7fffffff)
-
 uint32
 Util_FastRand(uint32 seed)
 {
