@@ -25,13 +25,12 @@
 #ifndef _VIX_COMMANDS_H_
 #define _VIX_COMMANDS_H_
 
-#include "vm_version.h"
 #include "vixOpenSource.h"
 
 /*
  * These describe the format of the message objects.
  * This will change when the client/vmx support different
- * structures for the message header. Hopefully, that won't 
+ * structures for the message header. Hopefully, that won't
  * happen.
  */
 #define VIX_COMMAND_MAGIC_WORD        0xd00d0001
@@ -41,12 +40,19 @@
  * These give upper bounds for how big any VIX IPC meesage
  * should be. There are for sanity checks and to ignore maliciously
  * large messages that may be part of an DoS attack. The may need to
- * be revised if large messages are added to the protocol. 
+ * be revised if large messages are added to the protocol.
  */
 
-#define VIX_COMMAND_MAX_SIZE           (16 * 1024 * 1024)  
+#define VIX_COMMAND_MAX_SIZE           (16 * 1024 * 1024)
 #define VIX_COMMAND_MAX_REQUEST_SIZE   65536
 
+/*
+ * We don't want to allow guest ops commands with input size too large.
+ * Limit it to the max request size with enough room for the credentials.
+ * Check bugs 824773, 926819 for more details.
+ */
+#define VIX_COMMAND_MAX_USER_INPUT_SIZE (VIX_COMMAND_MAX_REQUEST_SIZE - 5000)
+
 /*
  * The types of credential we can pass with any request.
  */
@@ -1699,7 +1705,7 @@ VixMsgReadEnvironmentVariablesRequest;
 
 typedef
 #include "vmware_pack_begin.h"
-struct VixMsgAddAuthPrincipalRequest {
+struct VixMsgAddAliasRequest {
    VixCommandRequestHeader header;
 
    uint32                  options;
@@ -1708,22 +1714,22 @@ struct VixMsgAddAuthPrincipalRequest {
    uint32                  pemCertLen;
    Bool                    addMapping;
 
-   int32                   principalType;    // one of VixGuestAuthPrincipalType
-   uint32                  principalNameLen;
-   uint32                  principalCommentLen;
+   int32                   subjectType;    // one of VixGuestAuthSubjectType
+   uint32                  subjectNameLen;
+   uint32                  aliasCommentLen;
 
    /* Followed by the NUL-terminated string arguments. */
    /* char[]               userName; */
    /* char[]               pemCert; */
-   /* char[]               principalName; */
-   /* char[]               principalComment; */
+   /* char[]               subjectName; */
+   /* char[]               aliasComment; */
 }
 #include "vmware_pack_end.h"
-VixMsgAddAuthPrincipalRequest;
+VixMsgAddAuthAliasRequest;
 
 typedef
 #include "vmware_pack_begin.h"
-struct VixMsgRemoveAuthPrincipalRequest {
+struct VixMsgRemoveAuthAliasRequest {
    VixCommandRequestHeader header;
 
    uint32                  options;
@@ -1731,22 +1737,22 @@ struct VixMsgRemoveAuthPrincipalRequest {
    uint32                  userNameLen;
    uint32                  pemCertLen;
 
-   // special case for RemoveProvider:
-   // if principalType is NONE, then all principals will be removed.
-   int32                   principalType;    // one of VixGuestAuthPrincipalType
-   uint32                  principalNameLen;
+   // special case for RemoveAliasByCert:
+   // if subjectType is NONE, then all aliases will be removed.
+   int32                   subjectType;    // one of VixGuestAuthSubjectType
+   uint32                  subjectNameLen;
 
    /* Followed by the NUL-terminated string arguments. */
    /* char[]               userName; */
    /* char[]               pemCert; */
-   /* char[]               principalName; */
+   /* char[]               subjectName; */
 }
 #include "vmware_pack_end.h"
-VixMsgRemoveAuthPrincipalRequest;
+VixMsgRemoveAuthAliasRequest;
 
 typedef
 #include "vmware_pack_begin.h"
-struct VixMsgListAuthPrincipalsRequest {
+struct VixMsgListAuthAliasesRequest {
    VixCommandRequestHeader header;
 
    uint32                  options;
@@ -1756,17 +1762,17 @@ struct VixMsgListAuthPrincipalsRequest {
    /* char[]               userName; */
 }
 #include "vmware_pack_end.h"
-VixMsgListAuthPrincipalsRequest;
+VixMsgListAuthAliasesRequest;
 
 typedef
 #include "vmware_pack_begin.h"
-struct VixMsgListMappedPrincipalsRequest {
+struct VixMsgListMappedAliasesRequest {
    VixCommandRequestHeader header;
 
    uint32                  options;
 }
 #include "vmware_pack_end.h"
-VixMsgListMappedPrincipalsRequest;
+VixMsgListMappedAliasesRequest;
 
 /*
  * Windows Registry Management Support.
@@ -2429,10 +2435,10 @@ enum {
    VIX_COMMAND_DELETE_GUEST_DIRECTORY_EX        = 195,
    VIX_COMMAND_HOT_CHANGE_MONITOR_TYPE          = 196,
 
-   VIX_COMMAND_ADD_AUTH_PRINCIPAL               = 197,
-   VIX_COMMAND_REMOVE_AUTH_PRINCIPAL            = 198,
-   VIX_COMMAND_LIST_AUTH_PROVIDER_PRINCIPALS    = 199,
-   VIX_COMMAND_LIST_AUTH_MAPPED_PRINCIPALS      = 200,
+   VIX_COMMAND_ADD_AUTH_ALIAS                   = 197,
+   VIX_COMMAND_REMOVE_AUTH_ALIAS                = 198,
+   VIX_COMMAND_LIST_AUTH_PROVIDER_ALIASES       = 199,
+   VIX_COMMAND_LIST_AUTH_MAPPED_ALIASES         = 200,
 
    VIX_COMMAND_CREATE_REGISTRY_KEY              = 201,
    VIX_COMMAND_LIST_REGISTRY_KEYS               = 202,
