@@ -86,16 +86,8 @@
 #include "vm_basic_types.h"
 #include "vm_basic_defs.h"
 
-
-/*
- * XXX old file code
- */
-
-#ifdef FILECODEINT
-#error "Don't define FILECODEINT.  It is obsolete."
-#endif
-#ifdef FILECODE
-#error "Don't define FILECODE.  It is obsolete."
+#ifdef __cplusplus
+extern "C" {
 #endif
 
 /*
@@ -124,9 +116,13 @@
            Panic(_##name##Fmt " bugNr=%d\n", __FILE__, __LINE__, bug)
 #endif /* VMKERNEL */
 
-#define AssertLengthFmt     _AssertLengthFmt
-#define AssertUnexpectedFmt _AssertUnexpectedFmt
-#define AssertNotTestedFmt  _AssertNotTestedFmt
+#ifdef VMX86_DEVEL
+#   define _ASSERT_WARNING(name) \
+           Warning(_##name##Fmt "\n", __FILE__, __LINE__)
+#else
+#   define _ASSERT_WARNING(name) \
+           Log(_##name##Fmt "\n", __FILE__, __LINE__)
+#endif
 
 #endif // }
 
@@ -137,11 +133,7 @@
 #define _AssertNotImplementedFmt   "NOT_IMPLEMENTED %s:%d"
 #define _AssertNotReachedFmt       "NOT_REACHED %s:%d"
 #define _AssertMemAllocFmt         "MEM_ALLOC %s:%d"
-
-// these are complete formats with newline
-#define _AssertLengthFmt           "LENGTH %s:%d r=%#x e=%#x\n"
-#define _AssertUnexpectedFmt       "UNEXPECTED %s:%d bugNr=%d\n"
-#define _AssertNotTestedFmt        "NOT_TESTED %s:%d\n"
+#define _AssertNotTestedFmt        "NOT_TESTED %s:%d"
 
 
 
@@ -165,12 +157,6 @@ EXTERN void LogThrottled(uint32 *count, const char *fmt, ...)
 EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
             PRINTF_DECL(2, 3);
 
-/* DB family:  messages which are parsed by logfile database system */
-#define WarningDB Warning
-#define LogDB Log
-#define WarningThrottledDB WarningThrottled
-#define LogThrottledDB LogThrottled
-
 
 /*
  * Stress testing: redefine ASSERT_IFNOT() to taste
@@ -221,6 +207,10 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define PANIC()        _ASSERT_PANIC(AssertPanic)
 #define PANIC_BUG(bug) _ASSERT_PANIC_BUG(bug, AssertPanic)
 
+#ifdef VMKERNEL
+#define ASSERT_OR_IN_PANIC(cond) ASSERT((cond) || Panic_IsSystemInPanic())
+#endif
+
 #define ASSERT_NOT_IMPLEMENTED(cond) \
            ASSERT_IFNOT(cond, NOT_IMPLEMENTED())
 #define ASSERT_NOT_IMPLEMENTED_BUG(bug, cond) \
@@ -243,14 +233,6 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define ASSERT_MEM_ALLOC(cond) \
            ASSERT_IFNOT(cond, _ASSERT_PANIC(AssertMemAlloc))
 
-#ifdef VMX86_DEVEL
-   #define ASSERT_LENGTH(real, expected) \
-              ASSERT_IFNOT((real) == (expected), \
-                 Panic(AssertLengthFmt, __FILE__, __LINE__, real, expected))
-#else
-   #define ASSERT_LENGTH(real, expected) ASSERT((real) == (expected))
-#endif
-
 #ifdef VMX86_DEVEL
    #define ASSERT_DEVEL(cond) ASSERT(cond)
 #else
@@ -260,31 +242,9 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define ASSERT_NO_INTERRUPTS()  ASSERT(!INTERRUPTS_ENABLED())
 #define ASSERT_HAS_INTERRUPTS() ASSERT(INTERRUPTS_ENABLED())
 
-#define ASSERT_LOG_UNEXPECTED(bug, cond) \
-           (UNLIKELY(!(cond)) ? LOG_UNEXPECTED(bug) : 0)
-#ifdef VMX86_DEVEL
-   #define LOG_UNEXPECTED(bug) \
-              Warning(AssertUnexpectedFmt, __FILE__, __LINE__, bug)
-#else
-   #define LOG_UNEXPECTED(bug) \
-              Log(AssertUnexpectedFmt, __FILE__, __LINE__, bug)
-#endif
-
-#define ASSERT_NOT_TESTED(cond) (UNLIKELY(!(cond)) ? NOT_TESTED() : 0)
-#ifdef VMX86_DEVEL
-   #define NOT_TESTED() Warning(AssertNotTestedFmt, __FILE__, __LINE__)
-#else
-   #define NOT_TESTED() Log(AssertNotTestedFmt, __FILE__, __LINE__)
-#endif
-
-#define NOT_TESTED_ONCE()                                               \
-   do {                                                                 \
-      static Bool alreadyPrinted = FALSE;                               \
-      if (UNLIKELY(!alreadyPrinted)) {                                  \
-	 alreadyPrinted = TRUE;                                         \
-	 NOT_TESTED();                                                  \
-      }                                                                 \
-   } while (0)
+#define NOT_TESTED() _ASSERT_WARNING(AssertNotTested)
+#define ASSERT_NOT_TESTED(cond) (UNLIKELY(!(cond)) ? NOT_TESTED() : (void)0)
+#define NOT_TESTED_ONCE() DO_ONCE(NOT_TESTED())
 
 #define NOT_TESTED_1024()                                               \
    do {                                                                 \
@@ -293,14 +253,16 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
       count = (count + 1) & 1023;                                       \
    } while (0)
 
-#define LOG_ONCE(_s)                                                    \
-   do {                                                                 \
-      static Bool logged = FALSE;                                       \
-      if (!logged) {                                                    \
-	 Log _s;                                                        \
-         logged = TRUE;                                                 \
-      }                                                                 \
-   } while (0)
+#define LOG_ONCE(_s) DO_ONCE(Log _s)
+
+#ifdef VMX86_DEVEL
+   #define DEPRECATED(_fix) DO_ONCE(                                        \
+                               Warning("%s:%d: %s is DEPRECATED; %s\n",     \
+                                          __FILE__, __LINE__, __FUNCTION__, \
+                                          _fix))
+#else
+   #define DEPRECATED(_fix) do {} while (0)
+#endif
 
 
 /*
@@ -399,5 +361,8 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
       assertions \
    }
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
 
 #endif /* ifndef _VM_ASSERT_H_ */
