@@ -24,18 +24,35 @@
 #ifndef VM_LEGAL_H
 #define VM_LEGAL_H
 
-/*
- * COMPANY_NAME comes from vm_version.h
- */
-#include "vm_version.h"
+#include "vm_product.h"
+
+#ifndef WSTR
+#define WSTR_(x) L ## x
+#define WSTR(x) WSTR_(x)
+#endif
 
 
-#define COPYRIGHT_YEARS    "1998-2011"
-#define COPYRIGHT_STRING   "Copyright \251 " COPYRIGHT_YEARS " " COMPANY_NAME
-#define UTF8_COPYRIGHT_STRING   "Copyright \302\251 " COPYRIGHT_YEARS " " COMPANY_NAME
-#define GENERIC_COPYRIGHT_STRING   "Copyright (C) " COPYRIGHT_YEARS " " COMPANY_NAME
+#define COPYRIGHT_YEARS    "1998-2013"
+#define COPYRIGHT_STRING   "Copyright (C) " COPYRIGHT_YEARS " " COMPANY_NAME
 #define RIGHT_RESERVED     "All rights reserved."
 
+/*
+ * Use UTF8_COPYRIGHT_STRING_BASE when the COMPANY_NAME must be separated out
+ * to create a hyperlink.
+ */
+#define UTF8_COPYRIGHT_STRING_BASE   "Copyright \302\251 " COPYRIGHT_YEARS
+#define UTF8_COPYRIGHT_STRING   UTF8_COPYRIGHT_STRING_BASE " " COMPANY_NAME
+
+/*
+ * A UTF-16 version of the copyright string.  wchar_t is an
+ * implementation-defined type, but we can expect it to be UTF-16 on
+ * Windows. (Only Windows cares about UTF-16 anyway.)
+ */
+#ifdef _WIN32
+#define UTF16_COPYRIGHT_STRING L"Copyright \x00A9 " WSTR(COPYRIGHT_YEARS) L" " WSTR(COMPANY_NAME)
+#endif
+
+
 /*
  * Use PATENTS_STRING for showing the patents string in plaintext form.
  * PATENTS_FMT_STRING can be used with PATENTS_URL for creating hyperlinks.
@@ -45,7 +62,7 @@
 #define PATENTS_FMT_STRING PATENTS_STRING_BASE "%s."
 #define PATENTS_URL "http://www.vmware.com/go/patents"
 
-#define TRADEMARK_STRING   "VMware, the VMware \"boxes\" logo and design, Virtual SMP and VMotion are\nregistered trademarks or trademarks of VMware, Inc. in the United States\nand/or other jurisdictions."
+#define TRADEMARK_STRING "VMware is a registered trademark or trademark of VMware, Inc. in the\nUnited States and/or other jurisdictions."
 #define GENERIC_TRADEMARK_STRING "All other marks and names mentioned herein may be trademarks of their\nrespective companies."
 
 #endif /* VM_LEGAL_H */
