@@ -90,6 +90,7 @@
 #define PRODUCT_PLAYER_NAME_FOR_LICENSE PRODUCT_PLAYER_NAME
 #define PRODUCT_ACE_PLAYER_NAME MAKE_NAME(PRODUCT_ACE_PLAYER_BRIEF_NAME)
 #define PRODUCT_ACE_MANAGEMENT_SERVER_NAME MAKE_NAME(PRODUCT_ACE_MANAGEMENT_SERVER_BRIEF_NAME)
+#define PRODUCT_MAC_DESKTOP_NAME_FOR_LICENSE "VMware Fusion for Mac OS"
 
 #define PRODUCT_VMLS_SHORT_NAME "VMLS"
 #define PRODUCT_VMLS_NAME MAKE_NAME("License Server")
@@ -393,7 +394,7 @@
 #      define PRODUCT_SMP_NAME_FOR_LICENSE PRODUCT_ESX_SMP_NAME
 #   elif defined(VMX86_DESKTOP)
 #      if defined(__APPLE__)
-#         define PRODUCT_NAME_FOR_LICENSE "VMware Fusion for Mac OS"
+#         define PRODUCT_NAME_FOR_LICENSE PRODUCT_MAC_DESKTOP_NAME_FOR_LICENSE
 #      else
 #         define PRODUCT_NAME_FOR_LICENSE "VMware Workstation"
 #      endif
@@ -439,21 +440,6 @@
 #   else
 #      if defined(__APPLE__)
 #         define VMWARE_HOST_DIRECTORY VMWARE_HOST_DIRECTORY_PREFIX
-#      elif defined(__ACESC_LICENSE__)
-/*
- * This definition (__ACESC_LICENSE__) is used by the acesc licensing.
- * The licensing API uses VMWARE_HOST_DIRECTORY definition to save the activated
- * license (as well as searching).
- * In our case, the ACESC will use this customized directory, instead of the common
- * '/etc/vmware'.
- * The main motivation for this is that the acesc configuration application is
- * a web application (cgi app). Using the common directory, the apache process
- * does not have enough permission to write into the directory (/etc/vmware).
- * Instead of making the /etc/vmware writeable by everybody, we just create another
- * subdirectory (/etc/vmware/acesc).
- */
-#         define VMWARE_HOST_DIRECTORY "/etc/vmware/acesc"
-#         define DEFAULT_LIBDIRECTORY "/usr/lib/" PRODUCT_GENERIC_NAME_LOWER
 #      else
 #         define VMWARE_HOST_DIRECTORY "/etc/" PRODUCT_GENERIC_NAME_LOWER
 #         define DEFAULT_LIBDIRECTORY "/usr/lib/" PRODUCT_GENERIC_NAME_LOWER
@@ -502,7 +488,7 @@
  */
 #   if defined(VMX86_DESKTOP)
 #      if defined(__APPLE__)
-#         define PRODUCT_NAME_FOR_LICENSE "VMware Fusion for Mac OS"
+#         define PRODUCT_NAME_FOR_LICENSE PRODUCT_MAC_DESKTOP_NAME_FOR_LICENSE
 #      else
 #         define PRODUCT_NAME_FOR_LICENSE "VMware Workstation"
 #      endif
