@@ -30,7 +30,6 @@
 #define INCLUDE_ALLOW_VMCORE
 #include "includeCheck.h"
 
-#include "buildNumber.h"
 #include "vm_basic_types.h"
 #endif /* RC_INVOKED */
 
@@ -1045,6 +1044,20 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_ESX50_RELEASE_V_MNR  6
 #define   TOOLS_VERSION_ESX50_RELEASE_V_BASE 0
 
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_ESX50_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_ESX50_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_ESX50_UPDATE1_V_MJR  8
+#define   TOOLS_VERSION_ESX50_UPDATE1_V_MNR  6
+#define   TOOLS_VERSION_ESX50_UPDATE1_V_BASE 5
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_ESX50_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_ESX50_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_ESX50_UPDATE2_V_MJR  8
+#define   TOOLS_VERSION_ESX50_UPDATE2_V_MNR  6
+#define   TOOLS_VERSION_ESX50_UPDATE2_V_BASE 10
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_HOSTED11_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED11_BETA1_V)
 #endif /* RC_INVOKED */
@@ -1059,11 +1072,99 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_HOSTED11_BETA2_V_MNR  7
 #define   TOOLS_VERSION_HOSTED11_BETA2_V_BASE 1
 
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED11_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED11_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED11_RELEASE_V_MJR  8
+#define   TOOLS_VERSION_HOSTED11_RELEASE_V_MNR  8
+#define   TOOLS_VERSION_HOSTED11_RELEASE_V_BASE 0
+
+/* Fusion 4.1.2 (hosted11-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED11_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED11_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED11_UPDATE3_V_MJR  8
+#define   TOOLS_VERSION_HOSTED11_UPDATE3_V_MNR  8
+#define   TOOLS_VERSION_HOSTED11_UPDATE3_V_BASE 3
+
+/* Workstation 8.0.4, Fusion 4.1.3 (hosted11-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED11_UPDATE4 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED11_UPDATE4_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED11_UPDATE4_V_MJR  8
+#define   TOOLS_VERSION_HOSTED11_UPDATE4_V_MNR  8
+#define   TOOLS_VERSION_HOSTED11_UPDATE4_V_BASE 4
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_MNNEXT_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_MNNEXT_BETA1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_MNNEXT_BETA1_V_MJR  8
+#define   TOOLS_VERSION_MNNEXT_BETA1_V_MNR  9
+#define   TOOLS_VERSION_MNNEXT_BETA1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_MNNEXT_RC1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_MNNEXT_RC1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_MNNEXT_RC1_V_MJR  8
+#define   TOOLS_VERSION_MNNEXT_RC1_V_MNR  9
+#define   TOOLS_VERSION_MNNEXT_RC1_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_MNNEXT_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_MNNEXT_RELEASE_V)
+#define   TOOLS_VERSION_MNNEXT_RELEASE_V_MJR  9
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_MNNEXT_RELEASE_V_MNR  0
+#define   TOOLS_VERSION_MNNEXT_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_TECHPREVIEW2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_TECHPREVIEW2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_TECHPREVIEW2_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_TECHPREVIEW2_V_MNR  1
+#define   TOOLS_VERSION_HOSTED12_TECHPREVIEW2_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_RC_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_RC_V_MNR  1
+#define   TOOLS_VERSION_HOSTED12_RC_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_RELEASE_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_RELEASE_V_BASE 0
+
+/* Fusion 5.0.1 (hosted12-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_UPDATE1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_UPDATE1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_UPDATE1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_UPDATE1_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_UPDATE1_V_BASE 1
+
+/* Workstation 9.0.1, Fusion 5.0.2 (hosted12-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_UPDATE2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_UPDATE2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_UPDATE2_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_OP_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_BETA1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_OP_BETA1_V_MJR  9
+#define   TOOLS_VERSION_OP_BETA1_V_MNR  3
+#define   TOOLS_VERSION_OP_BETA1_V_BASE 1
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_NEXT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_NEXT_V)
 #endif /* RC_INVOKED */
-#define   TOOLS_VERSION_NEXT_V_MJR  8
-#define   TOOLS_VERSION_NEXT_V_MNR  9
+#define   TOOLS_VERSION_NEXT_V_MJR  9
+#define   TOOLS_VERSION_NEXT_V_MNR  5
 #define   TOOLS_VERSION_NEXT_V_BASE 0
 
 #define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_NEXT
