@@ -26,20 +26,23 @@
 
 typedef struct
 {
-   MXUserAcquisitionStats  acquisitionStats;
-   Atomic_Ptr              acquisitionHisto;
+   MXUserAcquisitionStats  data;
+   Atomic_Ptr              histo;
+} MXUserAcquireStats;
 
-   void                   *holder;
-   VmTimeType              holdStart;
-   MXUserBasicStats        heldStats;
-   Atomic_Ptr              heldHisto;
-} MXUserStats;
+typedef struct
+{
+   VmTimeType        holdStart;
+   MXUserBasicStats  data;
+   Atomic_Ptr        histo;
+} MXUserHeldStats;
 
 struct MXUserExclLock
 {
    MXUserHeader  header;
    MXRecLock     recursiveLock;
-   Atomic_Ptr    statsMem;
+   Atomic_Ptr    heldStatsMem;
+   Atomic_Ptr    acquireStatsMem;
 };
 
 
@@ -63,9 +66,18 @@ static void
 MXUserStatsActionExcl(MXUserHeader *header)  // IN:
 {
    MXUserExclLock *lock = (MXUserExclLock *) header;
-   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+   MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-   if (stats) {
+   if (UNLIKELY(heldStats != NULL)) {
+      MXUserDumpBasicStats(&heldStats->data, header);
+
+      if (Atomic_ReadPtr(&heldStats->histo) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&heldStats->histo), header);
+      }
+   }
+
+   if (LIKELY(acquireStats != NULL)) {
       Bool isHot;
       Bool doLog;
       double contentionRatio;
@@ -74,34 +86,30 @@ MXUserStatsActionExcl(MXUserHeader *header)  // IN:
        * Dump the statistics for the specified lock.
        */
 
-      MXUserDumpAcquisitionStats(&stats->acquisitionStats, header);
+      MXUserDumpAcquisitionStats(&acquireStats->data, header);
 
-      if (Atomic_ReadPtr(&stats->acquisitionHisto) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&stats->acquisitionHisto), header);
-      }
-
-      MXUserDumpBasicStats(&stats->heldStats, header);
-
-      if (Atomic_ReadPtr(&stats->heldHisto) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&stats->heldHisto), header);
+      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
       }
 
       /*
        * Has the lock gone "hot"? If so, implement the hot actions.
        */
 
-      MXUserKitchen(&stats->acquisitionStats, &contentionRatio, &isHot,
-                    &doLog);
+      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
 
-      if (isHot) {
-         MXUserForceHisto(&stats->acquisitionHisto,
+      if (UNLIKELY(isHot)) {
+         MXUserForceHisto(&acquireStats->histo,
                           MXUSER_STAT_CLASS_ACQUISITION,
                           MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
                           MXUSER_DEFAULT_HISTO_DECADES);
-         MXUserForceHisto(&stats->heldHisto,
-                          MXUSER_STAT_CLASS_HELD,
-                          MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
-                          MXUSER_DEFAULT_HISTO_DECADES);
+
+         if (UNLIKELY(heldStats != NULL)) {
+            MXUserForceHisto(&heldStats->histo,
+                             MXUSER_STAT_CLASS_HELD,
+                             MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
+                             MXUSER_DEFAULT_HISTO_DECADES);
+         }
 
          if (doLog) {
             Log("HOT LOCK (%s); contention ratio %f\n",
@@ -112,46 +120,6 @@ MXUserStatsActionExcl(MXUserHeader *header)  // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserDumpExclLock --
- *
- *      Dump an exclusive lock.
- *
- * Results:
- *      A dump.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-MXUserDumpExclLock(MXUserHeader *header)  // IN:
-{
-   MXUserExclLock *lock = (MXUserExclLock *) header;
-   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-   Warning("%s: Exclusive lock @ 0x%p\n", __FUNCTION__, lock);
-
-   Warning("\tsignature 0x%X\n", lock->header.signature);
-   Warning("\tname %s\n", lock->header.name);
-   Warning("\trank 0x%X\n", lock->header.rank);
-   Warning("\tserial number %u\n", lock->header.serialNumber);
-
-   Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
-
-   Warning("\taddress of owner data 0x%p\n",
-           &lock->recursiveLock.nativeThreadID);
-
-   if (stats && (stats->holder != NULL)) {
-      Warning("\tholder 0x%p\n", stats->holder);
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -181,22 +149,29 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
 
    switch (command) {
    case MXUSER_CONTROL_ACQUISITION_HISTO: {
-      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+      if (vmx86_stats) {
+         MXUserAcquireStats *acquireStats;
 
-      if (stats) {
-         va_list a;
-         uint64 minValue;
-         uint32 decades;
+         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-         va_start(a, command);
-         minValue = va_arg(a, uint64);
-         decades = va_arg(a, uint32);
-         va_end(a);
+         if (acquireStats == NULL) {
+            result = FALSE;
+         } else {
+            va_list a;
+            uint32 decades;
+            uint64 minValue;
 
-         MXUserForceHisto(&stats->acquisitionHisto,
-                          MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
+            va_start(a, command);
+            minValue = va_arg(a, uint64);
+            decades = va_arg(a, uint32);
+            va_end(a);
+
+            MXUserForceHisto(&acquireStats->histo,
+                             MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
+
+            result = TRUE;
+         }
 
-         result = TRUE;
       } else {
          result = FALSE;
       }
@@ -205,20 +180,77 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
    }
 
    case MXUSER_CONTROL_HELD_HISTO: {
-      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+      if (vmx86_stats) {
+         MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
 
-      if (stats) {
+         if (heldStats == NULL) {
+            result = FALSE;
+         } else {
+            va_list a;
+            uint32 decades;
+            uint32 minValue;
+
+            va_start(a, command);
+            minValue = va_arg(a, uint64);
+            decades = va_arg(a, uint32);
+            va_end(a);
+
+            MXUserForceHisto(&heldStats->histo, MXUSER_STAT_CLASS_HELD,
+                             minValue, decades);
+
+            result = TRUE;
+         }
+      } else {
+         result = FALSE;
+      }
+
+      break;
+   }
+
+   case MXUSER_CONTROL_ENABLE_STATS: {
+      if (vmx86_stats) {
          va_list a;
-         uint32 minValue;
-         uint32 decades;
+         Bool trackHeldTimes;
+         MXUserHeldStats *heldStats;
+         MXUserAcquireStats *acquireStats;
+
+         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
+
+         if (LIKELY(acquireStats == NULL)) {
+            MXUserAcquireStats *before;
+
+            acquireStats = Util_SafeCalloc(1, sizeof(*acquireStats));
+            MXUserAcquisitionStatsSetUp(&acquireStats->data);
+
+            before = Atomic_ReadIfEqualWritePtr(&lock->acquireStatsMem, NULL,
+                                                (void *) acquireStats);
+
+            if (before) {
+               free(acquireStats);
+            }
+         }
 
          va_start(a, command);
-         minValue = va_arg(a, uint64);
-         decades = va_arg(a, uint32);
+         trackHeldTimes = va_arg(a, int);
          va_end(a);
 
-         MXUserForceHisto(&stats->heldHisto, MXUSER_STAT_CLASS_HELD,
-                          minValue, decades);
+         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+
+         if ((heldStats == NULL) && trackHeldTimes) {
+            MXUserHeldStats *before;
+
+            heldStats = Util_SafeCalloc(1, sizeof(*heldStats));
+            MXUserBasicStatsSetUp(&heldStats->data, MXUSER_STAT_CLASS_HELD);
+
+            before = Atomic_ReadIfEqualWritePtr(&lock->heldStatsMem, NULL,
+                                                (void *) heldStats);
+
+            if (before) {
+               free(heldStats);
+            }
+         }
+
+         lock->header.statsFunc = MXUserStatsActionExcl;
 
          result = TRUE;
       } else {
@@ -228,32 +260,6 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
       break;
    }
 
-   case MXUSER_CONTROL_ENABLE_STATS: {
-      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-      if (LIKELY(stats == NULL)) {
-         MXUserStats *before;
-
-         stats = Util_SafeCalloc(1, sizeof(*stats));
-
-         MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
-         MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
-
-         before = (MXUserStats *) Atomic_ReadIfEqualWritePtr(&lock->statsMem,
-                                                             NULL,
-                                                             (void *) stats);
-
-         if (before) {
-            free(stats);
-         }
-
-         lock->header.statsFunc = MXUserStatsActionExcl;
-      }
-
-      result = TRUE;
-      break;
-   }
-
    default:
       result = FALSE;
    }
@@ -262,6 +268,41 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserDumpExclLock --
+ *
+ *      Dump an exclusive lock.
+ *
+ * Results:
+ *      A dump.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+MXUserDumpExclLock(MXUserHeader *header)  // IN:
+{
+   MXUserExclLock *lock = (MXUserExclLock *) header;
+
+   Warning("%s: Exclusive lock @ %p\n", __FUNCTION__, lock);
+
+   Warning("\tsignature 0x%X\n", lock->header.signature);
+   Warning("\tname %s\n", lock->header.name);
+   Warning("\trank 0x%X\n", lock->header.rank);
+   Warning("\tserial number %u\n", lock->header.serialNumber);
+
+   Warning("\tlock count %d\n", MXRecLockCount(&lock->recursiveLock));
+
+   Warning("\taddress of owner data %p\n",
+           &lock->recursiveLock.nativeThreadID);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -283,6 +324,7 @@ MXUserExclLock *
 MXUser_CreateExclLock(const char *userName,  // IN:
                       MX_Rank rank)          // IN:
 {
+   uint32 statsMode;
    char *properName;
    MXUserExclLock *lock;
 
@@ -307,11 +349,25 @@ MXUser_CreateExclLock(const char *userName,  // IN:
    lock->header.serialNumber = MXUserAllocSerialNumber();
    lock->header.dumpFunc = MXUserDumpExclLock;
 
-   if (MXUserStatsEnabled()) {
-      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS);
-   } else {
+   statsMode = MXUserStatsMode();
+
+   switch (statsMode) {
+   case 0:
       lock->header.statsFunc = NULL;
-      Atomic_WritePtr(&lock->statsMem, NULL);
+      Atomic_WritePtr(&lock->acquireStatsMem, NULL);
+      Atomic_WritePtr(&lock->heldStatsMem, NULL);
+      break;
+
+   case 1:
+      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS, FALSE);
+      break;
+
+   case 2:
+      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS, TRUE);
+      break;
+
+   default:
+      Panic("%s: unknown stats mode: %d!\n", __FUNCTION__, statsMode);
    }
 
    MXUserAddToList(&lock->header);
@@ -340,8 +396,6 @@ void
 MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 {
    if (lock != NULL) {
-      MXUserStats *stats;
-
       MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
       if (MXRecLockCount(&lock->recursiveLock) > 0) {
@@ -356,15 +410,27 @@ MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 
       MXUserRemoveFromList(&lock->header);
 
-      stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+      if (vmx86_stats) {
+         MXUserHeldStats *heldStats;
+         MXUserAcquireStats *acquireStats;
 
-      if (stats) {
-         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
-         MXUserBasicStatsTearDown(&stats->heldStats);
-         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
-         MXUserHistoTearDown(Atomic_ReadPtr(&stats->heldHisto));
+         acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-         free(stats);
+         if (LIKELY(acquireStats != NULL)) {
+            MXUserAcquisitionStatsTearDown(&acquireStats->data);
+            MXUserHistoTearDown(Atomic_ReadPtr(&acquireStats->histo));
+
+            free(acquireStats);
+         }
+
+         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+
+         if (UNLIKELY(heldStats != NULL)) {
+            MXUserBasicStatsTearDown(&heldStats->data);
+            MXUserHistoTearDown(Atomic_ReadPtr(&heldStats->histo));
+
+            free(heldStats);
+         }
       }
 
       free(lock->header.name);
@@ -393,48 +459,49 @@ MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
 void
 MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
-   MXUserStats *stats;
-
    ASSERT(lock);
    MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
    MXUserAcquisitionTracking(&lock->header, TRUE);
 
-   if (stats) {
-      VmTimeType begin = Hostinfo_SystemTimerNS();
-      VmTimeType value;
-      Bool contended;
-      MXUserHisto *histo;
+   if (vmx86_stats) {
+      VmTimeType value = 0;
+      MXUserHeldStats *heldStats;
+      MXUserAcquireStats *acquireStats;
 
-      contended = MXRecLockAcquire(&lock->recursiveLock);
+      acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
 
-      value = Hostinfo_SystemTimerNS() - begin;
+      MXRecLockAcquire(&lock->recursiveLock,
+                       (acquireStats == NULL) ? NULL : &value);
 
-      MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
-                              value);
+      if (LIKELY(acquireStats != NULL)) {
+         MXUserHisto *histo;
 
-      stats->holder = GetReturnAddress();
+         MXUserAcquisitionSample(&acquireStats->data, TRUE, value != 0,
+                                 value);
 
-      histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+         histo = Atomic_ReadPtr(&acquireStats->histo);
 
-      if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, value, stats->holder);
+         if (UNLIKELY(histo != NULL)) {
+            MXUserHistoSample(histo, value, GetReturnAddress());
+         }
+
+         heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
+
+         if (UNLIKELY(heldStats != NULL)) {
+            heldStats->holdStart = Hostinfo_SystemTimerNS();
+         }
       }
    } else {
-      MXRecLockAcquire(&lock->recursiveLock);
+      MXRecLockAcquire(&lock->recursiveLock,
+                       NULL);  // non-stats
    }
 
-   if (MXRecLockCount(&lock->recursiveLock) > 1) {
+   if (vmx86_debug && (MXRecLockCount(&lock->recursiveLock) > 1)) {
       MXUserDumpAndPanic(&lock->header,
                          "%s: Acquire on an acquired exclusive lock\n",
                          __FUNCTION__);
    }
-
-   if (stats) {
-      stats->holdStart = Hostinfo_SystemTimerNS();
-   }
 }
 
 
@@ -457,29 +524,26 @@ MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 void
 MXUser_ReleaseExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
-   MXUserStats *stats;
-
    ASSERT(lock);
    MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+   if (vmx86_stats) {
+      MXUserHeldStats *heldStats = Atomic_ReadPtr(&lock->heldStatsMem);
 
-   if (stats) {
-      VmTimeType value = Hostinfo_SystemTimerNS() - stats->holdStart;
-      MXUserHisto *histo;
+      if (UNLIKELY(heldStats != NULL)) {
+         VmTimeType value = Hostinfo_SystemTimerNS() - heldStats->holdStart;
+         MXUserHisto *histo = Atomic_ReadPtr(&heldStats->histo);
 
-      MXUserBasicStatsSample(&stats->heldStats, value);
+         MXUserBasicStatsSample(&heldStats->data, value);
 
-      histo = Atomic_ReadPtr(&stats->heldHisto);
-
-      if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, value, stats->holder);
-         stats->holder = NULL;
+         if (UNLIKELY(histo != NULL)) {
+            MXUserHistoSample(histo, value, GetReturnAddress());
+         }
       }
    }
 
-   if (!MXRecLockIsOwner(&lock->recursiveLock)) {
-      uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
+   if (vmx86_debug && !MXRecLockIsOwner(&lock->recursiveLock)) {
+      int lockCount = MXRecLockCount(&lock->recursiveLock);
 
       MXUserDumpAndPanic(&lock->header,
                          "%s: Non-owner release of an %s exclusive lock\n",
@@ -518,7 +582,6 @@ Bool
 MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
 {
    Bool success;
-   MXUserStats *stats;
 
    ASSERT(lock);
    MXUserValidateHeader(&lock->header, MXUSER_TYPE_EXCL);
@@ -532,18 +595,22 @@ MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
    if (success) {
       MXUserAcquisitionTracking(&lock->header, FALSE);
 
-      if (MXRecLockCount(&lock->recursiveLock) > 1) {
+      if (vmx86_debug && (MXRecLockCount(&lock->recursiveLock) > 1)) {
          MXUserDumpAndPanic(&lock->header,
                             "%s: Acquire on an acquired exclusive lock\n",
                             __FUNCTION__);
       }
    }
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+   if (vmx86_stats) {
+      MXUserAcquireStats *acquireStats;
 
-   if (stats) {
-      MXUserAcquisitionSample(&stats->acquisitionStats, success, !success,
-                              0ULL);
+      acquireStats = Atomic_ReadPtr(&lock->acquireStatsMem);
+
+      if (LIKELY(acquireStats != NULL)) {
+         MXUserAcquisitionSample(&acquireStats->data, success, !success,
+                                 0ULL);
+      }
    }
 
    return success;
@@ -605,18 +672,17 @@ MXUser_CreateSingletonExclLock(Atomic_Ptr *lockStorage,  // IN/OUT:
 
    ASSERT(lockStorage);
 
-   lock = (MXUserExclLock *) Atomic_ReadPtr(lockStorage);
+   lock = Atomic_ReadPtr(lockStorage);
 
    if (UNLIKELY(lock == NULL)) {
       MXUserExclLock *newLock = MXUser_CreateExclLock(name, rank);
 
-      lock = (MXUserExclLock *) Atomic_ReadIfEqualWritePtr(lockStorage, NULL,
-                                                           (void *) newLock);
+      lock = Atomic_ReadIfEqualWritePtr(lockStorage, NULL, (void *) newLock);
 
       if (lock) {
          MXUser_DestroyExclLock(newLock);
       } else {
-         lock = (MXUserExclLock *) Atomic_ReadPtr(lockStorage);
+         lock = Atomic_ReadPtr(lockStorage);
       }
    }
 
