@@ -33,7 +33,6 @@
 #include <sys/wait.h>
 #include <signal.h>
 #include <sys/time.h>
-#include <sys/timeb.h>
 #include <pwd.h>
 #include <pthread.h>
 #include <sys/resource.h>
@@ -44,16 +43,25 @@
 #if defined(__FreeBSD__) || defined(__APPLE__)
 # include <sys/sysctl.h>
 #endif
+#if !defined(__APPLE__)
+#define TARGET_OS_IPHONE 0
+#endif
 #if defined(__APPLE__)
 #include <assert.h>
+#include <TargetConditionals.h>
+#if !TARGET_OS_IPHONE
 #include <CoreServices/CoreServices.h>
+#endif
 #include <mach-o/dyld.h>
-#include <mach/host_info.h>
 #include <mach/mach_host.h>
 #include <mach/mach_init.h>
-#include <mach/mach.h>
 #include <mach/mach_time.h>
+#include <mach/vm_map.h>
 #include <sys/mman.h>
+#include <AvailabilityMacros.h>
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070 // Must run on Mac OS versions < 10.7
+#   include <dlfcn.h>
+#endif
 #elif defined(__FreeBSD__)
 #if !defined(RLIMIT_AS)
 #  if defined(RLIMIT_VMEM)
@@ -66,10 +74,8 @@
 #if !defined(USING_AUTOCONF) || defined(HAVE_SYS_VFS_H)
 #include <sys/vfs.h>
 #endif
-#if !defined(sun) && (!defined(USING_AUTOCONF) || (defined(HAVE_SYS_IO_H) && defined(HAVE_SYS_SYSINFO_H)))
-# ifndef __ANDROID__
-# include <sys/io.h>
-# endif
+#if !defined(sun) && !defined __ANDROID__ && (!defined(USING_AUTOCONF) || (defined(HAVE_SYS_IO_H) && defined(HAVE_SYS_SYSINFO_H)))
+#include <sys/io.h>
 #include <sys/sysinfo.h>
 #ifndef HAVE_SYSINFO
 #define HAVE_SYSINFO 1
@@ -94,7 +100,6 @@
 #include "hostinfo.h"
 #include "hostinfoInt.h"
 #include "safetime.h"
-#include "vm_version.h"
 #include "str.h"
 #include "err.h"
 #include "msg.h"
@@ -163,6 +168,7 @@ typedef struct distro_info {
 } DistroInfo;
 
 static const DistroInfo distroArray[] = {
+   {"OracleLinux",        "/etc/oracle-release"},
    {"RedHat",             "/etc/redhat-release"},
    {"RedHat",             "/etc/redhat_version"},
    {"Sun",                "/etc/sun-release"},
@@ -204,6 +210,14 @@ static const DistroInfo distroArray[] = {
    {NULL, NULL},
 };
 
+#if defined __ANDROID__
+/*
+ * Android doesn't support getloadavg() or iopl().
+ */
+#define NO_GETLOADAVG
+#define NO_IOPL
+#endif
+
 
 /*
  *----------------------------------------------------------------------
@@ -509,8 +523,11 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
       Str_Strcpy(distroShort, STR_OS_OPENSUSE, distroShortSize);
    } else if (strstr(distroLower, "suse")) {
       if (strstr(distroLower, "enterprise")) {
-         if (strstr(distroLower, "server 11") ||
-             strstr(distroLower, "desktop 11")) {
+         if (strstr(distroLower, "server 12") ||
+             strstr(distroLower, "desktop 12")) {
+            Str_Strcpy(distroShort, STR_OS_SLES_12, distroShortSize);
+         } else if (strstr(distroLower, "server 11") ||
+                    strstr(distroLower, "desktop 11")) {
             Str_Strcpy(distroShort, STR_OS_SLES_11, distroShortSize);
          } else if (strstr(distroLower, "server 10") ||
                     strstr(distroLower, "desktop 10")) {
@@ -560,6 +577,8 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
          Str_Strcpy(distroShort, STR_OS_DEBIAN_5, distroShortSize);
       } else if (strstr(distroLower, "6.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_6, distroShortSize);
+      } else if (strstr(distroLower, "7.0")) {
+         Str_Strcpy(distroShort, STR_OS_DEBIAN_7, distroShortSize);
       }
    } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
               StrUtil_StartsWith(distroLower, "oracle")) {
@@ -897,9 +916,12 @@ HostinfoOSData(void)
       } else if (majorVersion == 2 && Hostinfo_OSVersion(1) < 6) {
          Str_Strcpy(distro, STR_OS_OTHER_24_FULL, distroSize);
          Str_Strcpy(distroShort, STR_OS_OTHER_24, distroSize);
-      } else {
+      } else if (majorVersion == 2) {
          Str_Strcpy(distro, STR_OS_OTHER_26_FULL, distroSize);
          Str_Strcpy(distroShort, STR_OS_OTHER_26, distroSize);
+      } else {
+         Str_Strcpy(distro, STR_OS_OTHER_3X_FULL, distroSize);
+         Str_Strcpy(distroShort, STR_OS_OTHER_3X, distroSize);
       }
 
       /*
@@ -1417,7 +1439,7 @@ HostinfoGetLoadAverage(float *avg0,  // IN/OUT:
                        float *avg1,  // IN/OUT:
                        float *avg2)  // IN/OUT:
 {
-#if (defined(__linux__) && !defined(__UCLIBC__)) || defined(__APPLE__)
+#if !defined(NO_GETLOADAVG) && (defined(__linux__) && !defined(__UCLIBC__)) || defined(__APPLE__)
    double avg[3];
    int res;
 
@@ -1603,8 +1625,14 @@ HostinfoSystemTimerMach(VmTimeType *result)  // OUT
 {
 #if __APPLE__
 #  define vmx86_apple 1
+
+   typedef struct {
+      double  scalingFactor;
+      Bool    unity;
+   } timerData;
+
    VmTimeType raw;
-   mach_timebase_info_data_t *ptr;
+   timerData *ptr;
    static Atomic_Ptr atomic; /* Implicitly initialized to NULL. --mbellon */
 
    /*
@@ -1613,31 +1641,34 @@ HostinfoSystemTimerMach(VmTimeType *result)  // OUT
     */
 
    /* Ensure that the time base values are correct. */
-   ptr = (mach_timebase_info_data_t *) Atomic_ReadPtr(&atomic);
+   ptr = Atomic_ReadPtr(&atomic);
 
    if (UNLIKELY(ptr == NULL)) {
-      char *p;
+      mach_timebase_info_data_t timeBase;
+      timerData *try = Util_SafeMalloc(sizeof *try);
 
-      p = Util_SafeMalloc(sizeof(mach_timebase_info_data_t));
+      mach_timebase_info(&timeBase);
 
-      mach_timebase_info((mach_timebase_info_data_t *) p);
+      try->scalingFactor = ((double) timeBase.numer) /
+                           ((double) timeBase.denom);
 
-      if (Atomic_ReadIfEqualWritePtr(&atomic, NULL, p)) {
-         free(p);
+      try->unity = ((timeBase.numer == 1) && (timeBase.denom == 1));
+
+      if (Atomic_ReadIfEqualWritePtr(&atomic, NULL, try)) {
+         free(try);
       }
 
-      ptr = (mach_timebase_info_data_t *) Atomic_ReadPtr(&atomic);
+      ptr = Atomic_ReadPtr(&atomic);
    }
 
    raw = mach_absolute_time();
 
-   if ((ptr->numer == 1) && (ptr->denom == 1)) {
-      /* The scaling values are unity, save some time/arithmetic */
+   if (LIKELY(ptr->unity)) {
       *result = raw;
    } else {
-      /* The scaling values are not unity. Prevent overflow when scaling */
-      *result = ((double) raw) * (((double) ptr->numer) / ((double) ptr->denom));
+      *result = ((double) raw) * ptr->scalingFactor;
    }
+
    return TRUE;
 #else
 #  define vmx86_apple 0
@@ -1871,7 +1902,12 @@ Hostinfo_ResetProcessState(const int *keepFds, // IN:
          privileges --hpreg */
       ASSERT(euid != 0 || getuid() == 0);
       Id_SetEUid(0);
+#if defined NO_IOPL
+      NOT_IMPLEMENTED();
+      errno = ENOSYS;
+#else
       err = iopl(0);
+#endif
       Id_SetEUid(euid);
       ASSERT_NOT_IMPLEMENTED(err == 0);
    }
@@ -2610,42 +2646,62 @@ Hostinfo_Execute(const char *path,   // IN:
 
 
 #ifdef __APPLE__
+/*
+ * How to retrieve kernel zone information. A little bit of history
+ * ---
+ * 1) In Mac OS versions < 10.6, we could retrieve kernel zone information like
+ *    zprint did, i.e. by invoking the host_zone_info() Mach call.
+ *
+ *    osfmk/mach/mach_host.defs defines both arrays passed to host_zone_info()
+ *    as 'out' parameters, but the implementation of the function in
+ *    osfmk/kern/zalloc.c clearly treats them as 'inout' parameters. This issue
+ *    is confirmed in practice: the input values passed by the user process are
+ *    ignored. Now comes the scary part: is the input of the kernel function
+ *    deterministically invalid, or is it some non-deterministic garbage (in
+ *    which case the kernel could corrupt the user address space)? The answer
+ *    is in the Mach IPC code. A cursory kernel debugging session seems to
+ *    imply that the input pointer values are garbage, but the input size
+ *    values are always 0. So host_zone_info() seems safe to use in practice.
+ *
+ * 2) In Mac OS 10.6, Apple introduced the 64-bit kernel.
+ *
+ *    2.1) They modified host_zone_info() to always returns KERN_NOT_SUPPORTED
+ *         when the sizes (32-bit or 64-bit) of the user and kernel virtual
+ *         address spaces do not match. Was bug 377049.
+ *
+ *         zprint got away with it by re-executing itself to match the kernel.
+ *
+ *    2.2) They broke the ABI for 64-bit user processes: the
+ *         'zone_info.zi_*_size' fields are 32-bit in the Mac OS 10.5 SDK, and
+ *         64-bit in the Mac OS 10.6 SDK. So a 64-bit user process compiled
+ *         against the Mac OS 10.5 SDK works with the Mac OS 10.5 (32-bit)
+ *         kernel but fails with the Mac OS 10.6 64-bit kernel.
+ *
+ *         zprint in Mac OS 10.6 is compiled against the Mac OS 10.6 SDK, so it
+ *         got away with it.
+ *
+ *    The above two things made it very impractical for us to keep calling
+ *    host_zone_info(). Instead we invoked zprint and parsed its non-localized
+ *    output.
+ *
+ * 3) In Mac OS 10.7, Apple cleaned their mess and solved all the above
+ *    problems by introducing a new mach_zone_info() Mach call. So this is what
+ *    we use now, when available. Was bug 816610.
+ *
+ * 4) In Mac OS 10.8, Apple appears to have modified mach_zone_info() to always
+ *    return KERN_INVALID_HOST(!) when the calling process (not the calling
+ *    thread!) is not root.
+ */
+
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070 // Must run on Mac OS versions < 10.7
 /*
  *-----------------------------------------------------------------------------
  *
- * Hostinfo_GetKernelZoneElemSize --
+ * HostinfoGetKernelZoneElemSizeZprint --
  *
- *      Retrieve the size of the elements in a named kernel zone.
- *
- *      We used to do it like zprint (see
- *      darwinsource-10.4.5/system_cmds-336.10/zprint.tproj/zprint.c::main()),
- *      i.e. by calling host_zone_info(), but there are 3 problems with that:
- *
- *      1) mach/mach_host.defs defines both arrays passed to host_zone_info()
- *         as 'out' parameters, but the implementation of the function in
- *         xnu-792.13.8/osfmk/kern/zalloc.c clearly expects them as 'inout'
- *         parameters. This issue is confirmed in practice: the input values
- *         passed by the user process are ignored. Now comes the scary part: is
- *         the input of the kernel function deterministically invalid, or is it
- *         some non-deterministic garbage (in which case the user process can
- *         corrupt its address space)? The answer is in the Mach IPC code. A
- *         cursory kernel debugging session seems to imply that the input
- *         pointer values are garbage, but the input size values are always 0.
- *         So the function seems safe to use in practice.
- *
- *      2) Starting with Mac OS 10.6, host_zone_info() always returns
- *         KERN_NOT_SUPPORTED when the sizes of the user and kernel virtual
- *         address spaces (32-bit or 64-bit) do not match. Was bug 377049.
- *
- *      3) Apple broke the ABI: For 64-bit code, the 'zone_info.zi_*_size'
- *         fields are 32-bit in the Mac OS 10.5 SDK, but 64-bit in the Mac OS
- *         10.6 SDK. So a 64-bit VMX compiled against the Mac OS 10.5 SDK works
- *         with the Mac OS 10.5 (32-bit) kernel but fails with the Mac OS 10.6
- *         64-bit kernel.
- *
- *      So now we just let Apple deal with their own mess: we invoke zprint,
- *      and we parse its non-localized output. Should Apple stop shipping
- *      zprint, we can always ship our own replacement for it.
+ *      Retrieve the size of the elements in a named kernel zone, by invoking
+ *      zprint.
  *
  * Results:
  *      On success: the size (in bytes) > 0.
@@ -2657,8 +2713,8 @@ Hostinfo_Execute(const char *path,   // IN:
  *-----------------------------------------------------------------------------
  */
 
-size_t
-Hostinfo_GetKernelZoneElemSize(char const *name) // IN: Kernel zone name
+static size_t
+HostinfoGetKernelZoneElemSizeZprint(char const *name) // IN: Kernel zone name
 {
    size_t retval = 0;
    struct {
@@ -2764,6 +2820,93 @@ Hostinfo_GetKernelZoneElemSize(char const *name) // IN: Kernel zone name
 
    return retval;
 }
+#endif
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Hostinfo_GetKernelZoneElemSize --
+ *
+ *      Retrieve the size of the elements in a named kernel zone.
+ *
+ * Results:
+ *      On success: the size (in bytes) > 0.
+ *      On failure: 0.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+size_t
+Hostinfo_GetKernelZoneElemSize(char const *name) // IN: Kernel zone name
+{
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1070 // Compiles against SDK version < 10.7
+   typedef struct {
+      char mzn_name[80];
+   } mach_zone_name_t;
+   typedef struct {
+      uint64_t mzi_count;
+      uint64_t mzi_cur_size;
+      uint64_t mzi_max_size;
+      uint64_t mzi_elem_size;
+      uint64_t mzi_alloc_size;
+      uint64_t mzi_sum_size;
+      uint64_t mzi_exhaustible;
+      uint64_t mzi_collectable;
+   } mach_zone_info_t;
+#endif
+   size_t result = 0;
+   mach_zone_name_t *namesPtr;
+   mach_msg_type_number_t namesSize;
+   mach_zone_info_t *infosPtr;
+   mach_msg_type_number_t infosSize;
+   kern_return_t kr;
+   mach_msg_type_number_t i;
+
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1070 // Must run on Mac OS versions < 10.7
+   kern_return_t (*mach_zone_info)(host_t host,
+      mach_zone_name_t **names, mach_msg_type_number_t *namesCnt,
+      mach_zone_info_t **info, mach_msg_type_number_t *infoCnt) =
+         dlsym(RTLD_DEFAULT, "mach_zone_info");
+   if (!mach_zone_info) {
+      return HostinfoGetKernelZoneElemSizeZprint(name);
+   }
+#endif
+
+   ASSERT(name);
+
+   kr = mach_zone_info(mach_host_self(), &namesPtr, &namesSize, &infosPtr,
+                       &infosSize);
+   if (kr != KERN_SUCCESS) {
+      Warning("%s: mach_zone_info failed %u.\n", __FUNCTION__, kr);
+      return result;
+   }
+
+   ASSERT(namesSize == infosSize);
+   for (i = 0; i < namesSize; i++) {
+      if (!strcmp(namesPtr[i].mzn_name, name)) {
+         result = infosPtr[i].mzi_elem_size;
+         /* Check that nothing of value was lost during the cast. */
+         ASSERT(result == infosPtr[i].mzi_elem_size);
+         break;
+      }
+   }
+
+   ASSERT_ON_COMPILE(sizeof namesPtr <= sizeof (vm_address_t));
+   kr = vm_deallocate(mach_task_self(), (vm_address_t)namesPtr,
+                      namesSize * sizeof *namesPtr);
+   ASSERT(kr == KERN_SUCCESS);
+
+   ASSERT_ON_COMPILE(sizeof infosPtr <= sizeof (vm_address_t));
+   kr = vm_deallocate(mach_task_self(), (vm_address_t)infosPtr,
+                      infosSize * sizeof *infosPtr);
+   ASSERT(kr == KERN_SUCCESS);
+
+   return result;
+}
 
 
 /*
