@@ -26,10 +26,7 @@
 #include "safetime.h"
 #include "unicode.h"
 
-#if defined(N_PLAT_NLM)
-#  include <sys/timeval.h>
-#  include <nwtime.h>
-#elif defined(_WIN32)
+#if defined(_WIN32)
 #  include <wtypes.h>
 #else
 #  include <sys/time.h>
@@ -37,8 +34,6 @@
 #include <ctype.h>
 
 #include "vmware.h"
-/* For HARD_EXPIRE --hpreg */
-#include "vm_version.h"
 #include "vm_basic_asm.h"
 #include "timeutil.h"
 #include "str.h"
@@ -871,7 +866,6 @@ TimeUtil_GetTimeFormat(int64 utcTime,  // IN
 }
 
 
-#if !defined N_PLAT_NLM
 /*
  *-----------------------------------------------------------------------------
  *
@@ -957,7 +951,6 @@ TimeUtil_UnixTimeToNtTime(struct timespec unixTime) // IN: Time in Unix format
    return (VmTimeType)unixTime.tv_sec * 10000000 +
                                           unixTime.tv_nsec / 100 + UNIX_EPOCH;
 }
-#endif // N_PLAT_NLM
 
 #ifdef _WIN32
 /*
@@ -1093,7 +1086,7 @@ TimeUtil_GetLocalWindowsTimeZoneIndexAndName(char **ptzName)   // OUT: returning
       struct tm tim;
       localtime_r(&now, &tim);
 
-      #if defined(sun) || defined N_PLAT_NLM
+      #if defined(sun)
          /*
           * Offset is to standard (no need for DST adjustment).
           * Negative is east of prime meridian.
