@@ -47,14 +47,14 @@
 #include <errno.h>
 #include <ctype.h>
 
-#if !defined(_WIN32) && !defined(N_PLAT_NLM)
+#if !defined(_WIN32)
 #  include <unistd.h>
 #  include <getopt.h>
 #  include <pwd.h>
 #  include <dlfcn.h>
 #endif
 
-#if defined(__linux__) && !defined(VMX86_TOOLS)
+#if defined(__linux__) && !defined(VMX86_TOOLS) && !defined(__ANDROID__)
 #  include <link.h>
 #endif
 
@@ -62,8 +62,6 @@
 #include "msg.h"
 #include "util.h"
 #include "str.h"
-/* For HARD_EXPIRE --hpreg */
-#include "vm_version.h"
 #include "su.h"
 #include "posix.h"
 #include "file.h"
@@ -428,7 +426,7 @@ UtilBacktraceFromPointerCallback(struct _Unwind_Context *ctx, // IN: Unwind cont
    return _URC_END_OF_STACK;
 }
 
-#if !defined(_WIN32) && !defined(N_PLAT_NLM) && !defined(VMX86_TOOLS)
+#if !defined(_WIN32) && !defined(VMX86_TOOLS) && !defined(__ANDROID__)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -558,7 +556,7 @@ Util_BacktraceFromPointerWithFunc(uintptr_t *basePtr,
    data.skippedFrames = 0;
    _Unwind_Backtrace(UtilBacktraceFromPointerCallback, &data);
 
-#if !defined(_WIN32) && !defined(N_PLAT_NLM) && !defined(VMX86_TOOLS)
+#if !defined(_WIN32) && !defined(VMX86_TOOLS)
    /* 
     * We do a separate pass here that includes symbols in order to
     * make sure the base backtrace that does not call dladdr() etc.
@@ -575,7 +573,7 @@ Util_BacktraceFromPointerWithFunc(uintptr_t *basePtr,
 #elif !defined(VM_X86_64)
    uintptr_t *x = basePtr;
    int i;
-#if !defined(_WIN32) && !defined(N_PLAT_NLM) && !defined(VMX86_TOOLS)
+#if !defined(_WIN32) && !defined(VMX86_TOOLS) && !defined(__ANDROID__)
    Dl_info dli;
 #endif
 
@@ -588,7 +586,7 @@ Util_BacktraceFromPointerWithFunc(uintptr_t *basePtr,
       x = (uintptr_t *) x[0];
    }
 
-#if !defined(_WIN32) && !defined(N_PLAT_NLM) && !defined(VMX86_TOOLS)
+#if !defined(_WIN32) && !defined(VMX86_TOOLS) && !defined(__ANDROID__)
    /* 
     * We do a separate pass here that includes symbols in order to
     * make sure the base backtrace that does not call dladdr() etc.
@@ -772,7 +770,7 @@ Util_BacktraceWithFunc(int bugNr, Util_OutputFunc outFunc, void *outFuncData)
 
 
 /* XXX This should go in a separate utilPosix.c file --hpreg */
-#if !defined(_WIN32) && !defined(N_PLAT_NLM)
+#if !defined(_WIN32)
 /*
  *----------------------------------------------------------------------
  *
@@ -1140,7 +1138,6 @@ end:
 #endif
 }
 
-#if !defined(N_PLAT_NLM)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1377,5 +1374,3 @@ Util_SeparateStrings(char *source,              // IN
 
    return stringVector;
 }
-
-#endif /* !defined(N_PLAT_NLM) */
