@@ -39,6 +39,7 @@
 #include "vmware/tools/utils.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolslib_version.h"
 VM_EMBED_VERSION(VMTOOLSLIB_VERSION_STRING);
@@ -95,7 +96,7 @@ VMToolsDllInit(void *lib)
 #else
    (void) wiperData;
    success = Wiper_Init(NULL);
-   ASSERT(success);
+   ASSERT_NOT_IMPLEMENTED(success);
 #endif
 }
 
