@@ -163,6 +163,8 @@ VMToolsLogFormat(const gchar *message,
    char *msg = NULL;
    const char *slevel;
    size_t len = 0;
+   gboolean shared = TRUE;
+   gboolean addsTimestamp = TRUE;
 
    if (domain == NULL) {
       domain = gLogDomain;
@@ -205,11 +207,16 @@ VMToolsLogFormat(const gchar *message,
       slevel = "unknown";
    }
 
-   if (!data->logger->addsTimestamp) {
+   if (data->logger != NULL) {
+      shared = data->logger->shared;
+      addsTimestamp = data->logger->addsTimestamp;
+   }
+
+   if (!addsTimestamp) {
       char *tstamp;
 
       tstamp = System_GetTimeAsString();
-      if (data->logger->shared) {
+      if (shared) {
          len = VMToolsAsprintf(&msg, "[%s] [%8s] [%s:%s] %s\n",
                                (tstamp != NULL) ? tstamp : "no time",
                                slevel, gLogDomain, domain, message);
@@ -220,7 +227,7 @@ VMToolsLogFormat(const gchar *message,
       }
       free(tstamp);
    } else {
-      if (data->logger->shared) {
+      if (shared) {
          len = VMToolsAsprintf(&msg, "[%8s] [%s:%s] %s\n",
                                slevel, gLogDomain, domain, message);
       } else {
@@ -303,7 +310,7 @@ VMToolsLog(const gchar *domain,
 
       if (data->logger != NULL) {
          data->logger->logfn(domain, level, msg, data->logger);
-      } else {
+      } else if (gErrorData->logger != NULL) {
          gErrorData->logger->logfn(domain, level, msg, gErrorData->logger);
       }
       g_free(msg);
@@ -314,6 +321,78 @@ VMToolsLog(const gchar *domain,
 }
 
 
+/*
+ *******************************************************************************
+ * VMToolsGetLogFilePath --                                               */ /**
+ *
+ * @brief Fetches sanitized path for the log file.
+ *
+ * @param[in] key       The key for log file path.
+ * @param[in] domain    Domain name.
+ * @param[in] cfg       Config dictionary.
+ *
+ * @return Sanitized path for the log file.
+ *
+ *******************************************************************************
+ */
+
+static gchar *
+VMToolsGetLogFilePath(const gchar *key,
+                      const gchar *domain,
+                      GKeyFile *cfg)
+{
+   gsize len = 0;
+   gchar *path = NULL;
+   gchar *origPath = NULL;
+
+   path = g_key_file_get_string(cfg, LOGGING_GROUP, key, NULL);
+   if (path == NULL) {
+      return NULL;
+   }
+
+   len = strlen(path);
+   origPath = path;
+
+   /*
+    * Drop all the preceding '"' chars
+    */
+   while (*path == '"') {
+      path++;
+      len--;
+   }
+
+   /*
+    * Ensure that path contains something more
+    * meaningful than just '"' chars
+    */
+   if (len == 0) {
+      g_warning("Invalid path for domain '%s'.", domain);
+      g_free(origPath);
+      path = NULL;
+   } else {
+      /* Drop the trailing '"' chars */
+      gchar *sanePath = g_strdup(path);
+      g_free(origPath);
+      path = sanePath;
+
+      if (path != NULL) {
+         while (*(path + len - 1) == '"') {
+            *(path + len - 1) = '\0';
+            len--;
+         }
+
+         if (len == 0) {
+            g_warning("Invalid path for domain '%s'.", domain);
+            g_free(origPath);
+            path = NULL;
+         }
+      }
+   }
+
+   return path;
+}
+
+
 /*
  *******************************************************************************
  * VMToolsGetLogHandler --                                                */ /**
@@ -351,7 +430,7 @@ VMToolsGetLogHandler(const gchar *handler,
       handler = "file";
 
       g_snprintf(key, sizeof key, "%s.data", domain);
-      path = g_key_file_get_string(cfg, LOGGING_GROUP, key, NULL);
+      path = VMToolsGetLogFilePath(key, domain, cfg);
       if (path != NULL) {
          g_snprintf(key, sizeof key, "%s.maxLogSize", domain);
          maxSize = (guint) g_key_file_get_integer(cfg, LOGGING_GROUP, key, &err);
@@ -395,6 +474,10 @@ VMToolsGetLogHandler(const gchar *handler,
       g_warning("Invalid handler for domain '%s': %s", domain, handler);
    }
 
+   if (NULL == glogger) {
+      g_warning("Failed to create a logger for handler: '%s'", handler);
+   }
+
    logger = g_new0(LogHandler, 1);
    logger->domain = g_strdup(domain);
    logger->logger = glogger;
