@@ -61,21 +61,6 @@ static int VMCIEventRegisterSubscription(VMCISubscription *sub,
                                          void *callbackData);
 static VMCISubscription *VMCIEventUnregisterSubscription(VMCIId subID);
 
-
-#if defined(VMKERNEL)
-   /* VMK doesn't need BH locks, so use lower ranks. */
-#  define VMCIEventInitLock(_lock, _name) \
-   VMCI_InitLock(_lock, _name, VMCI_LOCK_RANK_HIGHER)
-#  define VMCIEventGrabLock(_lock, _flags)    VMCI_GrabLock(_lock, _flags)
-#  define VMCIEventReleaseLock(_lock, _flags) VMCI_ReleaseLock(_lock, _flags)
-#else // VMKERNEL
-#  define VMCIEventInitLock(_lock, _name) \
-   VMCI_InitLock(_lock, _name, VMCI_LOCK_RANK_HIGHER_BH)
-#  define VMCIEventGrabLock(_lock, _flags)    VMCI_GrabLock_BH(_lock, _flags)
-#  define VMCIEventReleaseLock(_lock, _flags) VMCI_ReleaseLock_BH(_lock, _flags)
-#endif // VMKERNEL
-
-
 static VMCIList subscriberArray[VMCI_EVENT_MAX];
 static VMCILock subscriberLock;
 
@@ -84,6 +69,10 @@ typedef struct VMCIDelayedEventInfo {
    uint8 eventPayload[sizeof(VMCIEventData_Max)];
 } VMCIDelayedEventInfo;
 
+typedef struct VMCIEventRef {
+   VMCISubscription *sub;
+   VMCIListItem   listItem;
+} VMCIEventRef;
 
 /*
  *----------------------------------------------------------------------
@@ -93,7 +82,7 @@ typedef struct VMCIDelayedEventInfo {
  *      General init code.
  *
  * Results:
- *      None.
+ *      VMCI_SUCCESS on success, appropriate error code otherwise.
  *
  * Side effects:
  *      None.
@@ -110,7 +99,8 @@ VMCIEvent_Init(void)
       VMCIList_Init(&subscriberArray[i]);
    }
 
-   return VMCIEventInitLock(&subscriberLock, "VMCIEventSubscriberLock");
+   return VMCI_InitLock(&subscriberLock, "VMCIEventSubscriberLock",
+                        VMCI_LOCK_RANK_EVENT);
 }
 
 
@@ -181,8 +171,8 @@ void
 VMCIEvent_Sync(void)
 {
    VMCILockFlags lockFlags;
-   VMCIEventGrabLock(&subscriberLock, &lockFlags);
-   VMCIEventReleaseLock(&subscriberLock, lockFlags);
+   VMCI_GrabLock_BH(&subscriberLock, &lockFlags);
+   VMCI_ReleaseLock_BH(&subscriberLock, lockFlags);
 }
 
 
@@ -291,9 +281,9 @@ EventReleaseCB(void *clientData) // IN
 
    ASSERT(sub);
 
-   VMCIEventGrabLock(&subscriberLock, &flags);
+   VMCI_GrabLock_BH(&subscriberLock, &flags);
    VMCIEventRelease(sub);
-   VMCIEventReleaseLock(&subscriberLock, flags);
+   VMCI_ReleaseLock_BH(&subscriberLock, flags);
 
    return 0;
 }
@@ -369,9 +359,9 @@ VMCIEventDelayedDispatchCB(void *data) // IN
 
    sub->callback(sub->id, ed, sub->callbackData);
 
-   VMCIEventGrabLock(&subscriberLock, &flags);
+   VMCI_GrabLock_BH(&subscriberLock, &flags);
    VMCIEventRelease(sub);
-   VMCIEventReleaseLock(&subscriberLock, flags);
+   VMCI_ReleaseLock_BH(&subscriberLock, flags);
 
    VMCI_FreeKernelMem(eventInfo, sizeof *eventInfo);
 }
@@ -400,11 +390,13 @@ VMCIEventDeliver(VMCIEventMsg *eventMsg)  // IN
    VMCIListItem *iter;
    VMCILockFlags flags;
 
+   VMCIList noDelayList;
+   VMCIList_Init(&noDelayList);
+
    ASSERT(eventMsg);
 
-   VMCIEventGrabLock(&subscriberLock, &flags);
+   VMCI_GrabLock_BH(&subscriberLock, &flags);
    VMCIList_Scan(iter, &subscriberArray[eventMsg->eventData.event]) {
-      VMCI_EventData *ed;
       VMCISubscription *cur = VMCIList_Entry(iter, VMCISubscription,
                                              subscriberListItem);
       ASSERT(cur && cur->event == eventMsg->eventData.event);
@@ -433,6 +425,40 @@ VMCIEventDeliver(VMCIEventMsg *eventMsg)  // IN
          }
 
       } else {
+         VMCIEventRef *eventRef;
+
+         /*
+          * To avoid possible lock rank voilation when holding
+          * subscriberLock, we construct a local list of
+          * subscribers and release subscriberLock before
+          * invokes the callbacks. This is similar to delayed
+          * callbacks, but callbacks is invoked right away here.
+          */
+         if ((eventRef = VMCI_AllocKernelMem(sizeof *eventRef,
+                                             (VMCI_MEMORY_ATOMIC |
+                                              VMCI_MEMORY_NONPAGED))) == NULL) {
+            err = VMCI_ERROR_NO_MEM;
+            goto out;
+         }
+
+         VMCIEventGet(cur);
+         eventRef->sub = cur;
+         VMCIList_InitEntry(&eventRef->listItem);
+         VMCIList_Insert(&eventRef->listItem, &noDelayList);
+      }
+   }
+
+out:
+   VMCI_ReleaseLock_BH(&subscriberLock, flags);
+
+   if (!VMCIList_Empty(&noDelayList)) {
+      VMCI_EventData *ed;
+      VMCIListItem *iter2;
+
+      VMCIList_ScanSafe(iter, iter2, &noDelayList) {
+         VMCIEventRef *eventRef = VMCIList_Entry(iter, VMCIEventRef,
+                                                 listItem);
+         VMCISubscription *cur = eventRef->sub;
          uint8 eventPayload[sizeof(VMCIEventData_Max)];
 
          /* We set event data before each callback to ensure isolation. */
@@ -441,12 +467,14 @@ VMCIEventDeliver(VMCIEventMsg *eventMsg)  // IN
                 (size_t)eventMsg->hdr.payloadSize);
          ed = (VMCI_EventData *)eventPayload;
          cur->callback(cur->id, ed, cur->callbackData);
+
+         VMCI_GrabLock_BH(&subscriberLock, &flags);
+         VMCIEventRelease(cur);
+         VMCI_ReleaseLock_BH(&subscriberLock, flags);
+         VMCI_FreeKernelMem(eventRef, sizeof *eventRef);
       }
    }
 
-out:
-   VMCIEventReleaseLock(&subscriberLock, flags);
-
    return err;
 }
 
@@ -535,9 +563,25 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
       /*
        * In the vmkernel we defer delivery of events to a helper world.  This
        * makes the event delivery more consistent across hosts and guests with
-       * regard to which locks are held.
+       * regard to which locks are held.  Memory access events are an exception
+       * to this, since clients need to know immediately that the device
+       * memory is disabled (if we delay such events, then clients may be
+       * notified too late).
        */
-      sub->runDelayed = TRUE;
+      if (VMCI_EVENT_MEM_ACCESS_ON == event ||
+          VMCI_EVENT_MEM_ACCESS_OFF == event) {
+         /*
+          * Client must expect to get such events synchronously, and should
+          * perform its locking accordingly.  If it can't handle this, then
+          * fail.
+          */
+         if (flags & VMCI_FLAG_EVENT_DELAYED_CB) {
+            return VMCI_ERROR_INVALID_ARGS;
+         }
+         sub->runDelayed = FALSE;
+      } else {
+         sub->runDelayed = TRUE;
+      }
    } else if (!VMCI_CanScheduleDelayedWork()) {
       /*
        * If the platform doesn't support delayed work callbacks then don't
@@ -561,7 +605,7 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
    sub->callbackData = callbackData;
    VMCIList_InitEntry(&sub->subscriberListItem);
 
-   VMCIEventGrabLock(&subscriberLock, &lockFlags);
+   VMCI_GrabLock_BH(&subscriberLock, &lockFlags);
 
    /* Check if creation of a new event is allowed. */
    if (!VMCI_CanCreate()) {
@@ -599,7 +643,7 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
    }
 
 exit:
-   VMCIEventReleaseLock(&subscriberLock, lockFlags);
+   VMCI_ReleaseLock_BH(&subscriberLock, lockFlags);
    return result;
 #  undef VMCI_EVENT_MAX_ATTEMPTS
 }
@@ -628,13 +672,13 @@ VMCIEventUnregisterSubscription(VMCIId subID)    // IN
    VMCILockFlags flags;
    VMCISubscription *s;
 
-   VMCIEventGrabLock(&subscriberLock, &flags);
+   VMCI_GrabLock_BH(&subscriberLock, &flags);
    s = VMCIEventFind(subID);
    if (s != NULL) {
       VMCIEventRelease(s);
       VMCIList_Remove(&s->subscriberListItem);
    }
-   VMCIEventReleaseLock(&subscriberLock, flags);
+   VMCI_ReleaseLock_BH(&subscriberLock, flags);
 
    if (s != NULL) {
       VMCI_WaitOnEvent(&s->destroyEvent, EventReleaseCB, s);
