@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -66,9 +66,11 @@
 #include "vmci_handle_array.h"
 #include "vmci_defs.h"
 #include "vmciKernelAPI.h"
+#include "vmciQueue.h"
 #include "vmciQueuePair.h"
 #include "vmciRoute.h"
 
+
 /*
  * VMCIQPair
  *
@@ -85,10 +87,332 @@ struct VMCIQPair {
    uint32 flags;
    VMCIPrivilegeFlags privFlags;
    Bool guestEndpoint;
+   uint32 blocked;
+   VMCIEvent event;
 };
 
-#define VMCI_QPAIR_NO_QUEUE(_qp) (!(_qp)->produceQ->qHeader || \
-                                  !(_qp)->consumeQ->qHeader)
+static int VMCIQPairMapQueueHeaders(VMCIQueue *produceQ, VMCIQueue *consumeQ,
+                                    Bool canBlock);
+static int VMCIQPairGetQueueHeaders(const VMCIQPair *qpair,
+                                    VMCIQueueHeader **produceQHeader,
+                                    VMCIQueueHeader **consumeQHeader);
+static int VMCIQPairWakeupCB(void *clientData);
+static int VMCIQPairReleaseMutexCB(void *clientData);
+static Bool VMCIQPairWaitForReadyQueue(VMCIQPair *qpair);
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairLock --
+ *
+ *      Helper routine that will lock the QPair before subsequent operations.
+ *
+ * Results:
+ *      VMCI_SUCCESS if lock acquired. VMCI_ERROR_WOULD_BLOCK if queue mutex
+ *      couldn't be acquired and qpair isn't allowed to block.
+ *
+ * Side effects:
+ *      May block.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE int
+VMCIQPairLock(const VMCIQPair *qpair) // IN
+{
+#if !defined VMX86_VMX
+   if (qpair->flags & VMCI_QPFLAG_PINNED) {
+      VMCI_LockQueueHeader(qpair->produceQ);
+      return VMCI_SUCCESS;
+   }
+   return VMCI_AcquireQueueMutex(qpair->produceQ,
+                                 !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+#else
+   return VMCI_SUCCESS
+#endif
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairUnlock --
+ *
+ *      Helper routine that will unlock the QPair after various operations.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+VMCIQPairUnlock(const VMCIQPair *qpair) // IN
+{
+#if !defined VMX86_VMX
+   if (qpair->flags & VMCI_QPFLAG_PINNED) {
+      VMCI_UnlockQueueHeader(qpair->produceQ);
+   } else {
+      VMCI_ReleaseQueueMutex(qpair->produceQ);
+   }
+#endif
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairHeaderLock --
+ *
+ *      Helper routine that will lock the queue pair header before subsequent
+ *      operations. If the queue pair is non blocking, a spinlock will be used.
+ *      Otherwise, a regular mutex locking the complete queue pair will be used.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      May block.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+VMCIQPairLockHeader(const VMCIQPair *qpair) // IN
+{
+#if !defined VMX86_VMX
+   if (qpair->flags & VMCI_QPFLAG_NONBLOCK) {
+      VMCI_LockQueueHeader(qpair->produceQ);
+   } else {
+      (void)VMCI_AcquireQueueMutex(qpair->produceQ,
+                                   !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+   }
+#endif
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairUnlockHeader --
+ *
+ *      Helper routine that unlocks the queue pair header after calling
+ *      VMCIQPairHeaderLock.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+VMCIQPairUnlockHeader(const VMCIQPair *qpair) // IN
+{
+#if !defined VMX86_VMX
+   if (qpair->flags & VMCI_QPFLAG_NONBLOCK) {
+      VMCI_UnlockQueueHeader(qpair->produceQ);
+   } else {
+      VMCI_ReleaseQueueMutex(qpair->produceQ);
+   }
+#endif
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairGetQueueHeaders --
+ *
+ *      Helper routine that will retrieve the produce and consume
+ *      headers of a given queue pair. If the guest memory of the
+ *      queue pair is currently not available, the saved queue headers
+ *      will be returned, if these are available.
+ *
+ * Results:
+ *      VMCI_SUCCESS if either current or saved queue headers are found.
+ *      Appropriate error code otherwise.
+ *
+ * Side effects:
+ *      May block.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+VMCIQPairGetQueueHeaders(const VMCIQPair *qpair,            // IN
+                         VMCIQueueHeader **produceQHeader,  // OUT
+                         VMCIQueueHeader **consumeQHeader)  // OUT
+{
+   int result;
+
+   result = VMCIQPairMapQueueHeaders(qpair->produceQ, qpair->consumeQ,
+                                     !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+   if (result == VMCI_SUCCESS) {
+      *produceQHeader = qpair->produceQ->qHeader;
+      *consumeQHeader = qpair->consumeQ->qHeader;
+   } else if (qpair->produceQ->savedHeader && qpair->consumeQ->savedHeader) {
+      ASSERT(!qpair->guestEndpoint);
+      *produceQHeader = qpair->produceQ->savedHeader;
+      *consumeQHeader = qpair->consumeQ->savedHeader;
+      result = VMCI_SUCCESS;
+   }
+
+   return result;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairMapQueueHeaders --
+ *
+ *      The queue headers may not be mapped at all times. If a queue is
+ *      currently not mapped, it will be attempted to do so.
+ *
+ * Results:
+ *      VMCI_SUCCESS if queues were validated, appropriate error code otherwise.
+ *
+ * Side effects:
+ *      May attempt to map in guest memory.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+VMCIQPairMapQueueHeaders(VMCIQueue *produceQ, // IN
+                         VMCIQueue *consumeQ, // IN
+                         Bool canBlock)       // IN
+{
+   int result;
+
+   if (NULL == produceQ->qHeader || NULL == consumeQ->qHeader) {
+      if (canBlock) {
+         /*
+          * We return data from creator of the queue in VM2VM case.
+          * That should be OK, as if they do not match then there
+          * is somebody else in progress of making them match, and
+          * you should not be looking at somebody else's queue if
+          * queue is active.
+          */
+         result = VMCIHost_MapQueues(0, produceQ, consumeQ, 0);
+      } else {
+         result = VMCI_ERROR_QUEUEPAIR_NOT_READY;
+      }
+      if (result < VMCI_SUCCESS) {
+         if (produceQ->savedHeader && consumeQ->savedHeader) {
+            return VMCI_ERROR_QUEUEPAIR_NOT_READY;
+         } else {
+            return VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
+         }
+      }
+   }
+
+   return VMCI_SUCCESS;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairWakeupCB --
+ *
+ *      Callback from VMCI queue pair broker indicating that a queue
+ *      pair that was previously not ready, now either is ready or
+ *      gone forever.
+ *
+ * Results:
+ *      VMCI_SUCCESS always.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+VMCIQPairWakeupCB(void *clientData)
+{
+   VMCIQPair *qpair = (VMCIQPair *)clientData;
+   ASSERT(qpair);
+
+   VMCIQPairLock(qpair);
+   while (qpair->blocked > 0) {
+      qpair->blocked--;
+      VMCI_SignalEvent(&qpair->event);
+   }
+   VMCIQPairUnlock(qpair);
+
+   return VMCI_SUCCESS;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairReleaseMutexCB --
+ *
+ *      Callback from VMCI_WaitOnEvent releasing the queue pair mutex
+ *      protecting the queue pair header state.
+ *
+ * Results:
+ *      0 always.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+VMCIQPairReleaseMutexCB(void *clientData)
+{
+   VMCIQPair *qpair = (VMCIQPair *)clientData;
+   ASSERT(qpair);
+   VMCIQPairUnlock(qpair);
+   return 0;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPairWaitForReadyQueue --
+ *
+ *      Makes the calling thread wait for the queue pair to become
+ *      ready for host side access.
+ *
+ * Results:
+ *     TRUE when thread is woken up after queue pair state change.
+ *     FALSE otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static Bool
+VMCIQPairWaitForReadyQueue(VMCIQPair *qpair)
+{
+   if (UNLIKELY(qpair->guestEndpoint)) {
+      ASSERT(FALSE);
+      return FALSE;
+   }
+   if (qpair->flags & VMCI_QPFLAG_NONBLOCK) {
+      return FALSE;
+   }
+   qpair->blocked++;
+   VMCI_WaitOnEvent(&qpair->event, VMCIQPairReleaseMutexCB, qpair);
+   VMCIQPairLock(qpair);
+   return TRUE;
+}
 
 
 /*
@@ -127,6 +451,8 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
    VMCIHandle src = VMCI_INVALID_HANDLE;
    VMCIHandle dst = VMCI_MAKE_HANDLE(peer, VMCI_INVALID_ID);
    VMCIRoute route;
+   VMCIEventReleaseCB wakeupCB;
+   void *clientData;
 
    /*
     * Restrict the size of a queuepair.  The device already enforces a limit
@@ -149,18 +475,6 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
       return VMCI_ERROR_NO_RESOURCES;
    }
 
-   myQPair = VMCI_AllocKernelMem(sizeof *myQPair, VMCI_MEMORY_NONPAGED);
-   if (!myQPair) {
-      return VMCI_ERROR_NO_MEM;
-   }
-   memset(myQPair, 0, sizeof *myQPair);
-
-   myQPair->produceQSize = produceQSize;
-   myQPair->consumeQSize = consumeQSize;
-   myQPair->peer = peer;
-   myQPair->flags = flags;
-   myQPair->privFlags = privFlags;
-
    retval = VMCI_Route(&src, &dst, FALSE, &route);
    if (retval < VMCI_SUCCESS) {
       if (VMCI_GuestPersonalityActive()) {
@@ -170,8 +484,54 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
       }
    }
 
+   if ((flags & (VMCI_QPFLAG_NONBLOCK | VMCI_QPFLAG_PINNED)) && !vmkernel) {
+#if defined(linux)
+      if (VMCI_ROUTE_AS_GUEST != route)
+#endif // linux
+      {
+         return VMCI_ERROR_INVALID_ARGS;
+      }
+   }
+
+   if (flags & VMCI_QPFLAG_PINNED) {
+      /*
+       * Pinned pages implies non-blocking mode.  Technically it doesn't
+       * have to, but there doesn't seem much point in pinning the pages if you
+       * can block since the queues will be small, so there's no performance
+       * gain to be had.
+       */
+
+      if (!(flags & VMCI_QPFLAG_NONBLOCK)) {
+         return VMCI_ERROR_INVALID_ARGS;
+      }
+
+      /* Limit the amount of memory that can be pinned. */
+
+      if (produceQSize + consumeQSize > VMCI_MAX_PINNED_QP_MEMORY) {
+         return VMCI_ERROR_NO_RESOURCES;
+      }
+   }
+
+   myQPair = VMCI_AllocKernelMem(sizeof *myQPair, VMCI_MEMORY_NONPAGED);
+   if (!myQPair) {
+      return VMCI_ERROR_NO_MEM;
+   }
+
+   myQPair->produceQSize = produceQSize;
+   myQPair->consumeQSize = consumeQSize;
+   myQPair->peer = peer;
+   myQPair->flags = flags;
+   myQPair->privFlags = privFlags;
+
+   wakeupCB = clientData = NULL;
    if (VMCI_ROUTE_AS_HOST == route) {
       myQPair->guestEndpoint = FALSE;
+      if (!(flags & VMCI_QPFLAG_LOCAL)) {
+         myQPair->blocked = 0;
+         VMCI_CreateEvent(&myQPair->event);
+         wakeupCB = VMCIQPairWakeupCB;
+         clientData = (void *)myQPair;
+      }
    } else {
       myQPair->guestEndpoint = TRUE;
    }
@@ -184,9 +544,14 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
                                 myQPair->peer,
                                 myQPair->flags,
                                 myQPair->privFlags,
-                                myQPair->guestEndpoint);
+                                myQPair->guestEndpoint,
+                                wakeupCB,
+                                clientData);
 
    if (retval < VMCI_SUCCESS) {
+      if (VMCI_ROUTE_AS_HOST == route && !(flags & VMCI_QPFLAG_LOCAL)) {
+         VMCI_DestroyEvent(&myQPair->event);
+      }
       VMCI_FreeKernelMem(myQPair, sizeof *myQPair);
       return retval;
    }
@@ -238,9 +603,9 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
     * there isn't much the caller can do, and we don't want to leak.
     */
 
-   memset(oldQPair, 0, sizeof *oldQPair);
-   oldQPair->handle = VMCI_INVALID_HANDLE;
-   oldQPair->peer = VMCI_INVALID_ID;
+   if (!(oldQPair->guestEndpoint || (oldQPair->flags & VMCI_QPFLAG_LOCAL))) {
+      VMCI_DestroyEvent(&oldQPair->event);
+   }
    VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
    *qpair = NULL;
 
@@ -248,68 +613,6 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
 }
 
 
-/*
- * "Windows blocking call."
- *
- *      Note that on the Windows platform, kernel module clients may
- *      block when calling into any these rouintes.  The reason is
- *      that a mutex has to be acquired in order to view/modify the
- *      VMCIQueue structure fields: pointers, handle, and buffer data.
- *      However, other platforms don't require the acquisition of a
- *      mutex and thus don't block.
- */
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIQPair_Lock --
- *
- *      Helper routine that will lock the QPair before subsequent operations.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Windows blocking call.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE void
-VMCIQPairLock(const VMCIQPair *qpair) // IN
-{
-#if !defined VMX86_VMX
-   VMCI_AcquireQueueMutex(qpair->produceQ);
-#endif
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIQPair_Unlock --
- *
- *      Helper routine that will unlock the QPair after various operations.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE void
-VMCIQPairUnlock(const VMCIQPair *qpair) // IN
-{
-#if !defined VMX86_VMX
-   VMCI_ReleaseQueueMutex(qpair->produceQ);
-#endif
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -334,25 +637,21 @@ VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
                             uint64 *producerTail,   // OUT
                             uint64 *consumerHead)   // OUT
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
-   } else {
-      VMCIQueueHeader_GetPointers(qpair->produceQ->qHeader,
-                                  qpair->consumeQ->qHeader,
-                                  producerTail,
-                                  consumerHead);
-      result = VMCI_SUCCESS;
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      VMCIQueueHeader_GetPointers(produceQHeader, consumeQHeader,
+                                  producerTail, consumerHead);
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    if (result == VMCI_SUCCESS &&
        ((producerTail && *producerTail >= qpair->produceQSize) ||
@@ -388,25 +687,21 @@ VMCIQPair_GetConsumeIndexes(const VMCIQPair *qpair, // IN
                             uint64 *consumerTail,   // OUT
                             uint64 *producerHead)   // OUT
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
-   } else {
-      VMCIQueueHeader_GetPointers(qpair->consumeQ->qHeader,
-                                  qpair->produceQ->qHeader,
-                                  consumerTail,
-                                  producerHead);
-      result = VMCI_SUCCESS;
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      VMCIQueueHeader_GetPointers(consumeQHeader, produceQHeader,
+                                  consumerTail, producerHead);
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    if (result == VMCI_SUCCESS &&
        ((consumerTail && *consumerTail >= qpair->consumeQSize) ||
@@ -442,23 +737,23 @@ VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceFreeSpace)
 int64
 VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int64 result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = 0;
-   } else {
-      result = VMCIQueueHeader_FreeSpace(qpair->produceQ->qHeader,
-                                         qpair->consumeQ->qHeader,
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      result = VMCIQueueHeader_FreeSpace(produceQHeader, consumeQHeader,
                                          qpair->produceQSize);
+   } else {
+      result = 0;
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    return result;
 }
@@ -489,23 +784,23 @@ VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeFreeSpace)
 int64
 VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int64 result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = 0;
-   } else {
-      result = VMCIQueueHeader_FreeSpace(qpair->consumeQ->qHeader,
-                                         qpair->produceQ->qHeader,
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      result = VMCIQueueHeader_FreeSpace(consumeQHeader, produceQHeader,
                                          qpair->consumeQSize);
+   } else {
+      result = 0;
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    return result;
 }
@@ -536,23 +831,23 @@ VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceBufReady)
 int64
 VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int64 result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = 0;
-   } else {
-      result = VMCIQueueHeader_BufReady(qpair->produceQ->qHeader,
-                                        qpair->consumeQ->qHeader,
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      result = VMCIQueueHeader_BufReady(produceQHeader, consumeQHeader,
                                         qpair->produceQSize);
+   } else {
+      result = 0;
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    return result;
 }
@@ -582,23 +877,23 @@ VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeBufReady)
 int64
 VMCIQPair_ConsumeBufReady(const VMCIQPair *qpair) // IN
 {
+   VMCIQueueHeader *produceQHeader;
+   VMCIQueueHeader *consumeQHeader;
    int64 result;
 
    if (!qpair) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
-
-   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
-      result = 0;
-   } else {
-      result = VMCIQueueHeader_BufReady(qpair->consumeQ->qHeader,
-                                        qpair->produceQ->qHeader,
+   VMCIQPairLockHeader(qpair);
+   result = VMCIQPairGetQueueHeaders(qpair, &produceQHeader, &consumeQHeader);
+   if (result == VMCI_SUCCESS) {
+      result = VMCIQueueHeader_BufReady(consumeQHeader, produceQHeader,
                                         qpair->consumeQSize);
+   } else {
+      result = 0;
    }
-
-   VMCIQPairUnlock(qpair);
+   VMCIQPairUnlockHeader(qpair);
 
    return result;
 }
@@ -630,14 +925,15 @@ VMCIQPair_ConsumeBufReady(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE ssize_t
+static ssize_t
 EnqueueLocked(VMCIQueue *produceQ,                   // IN
-              const VMCIQueue *consumeQ,             // IN
+              VMCIQueue *consumeQ,                   // IN
               const uint64 produceQSize,             // IN
               const void *buf,                       // IN
               size_t bufSize,                        // IN
               int bufType,                           // IN
-              VMCIMemcpyToQueueFunc memcpyToQueue)   // IN
+              VMCIMemcpyToQueueFunc memcpyToQueue,   // IN
+              Bool canBlock)                         // IN
 {
    int64 freeSpace;
    uint64 tail;
@@ -649,8 +945,9 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
       return (ssize_t) bufSize;
    }
 
-   if (UNLIKELY(!produceQ->qHeader || !consumeQ->qHeader)) {
-      return VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
+   result = VMCIQPairMapQueueHeaders(produceQ, consumeQ, canBlock);
+   if (UNLIKELY(result != VMCI_SUCCESS)) {
+      return result;
    }
 #endif
 
@@ -668,15 +965,17 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
    written = (size_t)(freeSpace > bufSize ? bufSize : freeSpace);
    tail = VMCIQueueHeader_ProducerTail(produceQ->qHeader);
    if (LIKELY(tail + written < produceQSize)) {
-      result = memcpyToQueue(produceQ, tail, buf, 0, written, bufType);
+      result = memcpyToQueue(produceQ, tail, buf, 0, written, bufType,
+                             canBlock);
    } else {
       /* Tail pointer wraps around. */
 
       const size_t tmp = (size_t)(produceQSize - tail);
 
-      result = memcpyToQueue(produceQ, tail, buf, 0, tmp, bufType);
+      result = memcpyToQueue(produceQ, tail, buf, 0, tmp, bufType, canBlock);
       if (result >= VMCI_SUCCESS) {
-         result = memcpyToQueue(produceQ, 0, buf, tmp, written - tmp, bufType);
+         result = memcpyToQueue(produceQ, 0, buf, tmp, written - tmp, bufType,
+                                canBlock);
       }
    }
 
@@ -712,15 +1011,16 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE ssize_t
+static ssize_t
 DequeueLocked(VMCIQueue *produceQ,                        // IN
-              const VMCIQueue *consumeQ,                  // IN
+              VMCIQueue *consumeQ,                        // IN
               const uint64 consumeQSize,                  // IN
               void *buf,                                  // IN
               size_t bufSize,                             // IN
               int bufType,                                // IN
               VMCIMemcpyFromQueueFunc memcpyFromQueue,    // IN
-              Bool updateConsumer)                        // IN
+              Bool updateConsumer,                        // IN
+              Bool canBlock)                              // IN
 {
    int64 bufReady;
    uint64 head;
@@ -728,9 +1028,9 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
    ssize_t result;
 
 #if !defined VMX86_VMX
-   if (UNLIKELY(!produceQ->qHeader ||
-                !consumeQ->qHeader)) {
-      return VMCI_ERROR_QUEUEPAIR_NODATA;
+   result = VMCIQPairMapQueueHeaders(produceQ, consumeQ, canBlock);
+   if (UNLIKELY(result != VMCI_SUCCESS)) {
+      return result;
    }
 #endif
 
@@ -747,15 +1047,16 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
    read = (size_t)(bufReady > bufSize ? bufSize : bufReady);
    head = VMCIQueueHeader_ConsumerHead(produceQ->qHeader);
    if (LIKELY(head + read < consumeQSize)) {
-      result = memcpyFromQueue(buf, 0, consumeQ, head, read, bufType);
+      result = memcpyFromQueue(buf, 0, consumeQ, head, read, bufType, canBlock);
    } else {
       /* Head pointer wraps around. */
 
       const size_t tmp = (size_t)(consumeQSize - head);
 
-      result = memcpyFromQueue(buf, 0, consumeQ, head, tmp, bufType);
+      result = memcpyFromQueue(buf, 0, consumeQ, head, tmp, bufType, canBlock);
       if (result >= VMCI_SUCCESS) {
-         result = memcpyFromQueue(buf, tmp, consumeQ, 0, read - tmp, bufType);
+         result = memcpyFromQueue(buf, tmp, consumeQ, 0, read - tmp, bufType,
+                                  canBlock);
       }
    }
 
@@ -803,13 +1104,26 @@ VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
 
-   result = EnqueueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->produceQSize,
-                          buf, bufSize, bufType,
-                          VMCIMemcpyToQueue);
+   do {
+      result = EnqueueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->produceQSize,
+                             buf, bufSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyToQueueLocal:
+                             VMCIMemcpyToQueue,
+                             !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
@@ -847,14 +1161,26 @@ VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
 
-   result = DequeueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->consumeQSize,
-                          buf, bufSize, bufType,
-                          VMCIMemcpyFromQueue,
-                          TRUE);
+   do {
+      result = DequeueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->consumeQSize,
+                             buf, bufSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyFromQueueLocal:
+                             VMCIMemcpyFromQueue,
+                             TRUE, !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
@@ -893,14 +1219,26 @@ VMCIQPair_Peek(VMCIQPair *qpair,    // IN
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
 
-   result = DequeueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->consumeQSize,
-                          buf, bufSize, bufType,
-                          VMCIMemcpyFromQueue,
-                          FALSE);
+   do {
+      result = DequeueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->consumeQSize,
+                             buf, bufSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyFromQueueLocal:
+                             VMCIMemcpyFromQueue,
+                             FALSE, !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
@@ -942,13 +1280,26 @@ VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
 
-   result = EnqueueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->produceQSize,
-                          iov, iovSize, bufType,
-                          VMCIMemcpyToQueueV);
+   do {
+      result = EnqueueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->produceQSize,
+                             iov, iovSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyToQueueVLocal:
+                             VMCIMemcpyToQueueV,
+                             !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
@@ -982,18 +1333,30 @@ VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
 {
    ssize_t result;
 
-   VMCIQPairLock(qpair);
-
    if (!qpair || !iov) {
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   result = DequeueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->consumeQSize,
-                          iov, iovSize, bufType,
-                          VMCIMemcpyFromQueueV,
-                          TRUE);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
+
+   do {
+      result = DequeueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->consumeQSize,
+                             iov, iovSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyFromQueueVLocal:
+                             VMCIMemcpyFromQueueV,
+                             TRUE, !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
@@ -1032,14 +1395,26 @@ VMCIQPair_PeekV(VMCIQPair *qpair,           // IN
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-   VMCIQPairLock(qpair);
+   result = VMCIQPairLock(qpair);
+   if (result != VMCI_SUCCESS) {
+      return result;
+   }
 
-   result = DequeueLocked(qpair->produceQ,
-                          qpair->consumeQ,
-                          qpair->consumeQSize,
-                          iov, iovSize, bufType,
-                          VMCIMemcpyFromQueueV,
-                          FALSE);
+   do {
+      result = DequeueLocked(qpair->produceQ,
+                             qpair->consumeQ,
+                             qpair->consumeQSize,
+                             iov, iovSize, bufType,
+                             qpair->flags & VMCI_QPFLAG_LOCAL?
+                             VMCIMemcpyFromQueueVLocal:
+                             VMCIMemcpyFromQueueV,
+                             FALSE, !(qpair->flags & VMCI_QPFLAG_NONBLOCK));
+      if (result == VMCI_ERROR_QUEUEPAIR_NOT_READY) {
+         if (!VMCIQPairWaitForReadyQueue(qpair)) {
+            result = VMCI_ERROR_WOULD_BLOCK;
+         }
+      }
+   } while (result == VMCI_ERROR_QUEUEPAIR_NOT_READY);
 
    VMCIQPairUnlock(qpair);
 
