@@ -80,6 +80,10 @@ sys_ioctl(unsigned int fd, unsigned int cmd, unsigned long arg);
 
 #define LGPFX "VMCI: "
 
+#define VMCI_DEVICE_NAME   "vmci"
+#define VMCI_MODULE_NAME   "vmci"
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -121,7 +125,7 @@ static int vmci_probe_device(struct pci_dev *pdev,
 static void vmci_remove_device(struct pci_dev* pdev);
 
 static struct pci_driver vmci_driver = {
-   .name     = "vmci",
+   .name     = VMCI_DEVICE_NAME,
    .id_table = vmci_ids,
    .probe = vmci_probe_device,
    .remove = __devexit_p(vmci_remove_device),
@@ -146,11 +150,22 @@ static void process_bitmap(unsigned long data);
 #  define VMCI_DISABLE_MSIX   1
 #endif
 
+/*
+ * Linux kernel < 2.6.31 takes 'int' for 'bool' module parameters.
+ * Linux kernel >= 3.3.0 takes 'bool' for 'bool' module parameters.
+ * Kernels between the two take either.  So flip switch at 3.0.0.
+ */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 0)
+   typedef bool compat_bool;
+#else
+   typedef int compat_bool;
+#endif
+
 static vmci_device vmci_dev;
-static int vmci_disable_host = 0;
-static int vmci_disable_guest = 0;
-static int vmci_disable_msi;
-static int vmci_disable_msix = VMCI_DISABLE_MSIX;
+static compat_bool vmci_disable_host = 0;
+static compat_bool vmci_disable_guest = 0;
+static compat_bool vmci_disable_msi;
+static compat_bool vmci_disable_msix = VMCI_DISABLE_MSIX;
 
 DECLARE_TASKLET(vmci_dg_tasklet, dispatch_datagrams,
                 (unsigned long)&vmci_dev);
@@ -206,16 +221,11 @@ typedef struct VMCILinux {
 #define LINUXLOG_BUFFER_SIZE  1024
 
 typedef struct VMCILinuxState {
-   int major;
-   int minor;
    struct miscdevice misc;
-   char deviceName[VM_DEVICE_NAME_SIZE];
    char buf[LINUXLOG_BUFFER_SIZE];
    atomic_t activeContexts;
 } VMCILinuxState;
 
-static struct VMCILinuxState linuxState;
-
 static int VMCISetupNotify(VMCIContext *context, VA notifyUVA);
 static void VMCIUnsetNotifyInt(VMCIContext *context, Bool useLock);
 
@@ -241,7 +251,30 @@ static unsigned int LinuxDriverPoll(struct file *file, poll_table *wait);
 #define LinuxDriverUnlockIoctlPerFD(mutex) do {} while (0)
 #endif
 
-static struct file_operations vmuser_fops;
+/* should be const if not for older kernels support */
+static struct file_operations vmuser_fops = {
+   .owner            = THIS_MODULE,
+   .open             = LinuxDriver_Open,
+   .release          = LinuxDriver_Close,
+   .poll             = LinuxDriverPoll,
+#ifdef HAVE_UNLOCKED_IOCTL
+   .unlocked_ioctl   = LinuxDriver_UnlockedIoctl,
+#else
+   .ioctl            = LinuxDriver_Ioctl,
+#endif
+#ifdef HAVE_COMPAT_IOCTL
+   .compat_ioctl     = LinuxDriver_UnlockedIoctl,
+#endif
+};
+
+static struct VMCILinuxState linuxState = {
+   .misc             = {
+      .name          = VMCI_DEVICE_NAME,
+      .minor         = MISC_DYNAMIC_MINOR,
+      .fops          = &vmuser_fops,
+   },
+   .activeContexts   = ATOMIC_INIT(0),
+};
 
 
 /*
@@ -369,65 +402,40 @@ unregister_ioctl32_handlers(void)
  *-----------------------------------------------------------------------------
  */
 
-int
+static int
 vmci_host_init(void)
 {
-   int retval;
+   int error;
 
    if (VMCI_HostInit() < VMCI_SUCCESS) {
       return -ENOMEM;
    }
 
-   /*
-    * Initialize the file_operations structure. Because this code is always
-    * compiled as a module, this is fine to do it here and not in a static
-    * initializer.
-    */
-
-   memset(&vmuser_fops, 0, sizeof vmuser_fops);
-   vmuser_fops.owner = THIS_MODULE;
-   vmuser_fops.poll = LinuxDriverPoll;
-#ifdef HAVE_UNLOCKED_IOCTL
-   vmuser_fops.unlocked_ioctl = LinuxDriver_UnlockedIoctl;
-#else
-   vmuser_fops.ioctl = LinuxDriver_Ioctl;
-#endif
-#ifdef HAVE_COMPAT_IOCTL
-   vmuser_fops.compat_ioctl = LinuxDriver_UnlockedIoctl;
-#endif
-   vmuser_fops.open = LinuxDriver_Open;
-   vmuser_fops.release = LinuxDriver_Close;
-
-   sprintf(linuxState.deviceName, "vmci");
-   linuxState.major = 10;
-   linuxState.misc.minor = MISC_DYNAMIC_MINOR;
-   linuxState.misc.name = linuxState.deviceName;
-   linuxState.misc.fops = &vmuser_fops;
-   atomic_set(&linuxState.activeContexts, 0);
-
-   retval = misc_register(&linuxState.misc);
-
-   if (retval) {
-      Warning(LGPFX"Module registration error "
-              "(name=%s,major=%d,minor=%d,err=%d).\n",
-	      linuxState.deviceName, -retval, linuxState.major,
-              linuxState.minor);
-      goto exit;
-   }
-   linuxState.minor = linuxState.misc.minor;
-   Log(LGPFX"Module registered (name=%s,major=%d,minor=%d).\n",
-       linuxState.deviceName, linuxState.major, linuxState.minor);
-
-   retval = register_ioctl32_handlers();
-   if (retval) {
-      misc_deregister(&linuxState.misc);
+   error = misc_register(&linuxState.misc);
+   if (error) {
+      Warning(LGPFX "Module registration error "
+              "(name=%s, major=%d, minor=%d, err=%d).\n",
+              linuxState.misc.name, MISC_MAJOR, linuxState.misc.minor,
+              error);
+      goto err_host_cleanup;
    }
 
-exit:
-   if (retval) {
-      VMCI_HostCleanup();
+   error = register_ioctl32_handlers();
+   if (error) {
+      Warning(LGPFX "Failed to register ioctl32 handlers, err: %d\n", error);
+      goto err_misc_unregister;
    }
-   return retval;
+
+   Log(LGPFX "Module registered (name=%s, major=%d, minor=%d).\n",
+       linuxState.misc.name, MISC_MAJOR, linuxState.misc.minor);
+
+   return 0;
+
+err_misc_unregister:
+   misc_deregister(&linuxState.misc);
+err_host_cleanup:
+   VMCI_HostCleanup();
+   return error;
 }
 
 
@@ -524,7 +532,7 @@ LinuxDriver_Close(struct inode *inode, // IN
 
 static unsigned int
 LinuxDriverPoll(struct file *filp,
-		poll_table *wait)
+                poll_table *wait)
 {
    VMCILockFlags flags;
    VMCILinux *vmciLinux = (VMCILinux *) filp->private_data;
@@ -594,6 +602,58 @@ VMCICopyHandleArrayToUser(void *userBufUVA,             // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIDoQPBrokerAlloc --
+ *
+ *      Helper function for creating queue pair and copying the result
+ *      to user memory.
+ *
+ * Results:
+ *      0 if result value was copied to user memory, -EFAULT otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+VMCIDoQPBrokerAlloc(VMCIHandle handle,
+                    VMCIId peer,
+                    uint32 flags,
+                    uint64 produceSize,
+                    uint64 consumeSize,
+                    QueuePairPageStore *pageStore,
+                    VMCIContext *context,
+                    Bool vmToVm,
+                    void *resultUVA)
+{
+   VMCIId cid;
+   int result;
+   int retval;
+
+   cid = VMCIContext_GetId(context);
+
+   result = VMCIQPBroker_Alloc(handle, peer, flags, VMCI_NO_PRIVILEGE_FLAGS,
+                               produceSize, consumeSize, pageStore, context);
+   if (result == VMCI_SUCCESS && vmToVm) {
+      result = VMCI_SUCCESS_QUEUEPAIR_CREATE;
+   }
+   retval = copy_to_user(resultUVA, &result, sizeof result);
+   if (retval) {
+      retval = -EFAULT;
+      if (result >= VMCI_SUCCESS) {
+         result = VMCIQPBroker_Detach(handle, context);
+         ASSERT(result >= VMCI_SUCCESS);
+      }
+   }
+
+   return retval;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -694,11 +754,11 @@ LinuxDriver_Ioctl(struct inode *inode,
       initBlock.cid = VMCIContext_GetId(vmciLinux->context);
       retval = copy_to_user((void *)ioarg, &initBlock, sizeof initBlock);
       if (retval != 0) {
-	 VMCIContext_ReleaseContext(vmciLinux->context);
-	 vmciLinux->context = NULL;
-	 Log(LGPFX"Error writing init block.\n");
-	 retval = -EFAULT;
-	 goto init_release;
+         VMCIContext_ReleaseContext(vmciLinux->context);
+         vmciLinux->context = NULL;
+         Log(LGPFX"Error writing init block.\n");
+         retval = -EFAULT;
+         goto init_release;
       }
       ASSERT(initBlock.cid != VMCI_INVALID_ID);
 
@@ -731,14 +791,14 @@ LinuxDriver_Ioctl(struct inode *inode,
 
       if (sendInfo.len > VMCI_MAX_DG_SIZE) {
          Warning(LGPFX"Datagram too big (size=%d).\n", sendInfo.len);
-	 retval = -EINVAL;
-	 break;
+         retval = -EINVAL;
+         break;
       }
 
       if (sendInfo.len < sizeof *dg) {
          Warning(LGPFX"Datagram too small (size=%d).\n", sendInfo.len);
-	 retval = -EINVAL;
-	 break;
+         retval = -EINVAL;
+         break;
       }
 
       dg = VMCI_AllocKernelMem(sendInfo.len, VMCI_MEMORY_NORMAL);
@@ -799,70 +859,115 @@ LinuxDriver_Ioctl(struct inode *inode,
                                                     &size, &dg);
 
       if (recvInfo.result >= VMCI_SUCCESS) {
-	 ASSERT(dg);
-	 retval = copy_to_user((void *) ((uintptr_t) recvInfo.addr), dg,
-			       VMCI_DG_SIZE(dg));
-	 VMCI_FreeKernelMem(dg, VMCI_DG_SIZE(dg));
-	 if (retval != 0) {
-	    break;
-	 }
+         ASSERT(dg);
+         retval = copy_to_user((void *) ((uintptr_t) recvInfo.addr), dg,
+                               VMCI_DG_SIZE(dg));
+         VMCI_FreeKernelMem(dg, VMCI_DG_SIZE(dg));
+         if (retval != 0) {
+            break;
+         }
       }
       retval = copy_to_user((void *)ioarg, &recvInfo, sizeof recvInfo);
       break;
    }
 
    case IOCTL_VMCI_QUEUEPAIR_ALLOC: {
-      VMCIQueuePairAllocInfo queuePairAllocInfo;
-      VMCIQueuePairAllocInfo *info = (VMCIQueuePairAllocInfo *)ioarg;
-      int32 result;
-      VMCIId cid;
-
       if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
          Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_ALLOC only valid for contexts.\n");
          retval = -EINVAL;
          break;
       }
 
-      retval = copy_from_user(&queuePairAllocInfo, (void *)ioarg,
-                              sizeof queuePairAllocInfo);
+      if (vmciLinux->userVersion < VMCI_VERSION_NOVMVM) {
+         VMCIQueuePairAllocInfo_VMToVM queuePairAllocInfo;
+         VMCIQueuePairAllocInfo_VMToVM *info = (VMCIQueuePairAllocInfo_VMToVM *)ioarg;
+
+         retval = copy_from_user(&queuePairAllocInfo, (void *)ioarg,
+                                 sizeof queuePairAllocInfo);
+         if (retval) {
+            retval = -EFAULT;
+            break;
+         }
+
+         retval = VMCIDoQPBrokerAlloc(queuePairAllocInfo.handle,
+                                      queuePairAllocInfo.peer,
+                                      queuePairAllocInfo.flags,
+                                      queuePairAllocInfo.produceSize,
+                                      queuePairAllocInfo.consumeSize,
+                                      NULL,
+                                      vmciLinux->context,
+                                      TRUE, // VM to VM style create
+                                      &info->result);
+      } else {
+         VMCIQueuePairAllocInfo queuePairAllocInfo;
+         VMCIQueuePairAllocInfo *info = (VMCIQueuePairAllocInfo *)ioarg;
+         QueuePairPageStore pageStore;
+
+         retval = copy_from_user(&queuePairAllocInfo, (void *)ioarg,
+                                 sizeof queuePairAllocInfo);
+         if (retval) {
+            retval = -EFAULT;
+            break;
+         }
+
+         pageStore.pages = queuePairAllocInfo.ppnVA;
+         pageStore.len = queuePairAllocInfo.numPPNs;
+
+         retval = VMCIDoQPBrokerAlloc(queuePairAllocInfo.handle,
+                                      queuePairAllocInfo.peer,
+                                      queuePairAllocInfo.flags,
+                                      queuePairAllocInfo.produceSize,
+                                      queuePairAllocInfo.consumeSize,
+                                      &pageStore,
+                                      vmciLinux->context,
+                                      FALSE, // Not VM to VM style create
+                                      &info->result);
+      }
+      break;
+   }
+
+   case IOCTL_VMCI_QUEUEPAIR_SETVA: {
+      VMCIQueuePairSetVAInfo setVAInfo;
+      VMCIQueuePairSetVAInfo *info = (VMCIQueuePairSetVAInfo *)ioarg;
+      int32 result;
+
+      if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
+         Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_SETVA only valid for contexts.\n");
+         retval = -EINVAL;
+         break;
+      }
+
+      if (vmciLinux->userVersion < VMCI_VERSION_NOVMVM) {
+         Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_SETVA not supported for this VMX version.\n");
+         retval = -EINVAL;
+         break;
+      }
+
+      retval = copy_from_user(&setVAInfo, (void *)ioarg, sizeof setVAInfo);
       if (retval) {
          retval = -EFAULT;
          break;
       }
 
-      cid = VMCIContext_GetId(vmciLinux->context);
-      VMCIQPBroker_Lock();
+      if (setVAInfo.va) {
+         /*
+          * VMX is passing down a new VA for the queue pair mapping.
+          */
 
-      {
-	 QueuePairPageStore pageStore = { TRUE,
-					  queuePairAllocInfo.producePageFile,
-					  queuePairAllocInfo.consumePageFile,
-					  queuePairAllocInfo.producePageFileSize,
-					  queuePairAllocInfo.consumePageFileSize,
-					  0,
-					  0 };
+         result = VMCIQPBroker_Map(setVAInfo.handle, vmciLinux->context, setVAInfo.va);
+      } else {
+         /*
+          * The queue pair is about to be unmapped by the VMX.
+          */
 
-	 result = VMCIQPBroker_Alloc(queuePairAllocInfo.handle,
-                                     queuePairAllocInfo.peer,
-                                     queuePairAllocInfo.flags,
-                                     VMCI_NO_PRIVILEGE_FLAGS,
-                                     queuePairAllocInfo.produceSize,
-                                     queuePairAllocInfo.consumeSize,
-                                     &pageStore,
-                                     vmciLinux->context);
+         result = VMCIQPBroker_Unmap(setVAInfo.handle, vmciLinux->context, 0);
       }
-      Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_ALLOC (cid=%u,result=%d).\n", cid, result);
+
       retval = copy_to_user(&info->result, &result, sizeof result);
       if (retval) {
          retval = -EFAULT;
-         if (result >= VMCI_SUCCESS) {
-            result = VMCIQPBroker_Detach(queuePairAllocInfo.handle,
-                                         vmciLinux->context, TRUE);
-            ASSERT(result >= VMCI_SUCCESS);
-         }
       }
 
-      VMCIQPBroker_Unlock();
       break;
    }
 
@@ -870,9 +975,14 @@ LinuxDriver_Ioctl(struct inode *inode,
       VMCIQueuePairPageFileInfo pageFileInfo;
       VMCIQueuePairPageFileInfo *info = (VMCIQueuePairPageFileInfo *)ioarg;
       int32 result;
-      VMCIId cid;
-      Bool useUVA;
-      int size;
+
+      if (vmciLinux->userVersion < VMCI_VERSION_HOSTQP ||
+          vmciLinux->userVersion >= VMCI_VERSION_NOVMVM) {
+         Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_SETPAGEFILE not supported this VMX "
+             "(version=%d).\n", vmciLinux->userVersion);
+         retval = -EINVAL;
+         break;
+      }
 
       if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
          Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_SETPAGEFILE only valid for contexts.\n");
@@ -880,20 +990,7 @@ LinuxDriver_Ioctl(struct inode *inode,
          break;
       }
 
-      if (VMCIContext_SupportsHostQP(vmciLinux->context)) {
-         useUVA = TRUE;
-         size = sizeof *info;
-      } else {
-         /*
-          * An older VMX version won't supply the UVA of the page
-          * files backing the queue pair contents (and headers)
-          */
-
-         useUVA = FALSE;
-         size = sizeof(VMCIQueuePairPageFileInfo_NoHostQP);
-      }
-
-      retval = copy_from_user(&pageFileInfo, (void *)ioarg, size);
+      retval = copy_from_user(&pageFileInfo, (void *)ioarg, sizeof *info);
       if (retval) {
          retval = -EFAULT;
          break;
@@ -919,27 +1016,11 @@ LinuxDriver_Ioctl(struct inode *inode,
       result = VMCI_SUCCESS;
       retval = copy_to_user(&info->result, &result, sizeof result);
       if (retval == 0) {
-         cid = VMCIContext_GetId(vmciLinux->context);
-         VMCIQPBroker_Lock();
-
-         {
-            QueuePairPageStore pageStore = { TRUE,
-                                             pageFileInfo.producePageFile,
-                                             pageFileInfo.consumePageFile,
-                                             pageFileInfo.producePageFileSize,
-                                             pageFileInfo.consumePageFileSize,
-                                             useUVA ? pageFileInfo.produceVA : 0,
-                                             useUVA ? pageFileInfo.consumeVA : 0 };
-
-            result = VMCIQPBroker_SetPageStore(pageFileInfo.handle,
-                                               &pageStore,
-                                               vmciLinux->context);
-         }
-         VMCIQPBroker_Unlock();
-
+         result = VMCIQPBroker_SetPageStore(pageFileInfo.handle,
+                                            pageFileInfo.produceVA,
+                                            pageFileInfo.consumeVA,
+                                            vmciLinux->context);
          if (result < VMCI_SUCCESS) {
-            Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_SETPAGEFILE (cid=%u,result=%d).\n",
-                cid, result);
 
             retval = copy_to_user(&info->result, &result, sizeof result);
             if (retval != 0) {
@@ -978,7 +1059,6 @@ LinuxDriver_Ioctl(struct inode *inode,
       VMCIQueuePairDetachInfo detachInfo;
       VMCIQueuePairDetachInfo *info = (VMCIQueuePairDetachInfo *)ioarg;
       int32 result;
-      VMCIId cid;
 
       if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
          Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_DETACH only valid for contexts.\n");
@@ -992,91 +1072,17 @@ LinuxDriver_Ioctl(struct inode *inode,
          break;
       }
 
-      cid = VMCIContext_GetId(vmciLinux->context);
-      VMCIQPBroker_Lock();
-      result = VMCIQPBroker_Detach(detachInfo.handle, vmciLinux->context,
-                                FALSE); /* Probe detach operation. */
-      Log(LGPFX"IOCTL_VMCI_QUEUEPAIR_DETACH (cid=%u,result=%d).\n",
-          cid, result);
+      result = VMCIQPBroker_Detach(detachInfo.handle, vmciLinux->context);
+      if (result == VMCI_SUCCESS &&
+          vmciLinux->userVersion < VMCI_VERSION_NOVMVM) {
+         result = VMCI_SUCCESS_LAST_DETACH;
+      }
 
       retval = copy_to_user(&info->result, &result, sizeof result);
       if (retval) {
-         /* Could not copy to userland, don't perform the actual detach. */
          retval = -EFAULT;
-      } else {
-         if (result >= VMCI_SUCCESS) {
-            /* Now perform the actual detach. */
-            int32 result2 = VMCIQPBroker_Detach(detachInfo.handle,
-                                             vmciLinux->context, TRUE);
-            if (UNLIKELY(result != result2)) {
-               /*
-                * This should never happen.  But it's better to log a warning
-                * than to crash the host.
-                */
-               Warning(LGPFX"VMCIQPBroker_Detach returned different results:  "
-                       "(previous=%d,current=%d).\n", result, result2);
-            }
-         }
       }
 
-      VMCIQPBroker_Unlock();
-      break;
-   }
-
-   case IOCTL_VMCI_DATAGRAM_REQUEST_MAP: {
-      VMCIDatagramMapInfo mapInfo;
-      VMCIDatagramMapInfo *info = (VMCIDatagramMapInfo *)ioarg;
-      int32 result;
-      VMCIId cid;
-
-      if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
-         Log(LGPFX"IOCTL_VMCI_REQUEST_MAP only valid for contexts.\n");
-         retval = -EINVAL;
-         break;
-      }
-
-      retval = copy_from_user(&mapInfo, (void *)ioarg, sizeof mapInfo);
-      if (retval) {
-         retval = -EFAULT;
-         break;
-      }
-
-      cid = VMCIContext_GetId(vmciLinux->context);
-      result = VMCIDatagramRequestWellKnownMap(mapInfo.wellKnownID, cid,
-					       VMCIContext_GetPrivFlags(cid));
-      retval = copy_to_user(&info->result, &result, sizeof result);
-      if (retval) {
-         retval = -EFAULT;
-         break;
-      }
-      break;
-   }
-
-   case IOCTL_VMCI_DATAGRAM_REMOVE_MAP: {
-      VMCIDatagramMapInfo mapInfo;
-      VMCIDatagramMapInfo *info = (VMCIDatagramMapInfo *)ioarg;
-      int32 result;
-      VMCIId cid;
-
-      if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
-         Log(LGPFX"IOCTL_VMCI_REMOVE_MAP only valid for contexts.\n");
-         retval = -EINVAL;
-         break;
-      }
-
-      retval = copy_from_user(&mapInfo, (void *)ioarg, sizeof mapInfo);
-      if (retval) {
-         retval = -EFAULT;
-         break;
-      }
-
-      cid = VMCIContext_GetId(vmciLinux->context);
-      result = VMCIDatagramRemoveWellKnownMap(mapInfo.wellKnownID, cid);
-      retval = copy_to_user(&info->result, &result, sizeof result);
-      if (retval) {
-         retval = -EFAULT;
-         break;
-      }
       break;
    }
 
@@ -1116,7 +1122,7 @@ LinuxDriver_Ioctl(struct inode *inode,
 
       if (vmciLinux->ctType != VMCIOBJ_CONTEXT) {
          Log(LGPFX"IOCTL_VMCI_CTX_REMOVE_NOTIFICATION only valid for "
-	     "contexts.\n");
+             "contexts.\n");
          retval = -EINVAL;
          break;
       }
@@ -1156,16 +1162,16 @@ LinuxDriver_Ioctl(struct inode *inode,
 
       cid = VMCIContext_GetId(vmciLinux->context);
       getInfo.result = VMCIContext_GetCheckpointState(cid, getInfo.cptType,
-						      &getInfo.bufSize,
-						      &cptBuf);
+                                                      &getInfo.bufSize,
+                                                      &cptBuf);
       if (getInfo.result == VMCI_SUCCESS && getInfo.bufSize) {
-	 retval = copy_to_user((void *)(VA)getInfo.cptBuf, cptBuf,
-			       getInfo.bufSize);
-	 VMCI_FreeKernelMem(cptBuf, getInfo.bufSize);
-	 if (retval) {
-	    retval = -EFAULT;
-	    break;
-	 }
+         retval = copy_to_user((void *)(VA)getInfo.cptBuf, cptBuf,
+                               getInfo.bufSize);
+         VMCI_FreeKernelMem(cptBuf, getInfo.bufSize);
+         if (retval) {
+            retval = -EFAULT;
+            break;
+         }
       }
       retval = copy_to_user((void *)ioarg, &getInfo, sizeof getInfo);
       if (retval) {
@@ -1195,21 +1201,21 @@ LinuxDriver_Ioctl(struct inode *inode,
       cptBuf = VMCI_AllocKernelMem(setInfo.bufSize, VMCI_MEMORY_NORMAL);
       if (cptBuf == NULL) {
          Log(LGPFX"Cannot allocate memory to set cpt state (type=%d).\n",
-	     setInfo.cptType);
+             setInfo.cptType);
          retval = -ENOMEM;
          break;
       }
       retval = copy_from_user(cptBuf, (void *)(VA)setInfo.cptBuf,
-			      setInfo.bufSize);
+                              setInfo.bufSize);
       if (retval) {
-	 VMCI_FreeKernelMem(cptBuf, setInfo.bufSize);
+         VMCI_FreeKernelMem(cptBuf, setInfo.bufSize);
          retval = -EFAULT;
          break;
       }
 
       cid = VMCIContext_GetId(vmciLinux->context);
       setInfo.result = VMCIContext_SetCheckpointState(cid, setInfo.cptType,
-						      setInfo.bufSize, cptBuf);
+                                                      setInfo.bufSize, cptBuf);
       VMCI_FreeKernelMem(cptBuf, setInfo.bufSize);
       retval = copy_to_user((void *)ioarg, &setInfo, sizeof setInfo);
       if (retval) {
@@ -2254,7 +2260,7 @@ dispatch_datagrams(unsigned long data)
 
    if (dev == NULL) {
       printk(KERN_DEBUG "vmci: dispatch_datagrams(): no vmci device"
-	     "present.\n");
+             "present.\n");
       return;
    }
 
@@ -2265,7 +2271,7 @@ dispatch_datagrams(unsigned long data)
 
 
    VMCI_ReadDatagramsFromPort((VMCIIoHandle) 0, dev->ioaddr + VMCI_DATA_IN_ADDR,
-			      data_buffer, data_buffer_size);
+                              data_buffer, data_buffer_size);
 }
 
 
@@ -2293,7 +2299,7 @@ process_bitmap(unsigned long data)
 
    if (dev == NULL) {
       printk(KERN_DEBUG "vmci: process_bitmaps(): no vmci device"
-	     "present.\n");
+             "present.\n");
       return;
    }
 
@@ -2411,29 +2417,25 @@ vmci_init(void)
       return -ENOMEM;
    }
 
-   if (vmci_disable_guest) {
-      guestDeviceInit = 0;
-   } else {
+   if (!vmci_disable_guest) {
       retval = vmci_guest_init();
       if (retval != 0) {
          Warning(LGPFX"VMCI PCI device not initialized (err=%d).\n", retval);
-      }
-      guestDeviceInit = (retval == 0);
-      if (VMCI_GuestPersonalityActive()) {
-         Log(LGPFX"Using guest personality\n");
+      } else {
+         guestDeviceInit = TRUE;
+         if (VMCI_GuestPersonalityActive()) {
+            Log(LGPFX"Using guest personality\n");
+         }
       }
    }
 
-   if (vmci_disable_host) {
-      hostDeviceInit = 0;
-   } else {
+   if (!vmci_disable_host) {
       retval = vmci_host_init();
       if (retval != 0) {
          Warning(LGPFX"Unable to initialize host personality (err=%d).\n",
                  retval);
-      }
-      hostDeviceInit = (retval == 0);
-      if (hostDeviceInit) {
+      } else {
+         hostDeviceInit = TRUE;
          Log(LGPFX"Using host personality\n");
       }
    }
@@ -2443,7 +2445,7 @@ vmci_init(void)
       return -ENODEV;
    }
 
-   Log(LGPFX"Module (name=%s) is initialized\n", linuxState.deviceName);
+   Log(LGPFX"Module (name=%s) is initialized\n", VMCI_MODULE_NAME);
 
    return 0;
 }
@@ -2478,10 +2480,9 @@ vmci_exit(void)
 
       retval = misc_deregister(&linuxState.misc);
       if (retval) {
-         Warning(LGPFX"Module %s: error unregistering\n",
-                 linuxState.deviceName);
+         Warning(LGPFX "Module %s: error unregistering\n", VMCI_MODULE_NAME);
       } else {
-         Log(LGPFX"Module %s: unloaded\n", linuxState.deviceName);
+         Log(LGPFX"Module %s: unloaded\n", VMCI_MODULE_NAME);
       }
 
       hostDeviceInit = FALSE;
@@ -2506,7 +2507,7 @@ MODULE_PARM_DESC(disable_msi, "Disable MSI use in driver - (default=0)");
 
 module_param_named(disable_msix, vmci_disable_msix, bool, 0);
 MODULE_PARM_DESC(disable_msix, "Disable MSI-X use in driver - (default="
-		 __stringify(VMCI_DISABLE_MSIX) ")");
+                 __stringify(VMCI_DISABLE_MSIX) ")");
 
 MODULE_AUTHOR("VMware, Inc.");
 MODULE_DESCRIPTION("VMware Virtual Machine Communication Interface (VMCI).");
