@@ -50,7 +50,7 @@ static int HgfsPackOpenRequest(struct inode *inode,
 static int HgfsUnpackOpenReply(HgfsReq *req,
                                HgfsOp opUsed,
                                HgfsHandle *file,
-                               HgfsServerLock *lock);
+                               HgfsLockType *lock);
 static int HgfsGetOpenFlags(uint32 flags);
 
 /* HGFS file operations for files. */
@@ -83,7 +83,7 @@ static loff_t HgfsSeek(struct file *file,
 static int HgfsFsync(struct file *file,
 #if defined VMW_FSYNC_OLD
                      struct dentry *dentry,
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+#elif defined VMW_FSYNC_31
                      loff_t start,
                      loff_t end,
 #endif
@@ -374,7 +374,7 @@ static int
 HgfsUnpackOpenReply(HgfsReq *req,          // IN: Packet with reply inside
                     HgfsOp opUsed,         // IN: What request op did we send
                     HgfsHandle *file,      // OUT: Handle in reply packet
-                    HgfsServerLock *lock)  // OUT: The server lock we got
+                    HgfsLockType *lock)    // OUT: The server lock we got
 {
    HgfsReplyOpenV3 *replyV3;
    HgfsReplyOpenV2 *replyV2;
@@ -547,7 +547,7 @@ HgfsOpen(struct inode *inode,  // IN: Inode of the file to open
    HgfsOp opUsed;
    HgfsStatus replyStatus;
    HgfsHandle replyFile;
-   HgfsServerLock replyLock;
+   HgfsLockType replyLock;
    HgfsInodeInfo *iinfo;
    int result = 0;
 
@@ -992,7 +992,7 @@ static int
 HgfsFsync(struct file *file,		// IN: File we operate on
 #if defined VMW_FSYNC_OLD
           struct dentry *dentry,        // IN: Dentry for this file
-#elif LINUX_VERSION_CODE >= KERNEL_VERSION(3, 1, 0)
+#elif defined VMW_FSYNC_31
           loff_t start,                 // IN: start of range to sync
           loff_t end,                   // IN: end of range to sync
 #endif
