@@ -93,6 +93,9 @@ VSockVmci_ErrorToVSockError(int32 vmciError) // IN
    case VMCI_ERROR_INVALID_RESOURCE:
       err = EHOSTUNREACH;
       break;
+   case VMCI_ERROR_MODULE_NOT_LOADED:
+      err = ESYSNOTREADY;
+      break;
    case VMCI_ERROR_INVALID_ARGS:
    default:
       err = EINVAL;
