@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2012,2014 VMware, Inc. All rights reserved.
+ * Copyright (C) 2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,6 +26,9 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of VMware Inc. nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission of VMware Inc.
  *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -64,11 +67,12 @@
 #define BALLOON_NAME                    "vmmemctl"
 #define BALLOON_NAME_VERBOSE            "VMware memory control driver"
 
-#if defined __linux__ || (defined __APPLE__ && defined __LP64__)
-#define BALLOON_CAPABILITIES    (BALLOON_BASIC_CMDS|BALLOON_BATCHED_CMDS|\
-                                 BALLOON_BATCHED_2M_CMDS)
-#elif defined __FreeBSD__ || defined _WIN32 || (defined __APPLE__ && !defined __LP64__)
-#define BALLOON_CAPABILITIES    (BALLOON_BASIC_CMDS|BALLOON_BATCHED_CMDS)
+#if defined __linux__ || defined __FreeBSD__ || defined _WIN32
+/*
+ * FIXME: Even if the driver support batched commands keep using the
+ * non-batched one until more testing has been done.
+ */
+#define BALLOON_CAPABILITIES    BALLOON_BASIC_CMDS
 #else
 #define BALLOON_CAPABILITIES    BALLOON_BASIC_CMDS
 #endif
@@ -107,7 +111,7 @@
 #define	STATS_DEC(stat)
 #endif
 
-#define PPN_2_PA(_ppn)  ((PPN64)(_ppn) << PAGE_SHIFT)
+#define PPN_2_PA(_ppn)  ((_ppn) << PAGE_SHIFT)
 #define PA_2_PPN(_pa)   ((_pa) >> PAGE_SHIFT)
 
 #endif /* !BALLOONINT_H_ */
