@@ -74,37 +74,32 @@
  * Types
  */
 
-typedef void OSTimerHandler(void *clientData);
-typedef int  OSStatusHandler(char *buf, size_t size);
 typedef uintptr_t PageHandle;
+typedef uintptr_t Mapping;
 
-#define PAGE_HANDLE_INVALID 0
+#define PAGE_HANDLE_INVALID     0
+#define MAPPING_INVALID         0
 
 /*
  * Operations
  */
 
-extern Bool OS_Init(const char *name,
-                    const char *nameVerbose,
-                    OSStatusHandler *handler);
-extern void OS_Cleanup(void);
-extern BalloonGuest OS_Identity(void);
-
 extern void OS_MemZero(void *ptr, size_t size);
 extern void OS_MemCopy(void *dest, const void *src, size_t size);
-extern int  OS_Snprintf(char *buf, size_t size, const char *format, ...);
 
 extern void *OS_Malloc(size_t size);
 extern void OS_Free(void *ptr, size_t size);
 
 extern void OS_Yield(void);
 
-extern Bool OS_TimerStart(OSTimerHandler *handler, void *clientData);
-extern void OS_TimerStop(void);
-
 extern unsigned long OS_ReservedPageGetLimit(void);
-extern unsigned long OS_ReservedPageGetPPN(PageHandle handle);
+extern PA64          OS_ReservedPageGetPA(PageHandle handle);
+extern PageHandle    OS_ReservedPageGetHandle(PA64 pa);
 extern PageHandle    OS_ReservedPageAlloc(int canSleep);
 extern void          OS_ReservedPageFree(PageHandle handle);
 
+extern Mapping       OS_MapPageHandle(PageHandle handle);
+extern void          *OS_Mapping2Addr(Mapping mapping);
+extern void          OS_UnmapPage(Mapping mapping);
+
 #endif  /* OS_H */
