@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -48,9 +48,7 @@
 #define WIN_DIRSEPC     '\\'
 #define WIN_DIRSEPS     "\\"
 
-
-#ifndef DND_IS_XDG
-static const char *DnDCreateRootStagingDirectory(void);
+static ConstUnicode DnDCreateRootStagingDirectory(void);
 
 /*
  *-----------------------------------------------------------------------------
@@ -80,14 +78,14 @@ static const char *DnDCreateRootStagingDirectory(void);
  *-----------------------------------------------------------------------------
  */
 
-char *
+Unicode
 DnD_CreateStagingDirectory(void)
 {
-   const char *root;
-   char **stagingDirList;
+   ConstUnicode root;
+   Unicode *stagingDirList;
    int numStagingDirs;
    int i;
-   char *ret = NULL;
+   Unicode ret = NULL;
    Bool found = FALSE;
 
    /*
@@ -107,7 +105,7 @@ DnD_CreateStagingDirectory(void)
 
    for (i = 0; i < numStagingDirs; i++) {
       if (!found) {
-         char *stagingDir;
+         Unicode stagingDir;
 
          stagingDir = Unicode_Append(root, stagingDirList[i]);
 
@@ -121,11 +119,11 @@ DnD_CreateStagingDirectory(void)
                found = TRUE;
          }
 
-         free(stagingDir);
+         Unicode_Free(stagingDir);
       }
    }
 
-   Util_FreeStringList(stagingDirList, numStagingDirs);
+   Unicode_FreeList(stagingDirList, numStagingDirs);
 
    /* Only create a directory if we didn't find one above. */
    if (!found) {
@@ -134,14 +132,14 @@ DnD_CreateStagingDirectory(void)
       context = Random_QuickSeed((unsigned)time(NULL));
 
       for (i = 0; i < 10; i++) {
-         char *temp;
+         Unicode temp;
 
          /* Each staging directory is given a random name. */
-         free(ret);
+         Unicode_Free(ret);
          temp = Unicode_Format("%08x%c", Random_Quick(context), DIRSEPC);
-         VERIFY(temp);
+         ASSERT_MEM_ALLOC(temp);
          ret = Unicode_Append(root, temp);
-         free(temp);
+         Unicode_Free(temp);
 
          if (File_CreateDirectory(ret) &&
              DnDSetPermissionsOnStagingDir(ret)) {
@@ -155,13 +153,12 @@ DnD_CreateStagingDirectory(void)
 
 exit:
    if (!found && ret != NULL) {
-      free(ret);
+      Unicode_Free(ret);
       ret = NULL;
    }
 
    return ret;
 }
-#endif // ifndef DND_IS_XDG
 
 
 /*
@@ -182,8 +179,8 @@ exit:
  */
 
 Bool
-DnD_DeleteStagingFiles(const char *stagingDir,  // IN:
-                       Bool onReboot)           // IN:
+DnD_DeleteStagingFiles(ConstUnicode stagingDir,  // IN:
+                       Bool onReboot)            // IN:
 {
    Bool ret = TRUE;
 
@@ -205,8 +202,8 @@ DnD_DeleteStagingFiles(const char *stagingDir,  // IN:
    } else {
       int i;
       int numFiles;
-      char *base;
-      char **fileList = NULL;
+      Unicode base;
+      Unicode *fileList = NULL;
 
       /* get list of files in current directory */
       numFiles = File_ListDirectory(stagingDir, &fileList);
@@ -219,7 +216,7 @@ DnD_DeleteStagingFiles(const char *stagingDir,  // IN:
       base = Unicode_Append(stagingDir, DIRSEPS);
 
       for (i = 0; i < numFiles; i++) {
-         char *curPath;
+         Unicode curPath;
 
          curPath = Unicode_Append(base, fileList[i]);
 
@@ -233,17 +230,15 @@ DnD_DeleteStagingFiles(const char *stagingDir,  // IN:
             }
          }
 
-         free(curPath);
+         Unicode_Free(curPath);
       }
 
-      free(base);
+      Unicode_Free(base);
    }
 
    return ret;
 }
 
-
-#ifndef DND_IS_XDG
 /*
  *----------------------------------------------------------------------------
  *
@@ -261,10 +256,10 @@ DnD_DeleteStagingFiles(const char *stagingDir,  // IN:
  *----------------------------------------------------------------------------
  */
 
-static const char *
+static ConstUnicode
 DnDCreateRootStagingDirectory(void)
 {
-   const char *root;
+   ConstUnicode root;
 
    /*
     * DnD_GetFileRoot() gives us a pointer to a static string, so there's no
@@ -294,7 +289,120 @@ DnDCreateRootStagingDirectory(void)
 
    return root;
 }
-#endif // ifndef DND_IS_XDG
+
+
+/*
+ *----------------------------------------------------------------------------
+ *
+ * DnDPrependFileRoot --
+ *
+ *    Given a buffer of '\0' delimited filenames, this prepends the file root
+ *    to each one and uses delimiter for delimiting the output buffer.  The
+ *    buffer pointed to by *src will be freed and *src will point to a new
+ *    buffer containing the results.  *srcSize is set to the size of the new
+ *    buffer, not including the NUL-terminator.
+ *
+ *    The logic here and in the called functions appears to be UTF8-safe.
+ *
+ * Results:
+ *    TRUE on success, FALSE on failure.
+ *
+ * Side effects:
+ *    *src will be freed, and a new buffer will be allocated. This buffer must
+ *    be freed by the caller.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+Bool
+DnDPrependFileRoot(ConstUnicode fileRoot,  // IN    : file root to append
+                   char delimiter,         // IN    : delimiter for output buffer
+                   char **src,             // IN/OUT: NUL-delimited list of paths
+                   size_t *srcSize)        // IN/OUT: size of list
+{
+   char *newData = NULL;
+   size_t newDataLen = 0;
+   Bool firstPass = TRUE;
+   const char *begin;
+   const char *end;
+   const char *next;
+   size_t rootLen;
+   int len;
+
+   ASSERT(fileRoot);
+   ASSERT(src);
+   ASSERT(*src);
+   ASSERT(srcSize);
+
+   rootLen = strlen(fileRoot);
+
+   /*
+    * To prevent CPName_GetComponent() errors, we set begin to the first
+    * Non-NUL character in *src, and end to the last NUL character in *src.  We
+    * assume that the components are delimited with single NUL characters; if
+    * that is not true, CPName_GetComponent() will fail.
+    */
+
+   for (begin = *src; *begin == '\0'; begin++)
+      ;
+   end = CPNameUtil_Strrchr(*src, *srcSize, '\0');
+
+   /* Get the length of this component, and a pointer to the next */
+   while ((len = CPName_GetComponent(begin, end, &next)) != 0) {
+      size_t origNewDataLen = newDataLen;
+      int escapedLen;
+
+      if (len < 0) {
+         Log("%s: error getting next component\n", __FUNCTION__);
+         if (!firstPass) {
+            free(newData);
+         }
+
+         return FALSE;
+      }
+
+      /*
+       * Append this component to our list: allocate one more for NUL on first
+       * pass and delimiter on all other passes.
+       */
+
+      escapedLen = HgfsEscape_GetSize(begin, len);
+      if (escapedLen < 0) {
+         Log("%s: error calculating buffer size\n", __FUNCTION__);
+         return FALSE;
+      } else if (0 == escapedLen) {
+         newDataLen += rootLen + len + 1;
+         newData = (char *)Util_SafeRealloc(newData, newDataLen);
+
+         if (!firstPass) {
+            ASSERT(origNewDataLen > 0);
+            newData[origNewDataLen - 1] = delimiter;
+         }
+         memcpy(newData + origNewDataLen, fileRoot, rootLen);
+         memcpy(newData + origNewDataLen + rootLen, begin, len);
+      } else {
+         newDataLen += rootLen + 1;
+         newData = (char *)Util_SafeRealloc(newData, newDataLen);
+
+         if (!firstPass) {
+            ASSERT(origNewDataLen > 0);
+            newData[origNewDataLen - 1] = delimiter;
+         }
+         memcpy(newData + origNewDataLen, fileRoot, rootLen);
+         HgfsEscape_Do(begin, len, escapedLen, newData + origNewDataLen + rootLen);
+      }
+      newData[newDataLen - 1] = '\0';
+
+      firstPass = FALSE;
+      begin = next;
+   }
+
+   free(*src);
+   *src = newData;
+   /* Not including NUL terminator */
+   *srcSize = newDataLen - 1;
+   return TRUE;
+}
 
 
 /*
@@ -365,7 +473,7 @@ DnD_LegacyConvertToCPName(const char *nameIn,   // IN:  Buffer to convert
 
    nameSize = strlen(nameIn);
    fullNameSize = partialNameLen + partialNameSuffixLen + nameSize;
-   fullName = Util_SafeMalloc(fullNameSize + 1);
+   fullName = (char *)Util_SafeMalloc(fullNameSize + 1);
 
    memcpy(fullName, partialName, partialNameLen);
    memcpy(fullName + partialNameLen, partialNameSuffix, partialNameSuffixLen);
@@ -531,7 +639,7 @@ error:
  *    Try to get last directory name from a full path name.
  *
  * Results:
- *    The allocated UTF8 string, or NULL on failure.
+ *    The allocated Unicode string, or NULL on failure.
  *
  * Side effects:
  *    None.
@@ -539,8 +647,8 @@ error:
  *-----------------------------------------------------------------------------
  */
 
-char *
-DnD_GetLastDirName(const char *str) // IN
+Unicode
+DnD_GetLastDirName(ConstUnicode str) // IN
 {
    size_t end = strlen(str);
    size_t start;
@@ -671,7 +779,8 @@ DnD_TransportBufGetPacket(DnDTransportBuffer *buf,           // IN/OUT
       payloadSize = buf->totalSize - buf->offset;
    }
 
-   *packet = Util_SafeMalloc(payloadSize + DND_TRANSPORT_PACKET_HEADER_SIZE);
+   *packet = (DnDTransportPacketHeader *)Util_SafeMalloc(
+      payloadSize + DND_TRANSPORT_PACKET_HEADER_SIZE);
    (*packet)->type = DND_TRANSPORT_PACKET_TYPE_PAYLOAD;
    (*packet)->seqNum = buf->seqNum;
    (*packet)->totalSize = buf->totalSize;
@@ -684,7 +793,7 @@ DnD_TransportBufGetPacket(DnDTransportBuffer *buf,           // IN/OUT
    buf->offset += payloadSize;
 
    /* This time is used for timeout purpose. */
-   buf->lastUpdateTime = Hostinfo_SystemTimerUS();
+   Hostinfo_GetTimeOfDay(&buf->lastUpdateTime);
 
    return payloadSize + DND_TRANSPORT_PACKET_HEADER_SIZE;
 }
@@ -795,7 +904,7 @@ DnD_TransportMsgToPacket(uint8 *msg,                        // IN
 
    packetSize = msgSize + DND_TRANSPORT_PACKET_HEADER_SIZE;
 
-   *packet = Util_SafeMalloc(packetSize);
+   *packet = (DnDTransportPacketHeader *)Util_SafeMalloc(packetSize);
 
    (*packet)->type = DND_TRANSPORT_PACKET_TYPE_SINGLE;
    (*packet)->seqNum = seqNum;
@@ -831,7 +940,8 @@ size_t
 DnD_TransportReqPacket(DnDTransportBuffer *buf,           // IN
                        DnDTransportPacketHeader **packet) // OUT
 {
-   *packet = Util_SafeMalloc(DND_TRANSPORT_PACKET_HEADER_SIZE);
+   *packet = (DnDTransportPacketHeader *)Util_SafeMalloc(
+      DND_TRANSPORT_PACKET_HEADER_SIZE);
 
    (*packet)->type = DND_TRANSPORT_PACKET_TYPE_REQUEST;
    (*packet)->seqNum = buf->seqNum;
