@@ -61,15 +61,12 @@
 #include "vmware/tools/vmbackup.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
 #endif
 
-MY_ASSERTS(GI_SIZE_ASSERTS,
-           ASSERT_ON_COMPILE(GUESTINFO_MAX_VALUE_SIZE >= MAX_VALUE_LEN);
-)
-
 /**
  * Default poll interval is 30s (in milliseconds).
  */
@@ -81,9 +78,9 @@ MY_ASSERTS(GI_SIZE_ASSERTS,
  * Stores information about all guest information sent to the vmx.
  */
 
-typedef struct _GuestInfoCache{
+typedef struct _GuestInfoCache {
    /* Stores values of all key-value pairs. */
-   char value[INFO_MAX][GUESTINFO_MAX_VALUE_SIZE];
+   char          *value[INFO_MAX];
    NicInfoV3     *nicInfo;
    GuestDiskInfo *diskInfo;
 } GuestInfoCache;
@@ -232,9 +229,9 @@ GuestInfoVMSupport(RpcInData *data)
 static gboolean
 GuestInfoGather(gpointer data)
 {
-   char name[GUESTINFO_MAX_VALUE_SIZE];
-   char osNameFull[GUESTINFO_MAX_VALUE_SIZE];
-   char osName[GUESTINFO_MAX_VALUE_SIZE];
+   char name[256];  // Size is derived from the SUS2 specification
+                    // "Host names are limited to 255 bytes"
+   char *osString = NULL;
    gboolean disableQueryDiskInfo;
    NicInfoV3 *nicInfo = NULL;
    GuestDiskInfo *diskInfo = NULL;
@@ -257,17 +254,25 @@ GuestInfoGather(gpointer data)
    }
 
    /* Gather all the relevant guest information. */
-   if (!Hostinfo_GetOSName(sizeof osNameFull, sizeof osName, osNameFull,
-                           osName)) {
+   osString = Hostinfo_GetOSName();
+   if (osString == NULL) {
       g_warning("Failed to get OS info.\n");
    } else {
-      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME_FULL, osNameFull)) {
-         g_warning("Failed to update VMDB\n");
-      }
-      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME, osName)) {
+      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME_FULL, osString)) {
          g_warning("Failed to update VMDB\n");
       }
    }
+   free(osString);
+
+   osString = Hostinfo_GetOSGuestString();
+   if (osString == NULL) {
+      g_warning("Failed to get OS info.\n");
+   } else {
+      if (!GuestInfoUpdateVmdb(ctx, INFO_OS_NAME, osString)) {
+         g_warning("Failed to update VMDB\n");
+      }
+   }
+   free(osString);
 
    disableQueryDiskInfo =
       g_key_file_get_boolean(ctx->config, CONFGROUPNAME_GUESTINFO,
@@ -295,7 +300,13 @@ GuestInfoGather(gpointer data)
    /* Get NIC information. */
    if (!GuestInfo_GetNicInfo(&nicInfo)) {
       g_warning("Failed to get nic info.\n");
-   } else if (GuestInfo_IsEqual_NicInfoV3(nicInfo, gInfoCache.nicInfo)) {
+      /*
+       * Return an empty nic info.
+       */
+      nicInfo = Util_SafeCalloc(1, sizeof (struct NicInfoV3));
+   }
+
+   if (GuestInfo_IsEqual_NicInfoV3(nicInfo, gInfoCache.nicInfo)) {
       g_debug("Nic info not changed.\n");
       GuestInfo_FreeNicInfo(nicInfo);
    } else if (GuestInfoUpdateVmdb(ctx, INFO_IPADDRESS, nicInfo)) {
@@ -451,7 +462,8 @@ GuestInfoUpdateVmdb(ToolsAppCtx *ctx,       // IN: Application context
        * Above fall-through is intentional.
        */
 
-      if (strcmp(gInfoCache.value[infoType], (char *)info) == 0) {
+      if (gInfoCache.value[infoType] != NULL &&
+          strcmp(gInfoCache.value[infoType], (char *)info) == 0) {
          /* The value has not changed */
          g_debug("Value unchanged for infotype %d.\n", infoType);
          break;
@@ -463,9 +475,8 @@ GuestInfoUpdateVmdb(ToolsAppCtx *ctx,       // IN: Application context
       }
 
       /* Update the value in the cache as well. */
-      Str_Strcpy(gInfoCache.value[infoType],
-                 (char *)info,
-                 sizeof gInfoCache.value[infoType]);
+      free(gInfoCache.value[infoType]);
+      gInfoCache.value[infoType] = Util_SafeStrdup((char *) info);
       break;
 
    case INFO_IPADDRESS:
@@ -914,7 +925,8 @@ GuestInfoClearCache(void)
    int i;
 
    for (i = 0; i < INFO_MAX; i++) {
-      gInfoCache.value[i][0] = 0;
+      free(gInfoCache.value[i]);
+      gInfoCache.value[i] = NULL;
    }
 
    GuestInfo_FreeDiskInfo(gInfoCache.diskInfo);
@@ -1229,6 +1241,7 @@ GuestInfoServerSetOption(gpointer src,
 {
    char *ip;
    Bool ret = FALSE;
+   gchar *msg;
 
    if (strcmp(option, TOOLSOPTION_BROADCASTIP) != 0) {
       goto exit;
@@ -1244,15 +1257,18 @@ GuestInfoServerSetOption(gpointer src,
    }
 
    ip = NetUtil_GetPrimaryIP();
-
-   if (ip != NULL) {
-      gchar *msg;
-      msg = g_strdup_printf("info-set guestinfo.ip %s", ip);
-      ret = RpcChannel_Send(ctx->rpc, msg, strlen(msg) + 1, NULL, NULL);
-      vm_free(ip);
-      g_free(msg);
+   if (ip == NULL) {
+      /*
+       * If there is any error, then return a blank string.
+       */
+      ip = Util_SafeStrdup("");
    }
 
+   msg = g_strdup_printf("info-set guestinfo.ip %s", ip);
+   ret = RpcChannel_Send(ctx->rpc, msg, strlen(msg) + 1, NULL, NULL);
+   vm_free(ip);
+   g_free(msg);
+
 exit:
    return (gboolean) ret;
 }
