@@ -40,6 +40,7 @@
 #include "vmware/tools/utils.h"
 
 
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -533,6 +534,7 @@ ToolsOnLoad(ToolsAppCtx *ctx)
       { "DisplayTopology_Set",          &ResolutionDisplayTopologySetCB },
 #if defined(RESOLUTION_WIN32)
       { "DisplayTopologyModes_Set",     &ResolutionDisplayTopologyModesSetCB },
+      { "ChangeHost3DAvailabilityHint", &ResolutionChangeHost3DAvailabilityHintCB },
 #endif
    };
 
