@@ -33,6 +33,7 @@
 #include "vmware/tools/utils.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -77,8 +78,6 @@ ToolsOnLoad(ToolsAppCtx *ctx)
          FoundryToolsDaemonRunProgram, NULL, NULL, NULL, 0 },
       { VIX_BACKDOORCOMMAND_GET_PROPERTIES,
          FoundryToolsDaemonGetToolsProperties, NULL, NULL, 0 },
-      { VIX_BACKDOORCOMMAND_CHECK_USER_ACCOUNT,
-         ToolsDaemonTcloCheckUserAccount, NULL, NULL, NULL, 0 },
       { VIX_BACKDOORCOMMAND_SEND_HGFS_PACKET,
          ToolsDaemonHgfsImpersonated, NULL, NULL, NULL, 0 },
       { VIX_BACKDOORCOMMAND_COMMAND,
