@@ -36,6 +36,8 @@
 #include "system.h"
 #include "util.h"
 #include "vmcheck.h"
+#include "vm_tools_version.h"
+#include "vm_version.h"
 #include "vmware/guestrpc/tclodefs.h"
 #include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
@@ -382,12 +384,25 @@ ToolsCore_Setup(ToolsServiceState *state)
 
    ToolsCore_ReloadConfig(state, FALSE);
 
+   /*
+    * Useful for debugging purposes. Log the vesion and build information.
+    */
+   g_message("Tools Version: %s (%s)\n", TOOLS_VERSION_EXT_CURRENT_STR, BUILD_NUMBER);
+
    /* Initializes the app context. */
    gctx = g_main_context_default();
    state->ctx.version = TOOLS_CORE_API_V1;
    state->ctx.name = state->name;
    state->ctx.errorCode = EXIT_SUCCESS;
+#if defined(__APPLE__)
+   /*
+    * Mac OS doesn't use g_main_loop_run(), so need to create the loop as
+    * "running".
+    */
+   state->ctx.mainLoop = g_main_loop_new(gctx, TRUE);
+#else
    state->ctx.mainLoop = g_main_loop_new(gctx, FALSE);
+#endif
    state->ctx.isVMware = VmCheck_IsVirtualWorld();
    g_main_context_unref(gctx);
 
