@@ -35,10 +35,12 @@
 #include "system.h"
 #include "unicode.h"
 #include "util.h"
+#include "vmware/tools/log.h"
 #include "vmware/tools/i18n.h"
 #include "vmware/tools/utils.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -186,7 +188,7 @@ main(int argc,
 
       if (!Hostinfo_Daemonize(argv[0],
                               argv,
-                              HOSTINFO_DAEMONIZE_DEFAULT,
+                              HOSTINFO_DAEMONIZE_LOCKPID,
                               gState.pidFile, NULL, 0)) {
          goto exit;
       }
