@@ -31,6 +31,7 @@
 #include "util.h"
 #include "vmrpcdbgInt.h"
 
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -111,7 +112,7 @@ RpcDebugRun(ToolsAppCtx *ctx,
    ASSERT(suite != NULL);
 
    test = CU_add_test(suite, g_module_name(gPlugin), RpcDebugRunLoop);
-   ASSERT(test != NULL);
+   ASSERT_NOT_IMPLEMENTED(test != NULL);
 
    gLibRunData.ctx = ctx;
    gLibRunData.libData = ldata;
