@@ -33,10 +33,15 @@
 #include "toolboxcmd_version.h"
 #include "system.h"
 #include "unicode.h"
+#include "vmcheck.h"
 #include "vmware/tools/guestrpc.h"
 #include "vmware/tools/i18n.h"
+#include "vmware/tools/log.h"
 #include "vmware/tools/utils.h"
+#include "vm_version.h"
+#include "vm_product_versions.h"
 
+#include "vm_version.h"
 #include "embed_version.h"
 VM_EMBED_VERSION(TOOLBOXCMD_VERSION_STRING);
 
@@ -292,9 +297,7 @@ ToolboxCmdHelp(const char *progName,   // IN
                           "   script\n"
                           "   stat\n"
                           "   timesync\n"
-                          "   upgrade (not available on all operating systems)\n"
-                          "\n"
-                          "For additional information please visit http://www.vmware.com/support/\n\n"),
+                          "   upgrade (not available on all operating systems)\n"),
            progName, progName, cmd, progName);
 }
 
@@ -405,8 +408,9 @@ main(int argc,    // IN: length of command line arguments
    Bool show_help = FALSE;
    Bool show_version = FALSE;
    CmdTable *cmd = NULL;
+   GKeyFile *conf = NULL;
    int c;
-   int retval;
+   int retval = EXIT_FAILURE;
 
 #if defined(_WIN32)
    char **argv;
@@ -416,7 +420,8 @@ main(int argc,    // IN: length of command line arguments
 #endif
 
    setlocale(LC_ALL, "");
-   VMTools_ConfigLogging("toolboxcmd", NULL, FALSE, FALSE);
+   VMTools_LoadConfig(NULL, G_KEY_FILE_NONE, &conf, NULL);
+   VMTools_ConfigLogging("toolboxcmd", conf, FALSE, FALSE);
    VMTools_BindTextDomain(VMW_TEXT_DOMAIN, NULL, NULL);
 
    /*
@@ -425,7 +430,7 @@ main(int argc,    // IN: length of command line arguments
    if (!VmCheck_IsVirtualWorld()) {
       g_printerr(SU_(error.novirtual, "%s must be run inside a virtual machine.\n"),
                  argv[0]);
-      exit(EXIT_FAILURE);
+      goto exit;
    }
 
    /*
@@ -458,17 +463,19 @@ main(int argc,    // IN: length of command line arguments
          /* getopt_long already printed an error message. */
          g_printerr(SU_(help.hint, "Try '%s %s%s%s' for more information.\n"),
                     argv[0], "-h", "", "");
-         return EXIT_FAILURE;
+         goto exit;
 
       default:
-         return EXIT_FAILURE;
+         goto exit;
       }
    }
 
    if (show_version) {
       g_print("%s (%s)\n", TOOLBOXCMD_VERSION_STRING, BUILD_NUMBER);
+      retval = EXIT_SUCCESS;
    } else if (show_help) {
       ToolboxCmdHelp(argv[0], "help");
+      retval = EXIT_SUCCESS;
    } else {
       /* Process any remaining command line arguments (not options), and
        * execute corresponding command
@@ -503,9 +510,12 @@ main(int argc,    // IN: length of command line arguments
          g_printerr(SU_(help.hint, "Try '%s %s%s%s' for more information.\n"),
                     argv[0], "help", cmd ? " " : "", cmd ? cmd->command : "");
       }
-
-      return retval;
    }
 
-   return EXIT_SUCCESS;
+exit:
+   if (conf != NULL) {
+      g_key_file_free(conf);
+   }
+
+   return retval;
 }
